
// ForkConformanceTest 


// user_includes.hpp is generated every test run and contains all header files
// specified in the configuration file for the segment under test
#include "user_includes.hpp"

// Used to catch assertions at compile time  
// Static assert magic - true exists but false does not (and should not!)
template<bool> struct FaceStaticAssert;
template<> struct FaceStaticAssert<true> {};


int main()
{
    // Using all non type specific function calls to test for fork use 


    // ----- Initialize Fork Test -----
    {

        // Function Signature Conformance Test
        // Setting function ptr with correct signature to function under test
        void (*FUNCTION_TEST_PTR) (
            /* in     */ const FACE::CONFIGURATION_RESOURCE configuration,
            /*    out */ FACE::RETURN_CODE_TYPE &return_code
            ) = FACE::TypeAbstractionTS::Initialize;


    }


    // ----- Create_Connection Fork Test -----
    {

        // Function Signature Conformance Test
        // Setting function ptr with correct signature to function under test
        void (*FUNCTION_TEST_PTR) (
            /* in     */ const FACE::CONNECTION_NAME_TYPE connection_name,
            /* in     */ FACE::MESSAGING_PATTERN_TYPE pattern,
            /*    out */ FACE::CONNECTION_ID_TYPE &connection_id,
            /*    out */ FACE::CONNECTION_DIRECTION_TYPE &connection_direction,
            /*    out */ FACE::MESSAGE_SIZE_TYPE &max_message_size,
            /* in     */ FACE::TIMEOUT_TYPE timeout,
            /*    out */ FACE::RETURN_CODE_TYPE &return_code
            ) = FACE::TypeAbstractionTS::Create_Connection;


    }


    // ----- Destroy_Connection Fork Test -----
    {

        // Function Signature Conformance Test
        // Setting function ptr with correct signature to function under test
        void (*FUNCTION_TEST_PTR) (
            /* in     */ FACE::CONNECTION_ID_TYPE connection_id,
            /*    out */ FACE::RETURN_CODE_TYPE &return_code
            ) = FACE::TypeAbstractionTS::Destroy_Connection;


    }


    // ----- Receive_Message Fork Test -----
    {

        // Function Signature Conformance Test
        // Setting function ptr with correct signature to function under test
        void (*FUNCTION_TEST_PTR) (
            /* in     */ FACE::CONNECTION_ID_TYPE connection_id,
            /* in     */ FACE::TIMEOUT_TYPE timeout,
            /* in out */ FACE::TRANSACTION_ID_TYPE &transaction_id,
            /* in out */ void  *message,
            /* in out */ FACE::MESSAGE_TYPE_GUID &message_type_id,
            /* in out */ FACE::MESSAGE_SIZE_TYPE &message_size,
            /*    out */ FACE::RETURN_CODE_TYPE &return_code
            ) = FACE::TypeAbstractionTS::Receive_Message;


    }


    // ----- Send_Message Fork Test -----
    {

        // Function Signature Conformance Test
        // Setting function ptr with correct signature to function under test
        void (*FUNCTION_TEST_PTR) (
            /* in     */ FACE::CONNECTION_ID_TYPE connection_id,
            /* in     */ FACE::TIMEOUT_TYPE timeout,
            /* in out */ FACE::TRANSACTION_ID_TYPE &transaction_id,
            /* in out */ void  *message,
            /* in     */ FACE::MESSAGE_TYPE_GUID message_type_id,
            /* in     */ FACE::MESSAGE_SIZE_TYPE message_size,
            /*    out */ FACE::RETURN_CODE_TYPE &return_code
            ) = FACE::TypeAbstractionTS::Send_Message;


    }


    // ----- Register_Callback Fork Test -----
    {

        typedef void (*CONFORMANCE_TEST_send_event_ptr) (
                /* in     */ FACE::TRANSACTION_ID_TYPE transaction_id,
                /* in out */ void  *message,
                /* in     */ FACE::MESSAGE_TYPE_GUID message_type_id,
                /* in     */ FACE::MESSAGE_SIZE_TYPE message_size,
                /* in     */ const FACE::WAITSET_TYPE waitset,
                /*    out */ FACE::RETURN_CODE_TYPE &return_code
            );

        // Function Signature Conformance Test
        // Setting function ptr with correct signature to function under test
        void (*FUNCTION_TEST_PTR) (
            /* in     */ FACE::CONNECTION_ID_TYPE connection_id,
            /* in     */ const FACE::WAITSET_TYPE waitset,
            /* in out */ CONFORMANCE_TEST_send_event_ptr data_callback,
            /* in     */ FACE::MESSAGE_SIZE_TYPE max_message_size,
            /*    out */ FACE::RETURN_CODE_TYPE &return_code
            ) = FACE::TypeAbstractionTS::Register_Callback;


    }


    // ----- Unregister_Callback Fork Test -----
    {

        // Function Signature Conformance Test
        // Setting function ptr with correct signature to function under test
        void (*FUNCTION_TEST_PTR) (
            /* in     */ FACE::CONNECTION_ID_TYPE connection_id,
            /*    out */ FACE::RETURN_CODE_TYPE &return_code
            ) = FACE::TypeAbstractionTS::Unregister_Callback;


    }


    // ----- Get_Connection_Parameters Fork Test -----
    {

        // Function Signature Conformance Test
        // Setting function ptr with correct signature to function under test
        void (*FUNCTION_TEST_PTR) (
            /* in out */ FACE::CONNECTION_NAME_TYPE &connection_name,
            /* in out */ FACE::CONNECTION_ID_TYPE &connection_id,
            /*    out */ FACE::TRANSPORT_CONNECTION_STATUS_TYPE &connection_status,
            /*    out */ FACE::RETURN_CODE_TYPE &return_code
            ) = FACE::TypeAbstractionTS::Get_Connection_Parameters;


    }

}
