#include <stdint.h>

#include "common_conformance_test.hpp"
#include "user_includes.hpp"

// Static assert magic - true exists but false does not (and should not!)
template<bool> struct FaceStaticAssert;
template<> struct FaceStaticAssert<true> {};

int main(int argc, char *argv[])
{
	// Conformance test found in file Test1.cpp
	{
		// This type is used to represent 64-bit signed integer with a
		// 1 nanosecond resolution.
		// Testing FACE::SYSTEM_TIME_TYPE size is conformant (int64_t)
		{
			FaceStaticAssert< sizeof(FACE::SYSTEM_TIME_TYPE) == sizeof(CONFORMANCE_TEST::FACE::SYSTEM_TIME_TYPE) >();
		}
		// This type is used to represent 64-bit signed integer with a
		// 1 nanosecond resolution.
		// Testing FACE::SYSTEM_TIME_TYPE type is conformant (int64_t)
		FACE::SYSTEM_TIME_TYPE var = 0;
		CONFORMANCE_TEST::FACE::SYSTEM_TIME_TYPE cvar;
		cvar = var;
	}

	// Conformance test found in file Test2.cpp
	{
		// This type is used to represent an infinitely long time value.
		// It is often used to specify that the caller is willing to wait
		// forever for an operation to complete and does not wish to timeout.
		// Testing FACE::INF_TIME_VALUE size is conformant (int64_t)
		{
			FaceStaticAssert< sizeof(FACE::INF_TIME_VALUE) == sizeof(CONFORMANCE_TEST::FACE::INF_TIME_VALUE) >();
		}
		// This type is used to represent an infinitely long time value.
		// It is often used to specify that the caller is willing to wait
		// forever for an operation to complete and does not wish to timeout.
		// Testing constant FACE::INF_TIME_VALUE value (-1) is conformant
		FaceStaticAssert< FACE::INF_TIME_VALUE == CONFORMANCE_TEST::FACE::INF_TIME_VALUE >();
	}

	// Conformance test found in file Test3.cpp
	{
		// This enumeration defines the possible set of status codes which may be
		// returned by a method defined in the FACE API.
		// Testing FACE::RETURN_CODE_TYPE size is conformant (enumeration)
		{
			FaceStaticAssert< sizeof(FACE::RETURN_CODE_TYPE) == sizeof(CONFORMANCE_TEST::FACE::RETURN_CODE_TYPE) >();
		}
		
		// FACE::RETURN_CODE_TYPE enumeration test 
		FACE::RETURN_CODE_TYPE var;
		// request valid and operation performed
		var = FACE::NO_ERROR;
		// status of system unaffected by request
		var = FACE::NO_ACTION;
		// no message was available
		var = FACE::NOT_AVAILABLE;
		// address currently in use
		var = FACE::ADDR_IN_USE;
		// invalid parameter specified in request
		var = FACE::INVALID_PARAM;
		// parameter incompatible with configuration
		var = FACE::INVALID_CONFIG;
		// no permission to send or connecting to wrong partition
		var = FACE::PERMISSION_DENIED;
		// request incompatible with current mode
		var = FACE::INVALID_MODE;
		// the time expired before the request could be filled
		var = FACE::TIMED_OUT;
		// current time - timestamp exceeds configured limits
		var = FACE::MESSAGE_STALE;
		// asynchronous connection in progress
		var = FACE::CONNECTION_IN_PROGRESS;
		// connection was closed
		var = FACE::CONNECTION_CLOSED;
		// Data Buffer was too small for message
		var = FACE::DATA_BUFFER_TOO_SMALL;
	}

	// Conformance test found in file Test4.cpp
	{
		// This type is used to represent a system address.
		// 
		// Note: This is expected to be a pointer type such as (void *) in C.
		// Testing FACE::SYSTEM_ADDRESS_TYPE size is conformant (void *)
		{
			FaceStaticAssert< sizeof(FACE::SYSTEM_ADDRESS_TYPE) == sizeof(CONFORMANCE_TEST::FACE::SYSTEM_ADDRESS_TYPE) >();
		}
		// This type is used to represent a system address.
		// 
		// Note: This is expected to be a pointer type such as (void *) in C.
		// Testing FACE::SYSTEM_ADDRESS_TYPE type is conformant (void *)
		FACE::SYSTEM_ADDRESS_TYPE var = 0;
		CONFORMANCE_TEST::FACE::SYSTEM_ADDRESS_TYPE cvar;
		cvar = var;
	}

	// Conformance test found in file Test5.cpp
	{
		// This type has a one nanosecond resolution.
		// Testing FACE::TIMEOUT_TYPE size is conformant (FACE::SYSTEM_TIME_TYPE)
		{
			FaceStaticAssert< sizeof(FACE::TIMEOUT_TYPE) == sizeof(CONFORMANCE_TEST::FACE::TIMEOUT_TYPE) >();
		}
		// This type has a one nanosecond resolution.
		// Testing FACE::TIMEOUT_TYPE type is conformant (FACE::SYSTEM_TIME_TYPE)
		FACE::TIMEOUT_TYPE var = 0;
		CONFORMANCE_TEST::FACE::TIMEOUT_TYPE cvar;
		cvar = var;
	}

	// Conformance test found in file Test6.cpp
	{
		// This type is used to represent the message range, number of messages,
		// and message type in the I/O and Transport APIs.
		// Testing FACE::MESSAGE_RANGE_TYPE size is conformant (int32_t)
		{
			FaceStaticAssert< sizeof(FACE::MESSAGE_RANGE_TYPE) == sizeof(CONFORMANCE_TEST::FACE::MESSAGE_RANGE_TYPE) >();
		}
		// This type is used to represent the message range, number of messages,
		// and message type in the I/O and Transport APIs.
		// Testing FACE::MESSAGE_RANGE_TYPE type is conformant (int32_t)
		FACE::MESSAGE_RANGE_TYPE var = 0;
		CONFORMANCE_TEST::FACE::MESSAGE_RANGE_TYPE cvar;
		cvar = var;
	}

	// Conformance test found in file Test7.cpp
	{
		// Testing FACE::RELIABILITY_TYPE size is conformant (enumeration)
		{
			FaceStaticAssert< sizeof(FACE::RELIABILITY_TYPE) == sizeof(CONFORMANCE_TEST::FACE::RELIABILITY_TYPE) >();
		}
		
		// FACE::RELIABILITY_TYPE enumeration test 
		FACE::RELIABILITY_TYPE var;
		var = FACE::RELIABLE;
		var = FACE::NON_RELIABLE;
	}

	// Conformance test found in file Test8.cpp
	{
		// Testing FACE::READ_WRITE_BEHAVIOR_TYPE size is conformant (enumeration)
		{
			FaceStaticAssert< sizeof(FACE::READ_WRITE_BEHAVIOR_TYPE) == sizeof(CONFORMANCE_TEST::FACE::READ_WRITE_BEHAVIOR_TYPE) >();
		}
		
		// FACE::READ_WRITE_BEHAVIOR_TYPE enumeration test 
		FACE::READ_WRITE_BEHAVIOR_TYPE var;
		var = FACE::QUEUING;
		var = FACE::SAMPLING;
	}
	return 0;
}
