#ifndef CONFORMANCE_TEST_FACE_TS_H
#define CONFORMANCE_TEST_FACE_TS_H

/* Constants from FACE_common and FACE_TS_common that are needed for conformance testing */


#ifdef __cplusplus
extern "C" {
#endif /* __cplusplus */

#include <stdint.h>

/*This type is used to represent 64-bit signed integer with a */
/*1 nanosecond resolution. */
typedef int64_t CONFORMANCE_TEST_FACE_SYSTEM_TIME_TYPE;

/*This type is used to represent an infinitely long time value. */
/*It is often used to specify that the caller is willing to wait */
/*forever for an operation to complete and does not wish to timeout. */
#define CONFORMANCE_TEST_FACE_INF_TIME_VALUE ((CONFORMANCE_TEST_FACE_SYSTEM_TIME_TYPE) -1)

typedef char CONFORMANCE_TEST_FACE_CONFIGURATION_RESOURCE[256];

/*This enumeration defines the possible set of status codes which may be */
/*returned by a method defined in the FACE API. */
typedef enum {
	/*request valid and operation performed */
	CONFORMANCE_TEST_FACE_NO_ERROR, 
	/*status of system unaffected by request */
	CONFORMANCE_TEST_FACE_NO_ACTION, 
	/*no message was available */
	CONFORMANCE_TEST_FACE_NOT_AVAILABLE, 
	/*address currently in use */
	CONFORMANCE_TEST_FACE_ADDR_IN_USE, 
	/*invalid parameter specified in request */
	CONFORMANCE_TEST_FACE_INVALID_PARAM, 
	/*parameter incompatible with configuration */
	CONFORMANCE_TEST_FACE_INVALID_CONFIG, 
	/*no permission to send or connecting to wrong partition */
	CONFORMANCE_TEST_FACE_PERMISSION_DENIED, 
	/*request incompatible with current mode */
	CONFORMANCE_TEST_FACE_INVALID_MODE, 
	/*the time expired before the request could be filled */
	CONFORMANCE_TEST_FACE_TIMED_OUT, 
	/*current time - timestamp exceeds configured limits */
	CONFORMANCE_TEST_FACE_MESSAGE_STALE, 
	/*asynchronous connection in progress */
	CONFORMANCE_TEST_FACE_CONNECTION_IN_PROGRESS, 
	/*connection was closed */
	CONFORMANCE_TEST_FACE_CONNECTION_CLOSED, 
	/*Data Buffer was too small for message */
	CONFORMANCE_TEST_FACE_DATA_BUFFER_TOO_SMALL 
} CONFORMANCE_TEST_FACE_RETURN_CODE_TYPE;

/*This type is used to represent a system address. */
/* */
/*Note: This is expected to be a pointer type such as (void *) in C. */
typedef void * CONFORMANCE_TEST_FACE_SYSTEM_ADDRESS_TYPE;

/*This type has a one nanosecond resolution. */
typedef CONFORMANCE_TEST_FACE_SYSTEM_TIME_TYPE CONFORMANCE_TEST_FACE_TIMEOUT_TYPE;

/*This type is used to represent the message range, number of messages, */
/*and message type in the I/O and Transport APIs. */
typedef int32_t CONFORMANCE_TEST_FACE_MESSAGE_RANGE_TYPE;

typedef enum {
	CONFORMANCE_TEST_FACE_RELIABLE, 
	CONFORMANCE_TEST_FACE_NON_RELIABLE 
} CONFORMANCE_TEST_FACE_RELIABILITY_TYPE;

typedef enum {
	CONFORMANCE_TEST_FACE_QUEUING, 
	CONFORMANCE_TEST_FACE_SAMPLING 
} CONFORMANCE_TEST_FACE_READ_WRITE_BEHAVIOR_TYPE;





typedef char CONFORMANCE_TEST_FACE_CONNECTION_NAME_TYPE [64];
typedef int32_t CONFORMANCE_TEST_FACE_MESSAGE_SIZE_TYPE;
typedef int64_t CONFORMANCE_TEST_FACE_MESSAGE_TYPE_GUID;
	
// Defined by the Transport Services configuration file
// For the TWO_WAY direction types, if a single connection
// is being used for both the request and the reply,
// SOURCE is to be used at the endpoint where the request
// originates and DESTINATION is to be used at the endpoint
// where the reply originates.

typedef enum  {
		CONFORMANCE_TEST_FACE_SOURCE,             // maps to O_WRONLY (ARINC 653 and POSIX)
		CONFORMANCE_TEST_FACE_DESTINATION,        // maps to O_RDONLY (ARINC 653 and POSIX)
		CONFORMANCE_TEST_FACE_BI_DIRECTIONAL,     // maps to O_RDWR (POSIX ONLY)
		CONFORMANCE_TEST_FACE_ONE_WAY_REQUEST_SOURCE,
		CONFORMANCE_TEST_FACE_ONE_WAY_REQUEST_DESTINATION,
		CONFORMANCE_TEST_FACE_TWO_WAY_REQUEST_SYNCHRONOUS_SOURCE,
		CONFORMANCE_TEST_FACE_TWO_WAY_REQUEST_SYNCHRONOUS_DESTINATION,
		CONFORMANCE_TEST_FACE_TWO_WAY_REQUEST_REPLY_ASYNCHRONOUS_SOURCE,
		CONFORMANCE_TEST_FACE_TWO_WAY_REQUEST_REPLY_ASYNCHRONOUS_DESTINATION,
		CONFORMANCE_TEST_FACE_NOT_DEFINED_CONNECTION_DIRECTION_TYPE
	} CONFORMANCE_TEST_FACE_CONNECTION_DIRECTION_TYPE;

typedef int64_t CONFORMANCE_TEST_FACE_CONNECTION_ID_TYPE;
typedef int64_t CONFORMANCE_TEST_FACE_TRANSACTION_ID_TYPE; // used to tie together request/response messages
    
// Defined by the Transport Services configuration file
typedef enum {
		CONFORMANCE_TEST_FACE_SAMPLING_PORT,   // ARINC 653 sampling port
		CONFORMANCE_TEST_FACE_QUEUING_PORT,    // ARINC 653 queuing port
		CONFORMANCE_TEST_FACE_SOCKET,          // POSIX socket
        // POSIX message queue is between POSIX processes belonging to the same 
        // partition
		CONFORMANCE_TEST_FACE_MQ,              // POSIX queue
        // POSIX shared memory is between POSIX processes belonging to the same 
        // partition
		CONFORMANCE_TEST_FACE_SHM,             // POSIX shared memory
		CONFORMANCE_TEST_FACE_CORBA,           // CORBA              // Data Distribution Services
		CONFORMANCE_TEST_FACE_DDS
	} CONFORMANCE_TEST_FACE_CONNECTION_TYPE;

typedef _Bool CONFORMANCE_TEST_FACE_WAITSET_TYPE [32];

typedef enum {
		CONFORMANCE_TEST_FACE_FIFO,
		CONFORMANCE_TEST_FACE_PRIORITY,
		CONFORMANCE_TEST_FACE_NOT_DEFINED_QUEUING_DISCIPLINE_TYPE
	} CONFORMANCE_TEST_FACE_QUEUING_DISCIPLINE_TYPE;

// Defined by the Transport Services configuration file
typedef enum {
		CONFORMANCE_TEST_FACE_UNIX,     // AF_UNIX
		CONFORMANCE_TEST_FACE_INET,     // AF_INET
		CONFORMANCE_TEST_FACE_NOT_DEFINED_CONNECTION_DOMAIN_TYPE
	} CONFORMANCE_TEST_FACE_CONNECTION_DOMAIN_TYPE;

// Defined by the Transport Services configuration file
typedef enum  {
		CONFORMANCE_TEST_FACE_STREAM,      // SOCK_STREAM
		CONFORMANCE_TEST_FACE_DGRAM,       // SOCK_DGRAM
		CONFORMANCE_TEST_FACE_SEQPACKET,   // SOCK_SEQPACKET
		CONFORMANCE_TEST_FACE_NOT_DEFINED_SOCKET_TYPE
	} CONFORMANCE_TEST_FACE_SOCKET_TYPE;

// Defined by the Transport Services configuration file
typedef enum {
		CONFORMANCE_TEST_FACE_PEEK,                   // SOCK_STREAM
		CONFORMANCE_TEST_FACE_OOB_RECEIVE_FLAG_TYPE,  // SOCK_DGRAM
		CONFORMANCE_TEST_FACE_WAITALL,                // SOCK_SEQPACKET
		CONFORMANCE_TEST_FACE_NOT_DEFINED_RECEIVE_FLAG_TYPE
	} CONFORMANCE_TEST_FACE_RECEIVE_FLAG_TYPE;

// Defined by the Transport Services configuration file
typedef enum {
		CONFORMANCE_TEST_FACE_EOR,                 // SOCK_STREAM
		CONFORMANCE_TEST_FACE_OOB_SEND_FLAG_TYPE,  // SOCK_DGRAM
		CONFORMANCE_TEST_FACE_NOSIGNAL,            // SOCK_SEQPACKET
		CONFORMANCE_TEST_FACE_NOT_DEFINED_SEND_FLAG_TYPE
	} CONFORMANCE_TEST_FACE_SEND_FLAG_TYPE;

typedef enum {
		CONFORMANCE_TEST_FACE_INVALID,     // Also 'empty' and 'down'
		CONFORMANCE_TEST_FACE_VALID		 // Also 'occupied' and 'UP'
	} CONFORMANCE_TEST_FACE_VALIDITY_TYPE;

    // Addressed in Validity_Type
	// typedef enum { EMPTY = 0, OCCUPIED = 1} EMPTY_INDICATOR_TYPE;
    // typedef enum { DOWN = 0, UP = 1} EVENT_STATE_TYPE;

    // Support publish/subscribe and request/response
typedef enum {
		CONFORMANCE_TEST_FACE_PUB_SUB,
		CONFORMANCE_TEST_FACE_CLIENT,
		CONFORMANCE_TEST_FACE_SERVER
	} CONFORMANCE_TEST_FACE_MESSAGING_PATTERN_TYPE;

typedef int32_t CONFORMANCE_TEST_FACE_WAITING_RANGE_TYPE;

    // Elements defined by the Transport Services configuration file
typedef struct {
		CONFORMANCE_TEST_FACE_MESSAGE_RANGE_TYPE MESSAGE;
		CONFORMANCE_TEST_FACE_MESSAGE_RANGE_TYPE MAX_MESSAGE;
		CONFORMANCE_TEST_FACE_MESSAGE_SIZE_TYPE MAX_MESSAGE_SIZE;
		CONFORMANCE_TEST_FACE_CONNECTION_DIRECTION_TYPE CONNECTION_DIRECTION;
		CONFORMANCE_TEST_FACE_WAITING_RANGE_TYPE WAITING_PROCESSES_OR_MESSAGES;
		CONFORMANCE_TEST_FACE_SYSTEM_TIME_TYPE REFRESH_PERIOD; // Maximum age of a valid message
		CONFORMANCE_TEST_FACE_VALIDITY_TYPE LAST_MSG_VALIDITY;
	} CONFORMANCE_TEST_FACE_TRANSPORT_CONNECTION_STATUS_TYPE;

typedef void (*CONFORMANCE_TEST_send_event_ptr)
    ( /* in */    CONFORMANCE_TEST_FACE_TRANSACTION_ID_TYPE transaction_id,
      /* inout */ void *              message,
      /* in */    CONFORMANCE_TEST_FACE_MESSAGE_TYPE_GUID   message_type_id,
      /* in */    CONFORMANCE_TEST_FACE_MESSAGE_SIZE_TYPE   message_size,
      /* in */    CONFORMANCE_TEST_FACE_WAITSET_TYPE  waitset,
      /* out */   CONFORMANCE_TEST_FACE_RETURN_CODE_TYPE    *return_code);


#ifdef __cplusplus
}
#endif /* __cplusplus */


#endif // CONFORMANCE_TEST_FACE_TS_COMMON_HPP
