#ifndef CONFORMANCEINTERFACETESTS_C_COMMON_COMMON_CONFORMANCE_TEST_H
#define CONFORMANCEINTERFACETESTS_C_COMMON_COMMON_CONFORMANCE_TEST_H

#include <stdint.h>


/*This type is used to represent 64-bit signed integer with a */
/*1 nanosecond resolution. */
typedef int64_t CONFORMANCE_TEST_FACE_SYSTEM_TIME_TYPE;

/*This type is used to represent an infinitely long time value. */
/*It is often used to specify that the caller is willing to wait */
/*forever for an operation to complete and does not wish to timeout. */
#define CONFORMANCE_TEST_FACE_INF_TIME_VALUE -1

/*This enumeration defines the possible set of status codes which may be */
/*returned by a method defined in the FACE API. */
typedef enum {
	/*request valid and operation performed */
	CONFORMANCE_TEST_FACE_NO_ERROR, 
	/*status of system unaffected by request */
	CONFORMANCE_TEST_FACE_NO_ACTION, 
	/*no message was available */
	CONFORMANCE_TEST_FACE_NOT_AVAILABLE, 
	/*address currently in use */
	CONFORMANCE_TEST_FACE_ADDR_IN_USE, 
	/*invalid parameter specified in request */
	CONFORMANCE_TEST_FACE_INVALID_PARAM, 
	/*parameter incompatible with configuration */
	CONFORMANCE_TEST_FACE_INVALID_CONFIG, 
	/*no permission to send or connecting to wrong partition */
	CONFORMANCE_TEST_FACE_PERMISSION_DENIED, 
	/*request incompatible with current mode */
	CONFORMANCE_TEST_FACE_INVALID_MODE, 
	/*the time expired before the request could be filled */
	CONFORMANCE_TEST_FACE_TIMED_OUT, 
	/*current time - timestamp exceeds configured limits */
	CONFORMANCE_TEST_FACE_MESSAGE_STALE, 
	/*asynchronous connection in progress */
	CONFORMANCE_TEST_FACE_CONNECTION_IN_PROGRESS, 
	/*connection was closed */
	CONFORMANCE_TEST_FACE_CONNECTION_CLOSED, 
	/*Data Buffer was too small for message */
	CONFORMANCE_TEST_FACE_DATA_BUFFER_TOO_SMALL 
} CONFORMANCE_TEST_FACE_RETURN_CODE_TYPE;

/*This type is used to represent a system address. */
/* */
/*Note: This is expected to be a pointer type such as (void *) in C. */
typedef void * CONFORMANCE_TEST_FACE_SYSTEM_ADDRESS_TYPE;

/*This type has a one nanosecond resolution. */
typedef CONFORMANCE_TEST_FACE_SYSTEM_TIME_TYPE CONFORMANCE_TEST_FACE_TIMEOUT_TYPE;

/*This type is used to represent the message range, number of messages, */
/*and message type in the I/O and Transport APIs. */
typedef int32_t CONFORMANCE_TEST_FACE_MESSAGE_RANGE_TYPE;

typedef enum {
	CONFORMANCE_TEST_FACE_RELIABLE, 
	CONFORMANCE_TEST_FACE_NON_RELIABLE 
} CONFORMANCE_TEST_FACE_RELIABILITY_TYPE;

typedef enum {
	CONFORMANCE_TEST_FACE_QUEUING, 
	CONFORMANCE_TEST_FACE_SAMPLING 
} CONFORMANCE_TEST_FACE_READ_WRITE_BEHAVIOR_TYPE;

#endif /* CONFORMANCEINTERFACETESTS_C_COMMON_COMMON_CONFORMANCE_TEST_H */
