#!/usr/bin/env python
"""
------------------------------------------------------------------------------------
   Copyright (c) Vanderbilt University, 2020                                       
                                                                                   
   ALL RIGHTS RESERVED, UNLESS OTHERWISE STATED                                    
                                                                                   
   This software, authored by Vanderbilt University under a contract awarded to and
   managed by Alion Science and Technology, was funded by the U.S. Government under
   Contract No. FA8075-14-D-0014 and the U.S. Government has unlimited rights in   
   this software.  An unlimited rights license means that the U.S. Government can  
   use, modify, reproduce, release or disclose computer software in whole or in    
   part, in any manner, and for any purpose whatsoever, and to have or authorize   
   others to do so.                                                                
                                                                                   
   Vanderbilt University disclaims all warranties with regard to this software,    
   including all implied warranties of merchantability and fitness. In no event    
   shall Vanderbilt University be liable for any special, indirect or consequential
   damages or any damages whatsoever resulting from loss of use, data or profits,  
   whether in an action of contract, negligence or other tortious action, arising  
   out of or in connection with the use or performance of this software.           
                                                                                   
   May contain products produced under DoD SENSIAC contract HC104705D4000 under the
   sponsorship of the Defense Technical Information Center,                        
   ATTN: DTIC-AI, 8723 John J. Kingman Rd., Ste 0944, Fort Belvoir, VA 22060-6218. 
   SENSIAC is a DoD Information Analysis Center Sponsored by the Defense Technical 
   Information Center.                                                             
                                                                                   
   Conformance Test Suite Version 2.1.1r9 supporting FACE Standard Version 2.1.1   
                                                                                   
   Release Date: 29-Aug-2023                                                       
------------------------------------------------------------------------------------
"""

__author__ = 'david.fann@vanderbilt.edu'

import ConformanceMain

if __name__ == '__main__':
    import logging
    ConformanceMain.initializeLogger()


from tabwidget import TabWidget
import Dialogs
import fancyentry
from fancyentry import FancyEntry
#from fancyentry import FancyEntry, NONE, FILE, FILES, DIRECTORY, DIRECTORIES
from fancyradiogroup import FancyRadioGroup
from commonTools import listToString
import inspect
import commonTools
import constants
import TestConfiguration
import ConformanceTest
import VerificationSuite
import DataModel
import logging
from sys import platform
from collections import namedtuple
from os import pathsep, sep, getcwd, chdir, mkdir
import os.path
import copy
import SplashScreen
import platform

#from os.path import exists, join, dirname, basename
from Tkinter import *
import tkFileDialog
import Dialogs
import tkFont
import pickle

logger = logging.getLogger(__name__)





class ConfigurationGUI(Dialogs.Dialog):
    """
    classdocs
    """
    LINUX_CONFIG_DIALOG_DEFAULT_WIDTH = 850
    LINUX_CONFIG_DIALOG_DEFAULT_HEIGHT = 400
    LINUX_BOX_OFFSET = 5

    WINDOWS_CONFIG_DIALOG_DEFAULT_WIDTH = 850
    WINDOWS_CONFIG_DIALOG_DEFAULT_HEIGHT = 415
    WINDOWS_BOX_OFFSET = 3

    DEFAULT_DIALOG_PROMPT = \
        "If a directory is chosen, all objects in the given directory will be chosen (determined by " + \
        "the object suffix in the build configuration tab) and objects in recursive subdirectories will also be " + \
        "chosen. Alternatively, you may chose the specific objects by choosing each desired object file, " + \
        "separately. All libraries must be chosen specifically (not by choosing their associated directory). " + \
        "Use the 'D' or the 'F' buttons to select directories or files, respectively.  You may choose " + \
        "a combination of directories and files."

    JAVA_DIALOG_PROMPT = \
        "Java class path elements can be either directories or files (jar or zip files).  Use the 'D' or the 'F' " + \
        "buttons to select directories or files, respectively.  You may choose a combination of directories and files."

    def __init__(self, parent, **params):
        """
        Constructor
        """

        self.uopButtons = dict()
        self.uopHeadings = list()
        self.uopCheckButtons = dict()

        self.dialogTitle = params.get('title', constants.FACE_TESTSUITE_NAME)
        self.configurationFilename = params.get('configFilename', None)
        self.config = params.get('config', None)
        self.dialogPrompt = { TestConfiguration.Language.Java: self.JAVA_DIALOG_PROMPT }
        self.segmentObjectWidgets = list()
        self.includeWidgets = list()
        self.dialogWidth = params.get('width', self.LINUX_CONFIG_DIALOG_DEFAULT_WIDTH)
        self.dialogHeight = params.get('height', self.LINUX_CONFIG_DIALOG_DEFAULT_HEIGHT)
        self.box_offset = params.get('boxOffset', self.LINUX_BOX_OFFSET)
        if platform.system() == 'Windows':
            self.dialogWidth = params.get('width', self.WINDOWS_CONFIG_DIALOG_DEFAULT_WIDTH)
            self.dialogHeight = params.get('height', self.WINDOWS_CONFIG_DIALOG_DEFAULT_HEIGHT)
            self.box_offset = params.get('boxOffset', self.WINDOWS_BOX_OFFSET)

        self.x_offset = self.dialogWidth/4
        self.y_offset = self.dialogHeight/4

        if parent is not None:
            self.x_offset += parent.x_offset
            self.y_offset += parent.y_offset

        self.faceSpecifiedTypeCheckBox = IntVar()
        self.untypedCheckBox = IntVar()
        self.faceSpecifiedServerCheckBox = IntVar()
        self.callbackCheckBox = IntVar()
        self.infoWidgetsDefined = False
        self.pcsCheckBox= IntVar()
        self.tssCheckBox= IntVar()
        self.tssRegisterReadCallbackCheckBox = IntVar()
        self.pssCheckBox= IntVar()
        self.iosCheckBox= IntVar()
        self.ossCheckBox= IntVar()
        self.ossRemoteTestCheckBox = IntVar()
        self.UoPFrame = None
        self.dataModel = None
        self.lastProjectDir = None
        self.dataModelLine = None
        self.uopsToTest = dict()
        self.uopVar = dict()
        self.lastProjectDir = None
        self.defaultEntryWidth = 67
        self.defaultLabelWidth = 25
        self.defaultRadioWidth = 35

        # Java Approved Framework Additions - July 2019
        self.javaApprovedFrameworkWidgets = list()

        commonTools.pushParentGUI(self)

        Dialogs.Dialog.__init__(self, parent, self.dialogTitle)

    def apply(self):
        logger.info('This is where apply happened')

    def addFixedSizeFrame(self, parent_frame, **params):
        width = params.get("width", self.dialogWidth)
        height = params.get("height", self.dialogHeight)
        scrollable = params.get("scrollable", False)

        if scrollable:
            scrollable_frame = Dialogs.VerticalScrolledFrame(parent_frame, width=width, height=height)
            scrollable_frame.pack_propagate(0)
            scrollable_frame.pack(side=TOP)
            return scrollable_frame.interior

        else:
            fixed_frame = Frame(parent_frame, width=width, height=height)
            fixed_frame.pack_propagate(0)
            fixed_frame.pack(side=TOP)
            return fixed_frame

    def body(self, master):
        tabs = TabWidget(self)
        tabs.pack(side=TOP, padx=4, pady=4)
        tab_width = 8
        generalTab = tabs.appendTab(text=" General ")
        self.createGeneralTabWidgets(generalTab)

        buildTab = tabs.appendTab(text=" Build ")
        self.createBuildTabWidgets(buildTab)

        pcsTab = tabs.appendTab(text=" PCS ")
        self.createPcsTabWidgets(pcsTab)

        tssTab = tabs.appendTab(text=" TSS ")
        self.createTssTabWidgets(tssTab)

        pssTab = tabs.appendTab(text=" PSS ")
        self.createPssTabWidgets(pssTab)

        datatypesTab = tabs.appendTab(text=" Data Model ")
        self.createDatatypesTabWidgets(datatypesTab)

        iosTab =tabs.appendTab(text=" IOS ")
        self.createIosTabWidgets(iosTab)

        ossTab = tabs.appendTab(text=" OSS ")
        self.createOssTabWidgets(ossTab)

        # No longer using the verification tab...using our own automated tests for verifying test suite
        #verifyTab = tabs.appendTab(text="Verification")
        #self.createVerifyTabWidgets(verifyTab)

        notesTab = tabs.appendTab(text=" Notes ")
        self.createNotesTabWidgets(notesTab)

        self.tabs = tabs
        self.createConfigIoWidgets()

        if self.config is None:
            if self.configurationFilename is not None:
                self._loadConfiguration(self.configurationFilename)
        else:
            self._configurationToGui(self.config)

        self.infoWidgetsDefined = True
        self.enableInfoWidgets()

    def createGeneralTabWidgets(self, tab):
        tab = self.addFixedSizeFrame(tab)
        entryWidth = self.defaultEntryWidth
        labelWidth = self.defaultLabelWidth
        radioWidth = self.defaultRadioWidth
        pack_options = { "side": TOP }
        line_options = { "width": self.dialogWidth - 100, "spacerHeight": 5}

        Dialogs.insertHorizontalLine(tab, **line_options)
        label = Label(tab, text="General information about the project undergoing conformance testing")
        label.pack(side=TOP)
        Dialogs.insertHorizontalLine(tab, **line_options)

        self.projectDirLine = FancyEntry(tab, text="Project Base Directory:", selectionType=fancyentry.DIRECTORY,
                                validatecommand=self.updateProjectDirectory,
                                title="Select the project base directory",
                                labelWidth=labelWidth, entryWidth=entryWidth)
        self.projectDirLine.pack(pack_options)
        Dialogs.insertHorizontalLine(tab, **line_options)

        self.languageLine = FancyRadioGroup(tab, text="Programming Language:", labelWidth=labelWidth, radioWidth=radioWidth)
        self.languageLine.add(text="C",   value=TestConfiguration.Language.C,  selected=True, command=self.enableInfoWidgets)
        self.languageLine.add(text="C++", value=TestConfiguration.Language.CPP, command=self.enableInfoWidgets)
        self.languageLine.add(text="Ada", value=TestConfiguration.Language.Ada, command=self.enableInfoWidgets)
        self.languageLine.add(text="Java", value=TestConfiguration.Language.Java, command=self.enableInfoWidgets)
        self.languageLine.pack(pack_options)
        Dialogs.insertHorizontalLine(tab, **line_options)


        self.profileLine = FancyRadioGroup(tab, text="Operating System Profile:", labelWidth=labelWidth,
                                           radioWidth=radioWidth)
        self.generalPurposeButton = self.profileLine.add(text="General Purpose", value=TestConfiguration.Profile.General,    selected=True, command=self.enableInfoWidgets)
        self.safetyBaseButton = self.profileLine.add(text="Safety Base",     value=TestConfiguration.Profile.SafetyBase, selected=False, command=self.enableInfoWidgets)
        self.safetyExtendendButton = self.profileLine.add(text="Safety Extended", value=TestConfiguration.Profile.SafetyExt,  selected=False, command=self.enableInfoWidgets)
        self.securityButton = self.profileLine.add(text="Security",        value=TestConfiguration.Profile.Security,   selected=False, command=self.enableInfoWidgets)
        self.profileLine.pack(pack_options)
        Dialogs.insertHorizontalLine(tab, **line_options)


        self.partitionLine = FancyRadioGroup(tab, text="Operating System Partition:", labelWidth=labelWidth, radioWidth=radioWidth)
        self.posixRadioButton =    self.partitionLine.add(text="POSIX",
                                                          value=TestConfiguration.BaseConfig.Partition.POSIX,
                                                          selected=True,
                                                          command=self._onPosixPartition)
        self.arincRadioButton =    self.partitionLine.add(text="ARINC 653",
                                                          value=TestConfiguration.BaseConfig.Partition.ARINC_653,
                                                          command=self._onArincPartition)
        self.posixAndArincButton = self.partitionLine.add(text="POSIX+ARINC 653",
                                                          value=TestConfiguration.BaseConfig.Partition.POSIX_ARINC_653,
                                                          command=self._onPosixARINCPartition)
        self.partitionLine.pack(pack_options)
        Dialogs.insertHorizontalLine(tab, **line_options)


        self.openGLLine = FancyRadioGroup(tab, text="OpenGL Use:", labelWidth=labelWidth, radioWidth=radioWidth)
        self.openGLLine.add(text=TestConfiguration.OpenGLConfiguration.OpenGL_not_used,
                            value=TestConfiguration.OpenGLConfiguration.OpenGL_not_used,
                            selected=True)
        self.OpenGL_GLX_button = \
            self.openGLLine.add(text=TestConfiguration.OpenGLConfiguration.OpenGL_with_GLX,
                                value=TestConfiguration.OpenGLConfiguration.OpenGL_with_GLX)
        self.OpenGL_ES_button = \
            self.openGLLine.add(text=TestConfiguration.OpenGLConfiguration.OpenGL_ES_with_EGL,
                                value=TestConfiguration.OpenGLConfiguration.OpenGL_ES_with_EGL)

        self.OpenGL_SC_button = \
            self.openGLLine.add(text=TestConfiguration.OpenGLConfiguration.OpenGL_SC,
                                value=TestConfiguration.OpenGLConfiguration.OpenGL_SC)
        self.openGLLine.pack(pack_options)
        Dialogs.insertHorizontalLine(tab, **line_options)

        self.architectureLine = FancyRadioGroup(tab, text="Build O/S Architecture:", labelWidth= labelWidth, radioWidth=radioWidth)
        self.architectureLine.add(text="32-bit", value="32-bit", selected=True)
        self.architectureLine.add(text="64-bit", value="64-bit", selected=False)
        self.architectureLine.pack(pack_options)
        Dialogs.insertHorizontalLine(tab, **line_options)

        self.logLine          = FancyEntry(tab, text="Log Directory:", selectionType=fancyentry.DIRECTORY,
                                           title="Select Log Directory", labelWidth=labelWidth, entryWidth=entryWidth)
        self.logLine.pack(pack_options)

        self.browserLine      = FancyEntry(tab, text="Browser:", selectionType=fancyentry.FILE,
                                           title="Select Browser", labelWidth=labelWidth, entryWidth=entryWidth)
        self.browserLine.pack(pack_options)


    def createBuildTabWidgets(self, tab):
        tab = self.addFixedSizeFrame(tab, scrollable=True)

        entryWidth = self.defaultEntryWidth - 3
        labelWidth = self.defaultLabelWidth + 3
        horizontal_line_width = self.dialogWidth - 100

        if platform == "win32":
            executibleFilesTypes = [("executables", ".exe"), ("All files", "*")]
        else:
            executibleFilesTypes = [("All files", "*")]

        pack_options = { "side": TOP }
        line_options = { "width": self.dialogWidth - 100, "spacerHeight": 5}

        Dialogs.insertHorizontalLine(tab, **line_options)
        label = Label(tab, text="Build information associated with the project undergoing conformance testing (scroll for all options)")
        label.pack(side=TOP)
        Dialogs.insertHorizontalLine(tab, **line_options)

        self.addedPathDirsLine = FancyEntry(tab, text='Path Directories:',selectionType=fancyentry.DIRECTORIES,
                                            title="Select Additional Directories for Path",
                                            entryWidth = entryWidth, labelWidth=labelWidth)
        self.addedPathDirsLine.pack(pack_options)
        Dialogs.insertHorizontalLine(tab, **line_options)

        self.makeLine          = FancyEntry(tab, text="Make:", selectionType=fancyentry.FILE,
                                            title="Please select Make executable to use in build process",
                                            entryWidth=entryWidth,labelWidth=labelWidth,
                                            fileTypes= executibleFilesTypes, defaultValidate=False)
        self.makeLine.pack(pack_options)
        Dialogs.insertHorizontalLine(tab, **line_options)

        self.compilerLine      = FancyEntry(tab, text="Compiler:", selectionType=fancyentry.FILE,
                                            title="Please select compiler executable to use in build process",
                                            entryWidth=entryWidth, labelWidth=labelWidth,
                                            fileTypes= executibleFilesTypes, defaultValidate=False)
        self.compilerLine.pack(pack_options)

        self.compilerFlagLine  = FancyEntry(tab, text="Compiler Flags:",  selectionType=fancyentry.NONE,
                                            entryWidth=entryWidth+self.box_offset, labelWidth=labelWidth,
                                            defaultValidate=False)
        self.compilerFlagLine.pack(pack_options)

        self.processorSpecificFlagLine \
                               = FancyEntry(tab, text="Processor Specific Compiler Flags:",  selectionType=fancyentry.NONE,
                                            labelWidth = labelWidth, entryWidth=entryWidth+self.box_offset, defaultValidate=False)
        self.processorSpecificFlagLine.pack(pack_options)
        Dialogs.insertHorizontalLine(tab, **line_options)

        self.linkerLine        = FancyEntry(tab, text='Linker:', selectionType=fancyentry.FILE,
                                            title="Please select linker executable to use in build process",
                                            entryWidth=entryWidth, labelWidth=labelWidth,
                                            fileTypes= executibleFilesTypes, defaultValidate=False)
        self.linkerLine.pack(pack_options)

        self.linkerFlagLine    = FancyEntry(tab, text='Linker Flags:', selectionType=fancyentry.NONE,
                                            entryWidth=entryWidth+self.box_offset, labelWidth=labelWidth,
                                            defaultValidate=False)
        self.linkerFlagLine.pack(pack_options)
        Dialogs.insertHorizontalLine(tab, **line_options)

        self.binderLine        = FancyEntry(tab, text='Binder (Ada only):', selectionType=fancyentry.FILE,
                                            entryWidth=entryWidth, labelWidth= labelWidth,
                                            title="Please select binder executable to use in build process",
                                            fileTypes= executibleFilesTypes, defaultValidate=False)
        self.binderLine.pack(pack_options)

        self.binderFlagLine    = FancyEntry(tab, text='Binder Flags:', selectionType=fancyentry.NONE,
                                            entryWidth=entryWidth+self.box_offset, labelWidth=labelWidth,
                                            defaultValidate=False)
        self.binderFlagLine.pack(pack_options)
        Dialogs.insertHorizontalLine(tab, **line_options)

        self.archiverLine      = FancyEntry(tab, text='Archiver:', selectionType=fancyentry.FILE,
                                            title="Please select archiver executable to use in build process",
                                            entryWidth=entryWidth, labelWidth=labelWidth,
                                            fileTypes= executibleFilesTypes, defaultValidate=False)
        self.archiverLine.pack(pack_options)

        self.archiverFlagLine  = FancyEntry(tab, text='Archiver Flags:', selectionType=fancyentry.NONE,
                                            entryWidth=entryWidth+self.box_offset, labelWidth=labelWidth,
                                            defaultValidate=False)
        self.archiverFlagLine.pack(pack_options)
        Dialogs.insertHorizontalLine(tab, **line_options)

        self.objectSuffixLine  = FancyEntry(tab, text="Object file ext:", selectionType=fancyentry.NONE,
                                            entryWidth=entryWidth+self.box_offset, labelWidth=labelWidth,
                                            defaultValidate=False)
        self.objectSuffixLine.pack(pack_options)

        self.execSuffixLine    = FancyEntry(tab, text="Exec file ext:",   selectionType=fancyentry.NONE,
                                            entryWidth=entryWidth+self.box_offset, labelWidth=labelWidth,
                                            defaultValidate=False)
        self.execSuffixLine.pack(pack_options)

        Dialogs.insertHorizontalLine(tab, **line_options)
        Message(tab, text=
                "Data Model analysis utilizes a JVM.  For large data models, it may be required to increase the heap " +
                "size (-Xmx<size>[g|G|m|M|k|K]). On the line below, place any command line options needed to run the " +
                "JVM for the data model analysis.",
                width=horizontal_line_width).pack(pack_options)
        self.dataModelJVMOptionsLine = FancyEntry(tab, text='Java command line options: ', selectionType=fancyentry.NONE,
                                       entryWidth=entryWidth+self.box_offset, labelWidth=labelWidth,
                                       defaultValidate=False)
        self.dataModelJVMOptionsLine.pack(pack_options)
        Dialogs.insertHorizontalLine(tab, **line_options)



    def createPcsTabWidgets(self, tab):
        labelWidth = self.defaultLabelWidth
        entryWidth = self.defaultEntryWidth
        pack_options = { "side": TOP }
        line_options = { "width": self.dialogWidth - 100, "spacerHeight": 5}

        self.pcsTab = tab
        tab = self.addFixedSizeFrame(tab)

        self.pcsInfo = self.createSegmentInfo()
        self.pcsWidgets = list()

        Dialogs.insertHorizontalLine(tab, **line_options)
        text = "Check to enable PCS Segment testing. Then fill out of the required fields. " + \
               "Directories and file names should be separated by a \"" + pathsep + "\". You may add " + \
               "information by typing or by clicking the \"...\" button for a file browser."
        Message(tab, text=text, width=self.dialogWidth-100).pack(pack_options)

        Dialogs.insertHorizontalLine(tab, **line_options)

        Checkbutton(tab, text="Test Portable Components Segment", variable=self.pcsCheckBox, command=self.enableInfoWidgets).pack(side=TOP)

        line = FancyEntry(tab, text="Object/Library Files:", selectionType=fancyentry.CLASS_PATH,
                          initDir=self.getInitialDirectory(),
                          labelWidth=labelWidth, entryWidth=entryWidth,
                          title="Please select object/library files for the PCS")
        line.pack(pack_options)
        self.pcsWidgets.append(line)
        self.segmentObjectWidgets.append(line)

        self.pcsInfo.objectFiles = line.var

        # Java Approved Framework Additions - July 2019
        frameworkline = FancyEntry(tab, text="Approved Framework (Java Only):", selectionType=fancyentry.CLASS_PATH,
                          initDir=self.getInitialDirectory(),
                          labelWidth=labelWidth, entryWidth=entryWidth,
                          title="Please select Java approved framework files for the PCS")
        frameworkline.pack(pack_options)
        self.pcsWidgets.append(frameworkline)
        self.pcsInfo.frameworkFiles = frameworkline.var
        self.javaApprovedFrameworkWidgets.append(frameworkline)

    def createTssTabWidgets(self, tab):
        if constants.getFACEStandardVersion() == constants.FACE_STANDARD_VERSION_2_0:
            self.create2_0TssTabWidgets(tab)
        else:
            self.create2_1TssTabWidgets(tab)


    def create2_0TssTabWidgets(self, tab):
        labelWidth = self.defaultLabelWidth
        entryWidth = self.defaultEntryWidth
        horizontal_line_width = self.dialogWidth - 100
        pack_options = { "side": TOP }
        line_options = { "width": horizontal_line_width, "spacerHeight": 5}

        self.tssTab = tab
        tab = self.addFixedSizeFrame(tab)
        self.tssInfo = self.createSegmentInfo()
        self.tssWidgets = list()

        Dialogs.insertHorizontalLine(tab, **line_options)
        text = "Check to enable TSS Segment testing. Then fill out of the required fields. " +\
               "Directories and file names should be separated by a \"" + pathsep + "\". You may add " +\
               "information by typing or by clicking the \"...\" button for a file browser."
        Message(tab, text=text, width=horizontal_line_width).pack(pack_options)
        Dialogs.insertHorizontalLine(tab, **line_options)

        Checkbutton(tab, text="Test Transport Services Segment", variable=self.tssCheckBox, command=self.enableInfoWidgets).pack(side=TOP)

        line = FancyEntry(tab, text="Include Path:", selectionType=fancyentry.DIRECTORIES, initDir=self.getInitialDirectory(),
                          labelWidth=labelWidth, entryWidth=entryWidth, title="Please select directory(ies) for the TSS include path")
        line.pack(side=TOP)
        self.tssWidgets.append(line)
        self.includeWidgets.append(line)
        self.tssInfo.includePath = line.var

        line = FancyEntry(tab, text="Object/Library Files:", selectionType=fancyentry.CLASS_PATH, initDir=self.getInitialDirectory(),
                          labelWidth=labelWidth, entryWidth=entryWidth, title="Please select object/library files for the TSS")
        line.pack(side=TOP)
        self.tssWidgets.append(line)
        self.segmentObjectWidgets.append(line)
        self.tssInfo.objectFiles = line.var

        Dialogs.insertHorizontalLine(tab, **line_options)

        callbackCheckText = "Test Register Read Callback Interface"
        if constants.getFACEStandardVersion() in [constants.FACE_STANDARD_VERSION_2_1, constants.FACE_STANDARD_VERSION_2_1_1]:
            callbackCheckText = "Test Register Callback Interface"

        registerReadCallbackCheck = Checkbutton(tab, text=callbackCheckText, variable=self.tssRegisterReadCallbackCheckBox, command=self.enableInfoWidgets)

        registerReadCallbackCheck.pack(side=TOP)
        self.tssWidgets.append(registerReadCallbackCheck)

    def create2_1TssTabWidgets(self, tab):
        labelWidth = self.defaultLabelWidth
        entryWidth = self.defaultEntryWidth
        radioWidth = self.defaultRadioWidth

        horizontal_line_width = self.dialogWidth - 100
        pack_options = { "side": TOP }
        line_options = { "width": horizontal_line_width, "spacerHeight": 5}

        self.tssTab = tab
        tab = self.addFixedSizeFrame(tab)
        self.tssInfo = self.createSegmentInfo()
        self.tssWidgets = list()

        Dialogs.insertHorizontalLine(tab, **line_options)
        text = "Check to enable TSS Segment testing. Then fill out of the required fields. " +\
               "Directories and file names should be separated by a \"" + pathsep + "\". You may add " +\
               "information by typing or by clicking the \"...\" button for a file browser."
        Message(tab, text=text, width=horizontal_line_width).pack(pack_options)
        Dialogs.insertHorizontalLine(tab, **line_options)

        Checkbutton(tab, text="Test Transport Services Segment", variable=self.tssCheckBox, command=self.enableInfoWidgets).pack(side=TOP)

        Dialogs.insertHorizontalLine(tab, **line_options)

        self.tssInterfaceToTestLine = FancyRadioGroup(tab, text="TSS Interface to test:", labelWidth=labelWidth, radioWidth=radioWidth)
        self.testCompleteTSSRadioButton = self.tssInterfaceToTestLine.add(
            text=TestConfiguration.TsSegmentConfig.InterfaceUnderTest.TS_INTERFACE_UoP,
            value=TestConfiguration.TsSegmentConfig.InterfaceUnderTest.TS_INTERFACE_UoP,
            selected=True)
        self.tssWidgets.append(self.testCompleteTSSRadioButton)
        self.testTSSAbstractionLayerRadioButton = self.tssInterfaceToTestLine.add(
            text=TestConfiguration.TsSegmentConfig.InterfaceUnderTest.ABSTRACTION_LAYER,
            value=TestConfiguration.TsSegmentConfig.InterfaceUnderTest.ABSTRACTION_LAYER)
        self.tssWidgets.append(self.testTSSAbstractionLayerRadioButton)

        self.testTSSUoPRadioButton = self.tssInterfaceToTestLine.add(
            text=TestConfiguration.TsSegmentConfig.InterfaceUnderTest.TYPE_ABSTRACTION_INTERFACE_UoP,
            value=TestConfiguration.TsSegmentConfig.InterfaceUnderTest.TYPE_ABSTRACTION_INTERFACE_UoP)
        self.tssWidgets.append(self.testTSSUoPRadioButton)

        self.tssInterfaceToTestLine.pack(pack_options)
        Dialogs.insertHorizontalLine(tab, **line_options)


        line = FancyEntry(tab, text="Include Path:", selectionType=fancyentry.DIRECTORIES, initDir=self.getInitialDirectory(),
                          labelWidth=labelWidth, entryWidth=entryWidth, title="Please select directory(ies) for the TSS include path")
        line.pack(side=TOP)
        self.tssWidgets.append(line)
        self.includeWidgets.append(line)
        self.tssInfo.includePath = line.var

        self.includeFileLine = FancyEntry(tab, text="Include Files:", selectionType=fancyentry.FILES, initDir=self.getInitialDirectory(),
                          labelWidth=labelWidth, entryWidth=entryWidth, title="Please select include files containing TSS definitions")
        self.includeFileLine.pack(side=TOP)
        self.tssWidgets.append(self.includeFileLine)
        self.includeWidgets.append(self.includeFileLine)
        self.tssInfo.headerFiles = self.includeFileLine.var

        line = FancyEntry(tab, text="Object/Library Files:", selectionType=fancyentry.CLASS_PATH, initDir=self.getInitialDirectory(),
                          labelWidth=labelWidth, entryWidth=entryWidth, title="Please select object/library files for the TSS")
        line.pack(side=TOP)
        self.tssWidgets.append(line)
        self.segmentObjectWidgets.append(line)
        self.tssInfo.objectFiles = line.var

        # Java Approved Framework Additions - July 2019
        frameworkline = FancyEntry(tab, text="Approved Framework (Java Only):", selectionType=fancyentry.CLASS_PATH,
                          initDir=self.getInitialDirectory(),
                          labelWidth=labelWidth, entryWidth=entryWidth,
                          title="Please select Java approved framework files for the PSS")
        frameworkline.pack(pack_options)
        self.tssWidgets.append(frameworkline)
        self.tssInfo.frameworkFiles = frameworkline.var
        self.javaApprovedFrameworkWidgets.append(frameworkline)


        Dialogs.insertHorizontalLine(tab, **line_options)

        registerReadCallbackCheck = Checkbutton(tab, text="Test Register Read Callback Interface", variable=self.tssRegisterReadCallbackCheckBox, command=self.enableInfoWidgets)
        registerReadCallbackCheck.pack(side=TOP)
        self.tssWidgets.append(registerReadCallbackCheck)


    def createPssTabWidgets(self, tab):
        labelWidth = self.defaultLabelWidth
        entryWidth = self.defaultEntryWidth
        horizontal_line_width = self.dialogWidth - 100
        pack_options = { "side": TOP }
        line_options = { "width": horizontal_line_width, "spacerHeight": 5}

        self.pssTab = tab
        tab = self.addFixedSizeFrame(tab)
        self.pssInfo = self.createSegmentInfo()
        self.pssWidgets = list()

        Dialogs.insertHorizontalLine(tab, **line_options)
        text = "Check to enable PSS Segment testing. Then fill out of the required fields. " +\
               "Directories and file names should be separated by a \"" + pathsep + "\". You may add " +\
               "information by typing or by clicking the \"...\" button for a file browser."
        Message(tab, text=text, width=horizontal_line_width).pack(side=TOP)
        Dialogs.insertHorizontalLine(tab, **line_options)

        Checkbutton(tab, text="Test Platform Specific Services Segment", variable=self.pssCheckBox, command=self.enableInfoWidgets).pack(side=TOP)

        line = FancyEntry(tab, text="Object/Library Files:", selectionType=fancyentry.CLASS_PATH, initDir=self.getInitialDirectory(),
                          labelWidth=labelWidth, entryWidth=entryWidth, title="Please select object/library files for the PSS")

        line.pack(side=TOP)
        self.pssWidgets.append(line)
        self.segmentObjectWidgets.append(line)

        self.pssInfo.objectFiles = line.var

        # Java Approved Framework Additions - July 2019
        frameworkline = FancyEntry(tab, text="Approved Framework (Java Only):", selectionType=fancyentry.CLASS_PATH,
                          initDir=self.getInitialDirectory(),
                          labelWidth=labelWidth, entryWidth=entryWidth,
                          title="Please select Java approved framework files for the PSS")
        frameworkline.pack(pack_options)
        self.pssWidgets.append(frameworkline)
        self.pssInfo.frameworkFiles = frameworkline.var
        self.javaApprovedFrameworkWidgets.append(frameworkline)


    def getInitialDirectory(self):
        initial_dir = self.projectDirLine.var.get()
        if initial_dir.strip() == '':
            initial_dir = ConformanceMain.getRootDir()

        return initial_dir


    def createDatatypesTabWidgets(self, tab):
        baseConfigTitleFont = tkFont.Font(weight=tkFont.BOLD, slant=tkFont.ITALIC)
        labelWidth = self.defaultLabelWidth
        entryWidth = self.defaultEntryWidth
        horizontal_line_width = self.dialogWidth - 100
        pack_options = { "side": TOP }
        line_options = { "width": horizontal_line_width, "spacerHeight": 5}

        tab = self.addFixedSizeFrame(tab)
        Dialogs.insertHorizontalLine(tab, **line_options)
        Message(tab, text=
                "To test conformance of a Portable Components, Platform Specific Services, "
                + "or Transport Services segment, a FACE Data Model file that describes all "
                + "datatypes associated with the TSS interface is required.",
                width=horizontal_line_width).pack(pack_options)
        Dialogs.insertHorizontalLine(tab, **line_options)

        self.SharedDataModelLine = FancyEntry(tab, text='Shared Data Model: ', selectionType=fancyentry.FILE,
                                        labelWidth=labelWidth, initDir=self.getInitialDirectory(),
                                        entryWidth=entryWidth,title='Please select the FACE Shared Data Model file',
                                        fileTypes=[('all files','.*'),('FACE Data Model File','.face')])
        self.SharedDataModelLine.pack(pack_options)

        self.UoCDataModelLine = FancyEntry(tab, text='UoP Supplied Model (USM): ', selectionType=fancyentry.FILE, labelWidth=labelWidth,
                                        initDir=self.getInitialDirectory(), entryWidth=entryWidth,
                                        title='Please select the FACE UoP Supplied Model (USM) file',
                                        fileTypes=[('all files','.*'),('FACE Data Model File','.face')],
                                        validatecommand=self.getDataModelUoPs)
        self.UoCDataModelLine.pack(pack_options)
        self.dataModelWidgets = [ self.UoCDataModelLine, self.SharedDataModelLine ]
        Dialogs.insertHorizontalLine(tab, **line_options)

        self.uopFrame = Dialogs.VerticalScrolledFrame(tab, width=self.dialogWidth, height=self.dialogHeight-100)
        self.uopFrame.pack_propagate(0)
        self.uopFrame.pack(side=TOP)


        #self.uopFrame = Frame(tab)
        #self.uopFrame.pack(side=TOP)

    def createIosTabWidgets(self, tab):
        labelWidth = self.defaultLabelWidth
        entryWidth = self.defaultEntryWidth
        horizontal_line_width = self.dialogWidth - 100
        pack_options = { "side": TOP }
        line_options = { "width": horizontal_line_width, "spacerHeight": 5}

        self.iosTab = tab
        tab = self.addFixedSizeFrame(tab)
        self.iosInfo = self.createSegmentInfo()
        self.iosWidgets = list()

        Dialogs.insertHorizontalLine(tab, **line_options)

        text = "Check to enable IOS Segment testing. Then fill out of the required fields. " +\
               "Directories and file names should be separated by a \"" + pathsep + "\". You may add " +\
               "information by typing or by clicking the \"...\" button for a file browser."
        Message(tab, text=text, width=horizontal_line_width).pack(pack_options)

        Dialogs.insertHorizontalLine(tab, **line_options)

        Checkbutton(tab, text="Test I/O Services Segment", variable=self.iosCheckBox, command=self.enableInfoWidgets).pack(pack_options)

        line = FancyEntry(tab, text="Include Path:", selectionType=fancyentry.DIRECTORIES, initDir=self.getInitialDirectory(),
                          labelWidth=labelWidth, entryWidth=entryWidth, title="Please select directory(ies) for the IOS include path")
        line.pack(pack_options)
        self.iosWidgets.append(line)
        self.includeWidgets.append(line)
        self.iosInfo.includePath = line.var

        if constants.getFACEStandardVersion() in [constants.FACE_STANDARD_VERSION_2_1, constants.FACE_STANDARD_VERSION_2_1_1]:
            line = FancyEntry(tab, text="Include Files:", selectionType=fancyentry.FILES, initDir=self.getInitialDirectory(),
                              labelWidth=labelWidth, entryWidth=entryWidth, title="Please select include file(s) for the IOS")
            line.pack(pack_options)
            self.iosWidgets.append(line)
            self.includeWidgets.append(line)
            self.iosInfo.headerFiles = line.var

        line = FancyEntry(tab, text="Object/Library Files:", selectionType=fancyentry.CLASS_PATH, initDir=self.getInitialDirectory(),
                          labelWidth=labelWidth, entryWidth=entryWidth, title="Please select object/library files for the IOS")
        line.pack(pack_options)
        self.iosWidgets.append(line)
        self.segmentObjectWidgets.append(line)

        self.iosInfo.objectFiles = line.var

        # Java Approved Framework Additions - July 2019
        frameworkline = FancyEntry(tab, text="Approved Framework (Java Only):", selectionType=fancyentry.CLASS_PATH,
                          initDir=self.getInitialDirectory(),
                          labelWidth=labelWidth, entryWidth=entryWidth,
                          title="Please select Java approved framework files for the IOS")
        frameworkline.pack(pack_options)
        self.iosWidgets.append(frameworkline)
        self.iosInfo.frameworkFiles = frameworkline.var
        self.javaApprovedFrameworkWidgets.append(frameworkline)


    def createOssTabWidgets(self, tab):
        logger.info("createOssTabWidgets - Pressed OSS Tab")
        labelWidth = self.defaultLabelWidth
        entryWidth = self.defaultEntryWidth
        horizontal_line_width = self.dialogWidth - 100
        pack_options = { "side": TOP }
        line_options = { "width": horizontal_line_width, "spacerHeight": 5}

        self.ossTab = tab
        tab = self.addFixedSizeFrame(tab, scrollable=True)
        self.ossInfo = TestConfiguration.OsSegmentConfig()

        self.ossSubTestWidgets = dict()
        Dialogs.insertHorizontalLine(tab, **line_options)

        text = "Check to enable OSS Segment testing. Then fill out of the required fields. " +\
               "Directories and file names should be separated by a \"" + pathsep + "\". You may add " +\
               "information by typing or by clicking the \"...\" button for a file browser.  If the test must be conducted on a " +\
               "remote node, please enable the 'Test on remote node' checkbox (scroll down for all options)"
        Message(tab, text=text, width=horizontal_line_width).pack(pack_options)

        Dialogs.insertHorizontalLine(tab, **line_options)

        Checkbutton(tab, text="Test Operating System Segment", variable=self.ossCheckBox, command=self.enableInfoWidgets).pack(side=TOP)

        Dialogs.insertHorizontalLine(tab, **line_options)
        self.ossWidgets = list()

        numTest = 0
        for testName, testValues in sorted(self.ossInfo.subTests.iteritems()):
            testValues.underTest = IntVar()
            self.ossSubTestWidgets[testName] = list()

            if testName in [TestConfiguration.OsSegmentConfig.SubTests.JAVA_GENERAL,
                            TestConfiguration.OsSegmentConfig.SubTests.JAVA_SAFETY]:
                numTest += 1

                JavaApiTests = {TestConfiguration.OsSegmentConfig.SubTests.JAVA_GENERAL: 'J2EE8 & J2SE8',
                             TestConfiguration.OsSegmentConfig.SubTests.JAVA_SAFETY: 'J2SE8'}

                check = Checkbutton(tab, text="Test " + testName + " Profile API (" + JavaApiTests[testName] +")",
                                    variable=testValues.underTest, command=self.enableInfoWidgets)
                check.pack(pack_options)
                self.ossWidgets.append(check)
                testValues.checkBox = check

                line = FancyEntry(tab, text="Java Class Path:", selectionType=fancyentry.CLASS_PATH,
                                  labelWidth=labelWidth, entryWidth=entryWidth,
                                  title="Please select the Java Class Path for " + JavaApiTests[testName])
                line.pack(side=TOP)
                self.ossSubTestWidgets[testName].append(line)
                testValues.objectFiles = line.var
                Dialogs.insertHorizontalLine(tab, **line_options)
                continue

            numTest += 1
            check = Checkbutton(tab, text="Test " + testName + " API", variable=testValues.underTest, command=self.enableInfoWidgets)
            check.pack(pack_options)
            self.ossWidgets.append(check)
            testValues.checkBox = check

            line  = FancyEntry(tab, text="Additional Compiler Flags:",  selectionType=fancyentry.NONE,
                               labelWidth=labelWidth, entryWidth=entryWidth+self.box_offset)
            line.pack(side=TOP)
            self.ossSubTestWidgets[testName].append(line)
            testValues.compilerFlags = line.var

            line  = FancyEntry(tab, text="Additional Linker Flags:",  selectionType=fancyentry.NONE,
                               labelWidth=labelWidth, entryWidth=entryWidth+self.box_offset)
            line.pack(side=TOP)
            self.ossSubTestWidgets[testName].append(line)
            testValues.linkerFlags = line.var

            line = FancyEntry(tab, text="Include Path:", selectionType=fancyentry.DIRECTORIES,
                              labelWidth=labelWidth, entryWidth=entryWidth,
                              title="Please select directory(ies) for the OSS include path")
            line.pack(side=TOP)
            self.ossSubTestWidgets[testName].append(line)
            testValues.includePath = line.var

            if testName == 'ARINC 653':
                line = FancyEntry(tab, text="ARINC 653 Header Files:", selectionType=fancyentry.FILES,
                                  labelWidth=labelWidth, entryWidth=entryWidth,
                                  title="Please select ARINC 653 header definition file(s) for the OSS")
                line.pack(side=TOP)
                self.ossSubTestWidgets[testName].append(line)
                testValues.headerFiles = line.var

            line = FancyEntry(tab, text="Object/Library Files:", selectionType=fancyentry.CLASS_PATH,
                              labelWidth=labelWidth, entryWidth=entryWidth,
                              title="Please select object/library files for the OSS")
            line.pack(side=TOP)
            self.ossSubTestWidgets[testName].append(line)
            self.segmentObjectWidgets.append(line)

            testValues.objectFiles = line.var
            Dialogs.insertHorizontalLine(tab, **line_options)


    def createNotesTabWidgets(self, tab):
        tab = self.addFixedSizeFrame(tab)
        labelWidth = self.defaultLabelWidth
        entryWidth = self.defaultEntryWidth
        horizontal_line_width = self.dialogWidth - 100
        pack_options = { "side": TOP }
        line_options = { "width": horizontal_line_width, "spacerHeight": 5}

        Dialogs.insertHorizontalLine(tab, **line_options)
        text = "Place any notes concerning the configuration in the text box"
        self.label = Label(tab, text=text, width=entryWidth, anchor=N).pack(side=TOP)
        Dialogs.insertHorizontalLine(tab, **line_options)
        scrollbar = Scrollbar(tab)
        scrollbar.pack(side=RIGHT, fill=Y)
        self.notesText = Text(tab, width=self.dialogWidth-50, height=self.dialogHeight-50, yscrollcommand=scrollbar.set)
        self.notesText.pack(side=TOP)
        scrollbar.config(command=self.notesText.yview)

        Dialogs.insertHorizontalLine(tab, **line_options)


    ''' Helper Routines that help to change project base directory '''

    def updateProjectDirectory(self):
        newProjectDir = self.getInitialDirectory()
        returnValue = False

        if self.lastProjectDir is None:
            self.lastProjectDir = ''
            if self.config is not None:
                self.lastProjectDir = self.config.baseConfig.projectDirectory

        logger.debug('NewDir: ' + newProjectDir + ' Old Dir: ' + self.lastProjectDir)
        changeProjectDir = True

        if newProjectDir == self.lastProjectDir: # nothing changed...
            returnValue = True

        elif newProjectDir == '': # clearing the project directory
            if self.lastProjectDir != '':
                logger.warning('Erasing Project Directory: ' + self.lastProjectDir)
            self.lastProjectDir = ''
            returnValue = True

        else:
            newConfig = self._guiToConfiguration()
            # need to put the old project directory in...so we can change it to the new one
            newConfig.baseConfig.projectDirectory = self.lastProjectDir
            exceptionMsgList = list()
            newConfig.changeProjectDirectory(newProjectDir, True, exceptionMsgList)
            if len(exceptionMsgList) > 0:
                yesOrNo = Dialogs.YesNoMessageBox(constants.FACE_TESTSUITE_NAME,
                    'There are issues with changing project directory to ' + newProjectDir + '\n\n' +
                    '\n'.join(exceptionMsgList) +
                    '\n\nYou will need to manually fix these issues if you continue in changing the project ' +
                    'directory.\n\nDo you wish to continue? ',
                    parent=self)
                if yesOrNo == 'no':
                    self.projectDirLine.setValue(self.lastProjectDir)
                else:
                    exceptionMsgList = list()

            if len(exceptionMsgList) == 0:
                self._configurationToGui(newConfig)
                self.lastProjectDir = newProjectDir
                returnValue = True

        return returnValue



    ''' Helper routines that help disable/enable widgets based on configuration '''
    def enableInfoWidgets(self):
        """ enable all widgets based on which segment checkboxes are enabled """
        if not self.infoWidgetsDefined:
            return

        self.enableWidgets(self.pcsCheckBox, self.pcsWidgets, self.pcsTab)
        self.enableWidgets(self.tssCheckBox, self.tssWidgets, self.tssTab)
        self.enableWidgets(self.pssCheckBox, self.pssWidgets, self.pssTab)
        self.enableWidgets(self.iosCheckBox, self.iosWidgets, self.iosTab)
        self.enableWidgets(self.ossCheckBox, self.ossWidgets, self.ossTab)

        language = self.languageLine.var.get()
        profile = self.profileLine.var.get()

        dialogPrompt = self.DEFAULT_DIALOG_PROMPT
        findText = 'Java class path'
        replaceText = 'object/library files'

        if language == TestConfiguration.Language.Java:
            if not os.path.isdir(
                    os.path.join(ConformanceMain.getRootDir(), constants.JAVA_CONFORMANCE_TEST_DIR)):
                Dialogs.ErrorMessageBox('Must Install Java Conformance Extension',
                                        'In order to test a Java segment, you must install the Java Conformance ' + \
                                        'Extension for your Operating System.  See manual for more details. ' + \
                                        'Currently supported Operating Systems: ' + \
                                        ', '.join(constants.JAVA_CONFORMANCE_EXTENSION_OS_LIST))
            tempText = findText
            findText = replaceText
            replaceText = tempText

        if language in self.dialogPrompt:
            dialogPrompt = self.dialogPrompt[language]
        for s in self.segmentObjectWidgets:
            s.dialogPrompt = dialogPrompt
            s.title = s.title.replace(findText, replaceText)
            s.setLabel(s.getLabel().replace(findText.title(), replaceText.title()))

        if language == TestConfiguration.Language.Java:
            if self.profileLine.var.get() == TestConfiguration.Profile.Security:
                self.profileLine.var.set(TestConfiguration.Profile.General)
            self.securityButton.configure(state=DISABLED)
        else:
            self.securityButton.configure(state=NORMAL)

        profile = self.profileLine.var.get()

        ''' Handle OSS checkboxes '''
        if self.ossCheckBox.get() != 1:
            for key in self.ossInfo.subTests.keys():
                self.enableOSSWidgets(self.ossCheckBox, self.ossSubTestWidgets[key])

        else:
            '''
            validTests = {TestConfiguration.Language.C:
                          [TestConfiguration.OsSegmentConfig.SubTests.ARINC_653,
                           TestConfiguration.OsSegmentConfig.SubTests.C_STD_LIB,
                           TestConfiguration.OsSegmentConfig.SubTests.POSIX,
                           TestConfiguration.OsSegmentConfig.SubTests.HMFM] +
                          TestConfiguration.OpenGLConfiguration.ConformanceTests.validTests[profile],

                          TestConfiguration.Language.CPP:
                          [TestConfiguration.OsSegmentConfig.SubTests.CPP_STD_LIB,
                           TestConfiguration.OsSegmentConfig.SubTests.HMFM],

                          TestConfiguration.Language.Ada:
                          [TestConfiguration.OsSegmentConfig.SubTests.ARINC_653,
                           TestConfiguration.OsSegmentConfig.SubTests.HMFM],

                          TestConfiguration.Language.Java:
                            [TestConfiguration.OsSegmentConfig.SubTests.JAVA_GENERAL,
                             TestConfiguration.OsSegmentConfig.SubTests.JAVA_SAFETY]
                          }
            '''
            validTests = {TestConfiguration.Language.C:
                          [TestConfiguration.OsSegmentConfig.SubTests.C_STD_LIB,
                           TestConfiguration.OsSegmentConfig.SubTests.HMFM] +
                          TestConfiguration.OpenGLConfiguration.ConformanceTests.validTests[profile],

                          TestConfiguration.Language.CPP:
                          [TestConfiguration.OsSegmentConfig.SubTests.CPP_STD_LIB,
                           TestConfiguration.OsSegmentConfig.SubTests.HMFM],

                          TestConfiguration.Language.Ada:
                          [TestConfiguration.OsSegmentConfig.SubTests.ARINC_653,
                           TestConfiguration.OsSegmentConfig.SubTests.HMFM],

                          TestConfiguration.Language.Java:
                            [TestConfiguration.OsSegmentConfig.SubTests.JAVA_GENERAL,
                             TestConfiguration.OsSegmentConfig.SubTests.JAVA_SAFETY]
                          }
            partition = self.partitionLine.var.get()
            if partition == 'POSIX+ARINC_653':      # POSIX with a minimum set of ARINC
                validTests[TestConfiguration.Language.C].append(TestConfiguration.OsSegmentConfig.SubTests.ARINC_653)
                validTests[TestConfiguration.Language.C].append(TestConfiguration.OsSegmentConfig.SubTests.POSIX)
            elif partition == 'POSIX':              # POSIX Only
                validTests[TestConfiguration.Language.C].append(TestConfiguration.OsSegmentConfig.SubTests.POSIX)
            else:                                   # ARINC Only
                validTests[TestConfiguration.Language.C].append(TestConfiguration.OsSegmentConfig.SubTests.ARINC_653)


            javaValidProfileTests = {TestConfiguration.OsSegmentConfig.SubTests.JAVA_GENERAL:
                                         [TestConfiguration.Profile.General],
                                     TestConfiguration.OsSegmentConfig.SubTests.JAVA_SAFETY:
                                         [TestConfiguration.Profile.SafetyBase, TestConfiguration.Profile.SafetyExt]}

            for key, subTest in self.ossInfo.subTests.iteritems():
                if key in validTests[language] and (language != TestConfiguration.Language.Java or
                                                    profile in javaValidProfileTests[key]):
                        logger.debug('Enabling ' + key + ' in the language: ' + language + ' and profile: ' + profile)
                        self.enableOSSWidgets(subTest.underTest, self.ossSubTestWidgets[key])
                        subTest.checkBox.configure(state=NORMAL)
                else:
                    logger.debug('Disabling ' + key + ' in the language: ' + language + ' and profile: ' + profile)

                    subTest.underTest.set(0)
                    self.enableOSSWidgets(subTest.underTest, self.ossSubTestWidgets[key])
                    subTest.checkBox.configure(state=DISABLED)




        OpenGL_use = self.openGLLine.var.get()
        logger.debug('Profile: ' + profile)

        if language == TestConfiguration.Language.Java:
            self.OpenGL_GLX_button.configure(state=DISABLED)
            self.OpenGL_ES_button.configure(state=DISABLED)
            self.OpenGL_SC_button.configure(state=DISABLED)
            if OpenGL_use != TestConfiguration.OpenGLConfiguration.OpenGL_not_used:
                self.openGLLine.var.set(TestConfiguration.OpenGLConfiguration.OpenGL_not_used)

        elif profile == TestConfiguration.Profile.General:
            self.posixRadioButton.configure(state=NORMAL)
            self.OpenGL_GLX_button.configure(state=NORMAL)
            self.OpenGL_ES_button.configure(state=NORMAL)
            self.OpenGL_SC_button.configure(state=DISABLED)
            if OpenGL_use == TestConfiguration.OpenGLConfiguration.OpenGL_SC:
                self.openGLLine.var.set(TestConfiguration.OpenGLConfiguration.OpenGL_with_GLX)

        else:
            self.posixRadioButton.configure(state=DISABLED)
            if self.partitionLine.var.get() == TestConfiguration.BaseConfig.Partition.POSIX:
                self.partitionLine.var.set(TestConfiguration.BaseConfig.Partition.POSIX_ARINC_653)

            if profile in [TestConfiguration.Profile.SafetyBase,
                           TestConfiguration.Profile.SafetyExt]:
                self.OpenGL_GLX_button.configure(state=DISABLED)
                self.OpenGL_ES_button.configure(state=DISABLED)
                self.OpenGL_SC_button.configure(state=NORMAL)
                if OpenGL_use in [TestConfiguration.OpenGLConfiguration.OpenGL_with_GLX,
                                  TestConfiguration.OpenGLConfiguration.OpenGL_ES_with_EGL]:
                    self.openGLLine.var.set(TestConfiguration.OpenGLConfiguration.OpenGL_SC)

            else: # Security Profile
                self.OpenGL_GLX_button.configure(state=DISABLED)
                self.OpenGL_ES_button.configure(state=DISABLED)
                self.OpenGL_SC_button.configure(state=DISABLED)
                if OpenGL_use != TestConfiguration.OpenGLConfiguration.OpenGL_not_used:
                    self.openGLLine.var.set(TestConfiguration.OpenGLConfiguration.OpenGL_not_used)


    def enableWidgets(self, checkBox, widgets, tab):
        """ enable widgets for one segment if its checkbox is enabled

            checkBox - checkBox variable (IntVar)
            entryFrame - list of widgets
        """
        CHECKMARK = u'\u2713'
        tabText = self.tabs.getTabText(tab)

        if checkBox.get() == 1:
            newState = NORMAL
            if not tabText.endswith(CHECKMARK):
                self.tabs.setTabText(tab, tabText + CHECKMARK)
        else:
            newState = DISABLED
            if tabText.endswith(CHECKMARK):
                self.tabs.setTabText(tab, tabText[:-1])

        for w in widgets:
            w.configure(None, state=newState)
            if self.languageLine.var.get() == TestConfiguration.Language.Java:
                if w in self.includeWidgets:
                    w.configure(None, state=DISABLED)
            else:
                # Java Approved Framework Additions - July 2019
                if w in self.javaApprovedFrameworkWidgets:
                    w.configure(None, state=DISABLED)

    def enableOSSWidgets(self, checkBox, widgets):
        """ enable widgets for one segment if its checkbox is enabled

            checkBox - checkBox variable (IntVar)
            entryFrame - list of widgets
        """
        if checkBox.get() == 1:
            newState = NORMAL
        else:
            newState = DISABLED

        for w in widgets:
            w.configure(None, state=newState)


    ''' Helper routines to help the data model tab '''
    def datatypes_dialog(self, uopName):
        logger.debug('uop: ' + uopName)

        dList = list()
        messages = self.dataModel.getUoPMessages(uopName)
        Dialogs.InfoMessageBox(constants.FACE_TESTSUITE_NAME, "Messages for " + uopName + '\n' + "\n".join(messages), parent=self)

    def getDataModelUoPs(self):

        uocDataModelFile = self.UoCDataModelLine.var.get()
        logger.info("dataModelFile: " + str(uocDataModelFile))

        if uocDataModelFile is None or uocDataModelFile == "" or uocDataModelFile == "None":
            logger.debug("dataModelFile blank, not loading a data model file")
            self.dataModel = None # Clearing out old dataModel
            self.uopsToTest = dict() # Clearing out old Uops to test

        else:  # Need to process a dataModel
            loadDataModel = True

            if self.dataModel is not None:
                if self.dataModel.filename() == uocDataModelFile and os.path.isfile(uocDataModelFile):
                    if self.dataModel.getHashValue(uocDataModelFile) == self.dataModel.uocDMHashValue  and \
                        self.dataModel.uocDMHashValue is not None:
                            # Nothing changed...don't need to reload
                            loadDataModel = False
                    else: # hash value must have been different...
                        Dialogs.WarningMessageBox(constants.FACE_TESTSUITE_NAME,
                                "Data file has changed, will need to reload and pick UoPs under test", parent=self)

            # Otherwise, we need to load the new data model
            if loadDataModel:
                self.dataModel = None # clear the last data model
                if os.path.isfile(uocDataModelFile):
                    SplashScreen.show("Please wait...loading FACE data model")
                    self.dataModel = DataModel.DataModel(uocDataModelFilename=uocDataModelFile,
                                                         jvmCommandLineOptions=self.dataModelJVMOptionsLine.var.get())
                    if not self.dataModel.isValid(): # datamodel didn't load correctly
                        self.dataModel = None # Then clear it
                    SplashScreen.hide()
                else:
                    errorMsg = 'Data model file: '+ uocDataModelFile + ' does not exists'
                    Dialogs.ErrorMessageBox(constants.FACE_TESTSUITE_NAME, errorMsg, parent=self)
                    logger.error(errorMsg)
                    self.dataModel = None
                    self.uopsToTest = None

            # If the dataModel has the same UoPs as the last dataModel, then going to assume
            # that the last uopsToTest will be the same ones as the new uopsToTest
            if self.dataModel is not None and \
                    sorted(self.dataModel.getUoPNames()) == sorted(self.config.datatypeConfig.uopsToTest.keys()):
                self.uopsToTest = self.config.datatypeConfig.uopsToTest

        self.populateUoPFrame()

        return True

    def getUopButton(self, frame, uop):
        return Button(frame, text=uop + " Properties", command=lambda: self.datatypes_dialog(uop))

    def populateUoPFrame(self):

        # Destroy old widgets
        for u in self.uopButtons.itervalues():
            u.destroy()
        for u in self.uopCheckButtons.itervalues():
            u.destroy()
        for l in self.uopHeadings:
             l.destroy()

        # reinitialize widget holders
        self.uopHeadings = list()
        self.uopCheckButtons = dict()
        self.uopButtons = dict()
        self.uopVar = dict()
        row = 0

        headingFrame = Frame(self.uopFrame.interior)
        self.uopHeadings.append(headingFrame)
        headingFrame.pack(side=TOP)


        # No dataModel...means there isn't a valid data model file
        if self.dataModel is None:
            if self.UoCDataModelLine.var.get() == '': # means there isn't a file specified
                self.uopHeadings.append(
                    Label(headingFrame, text="Please load a data model file", font=tkFont.Font(size=8, weight = tkFont.BOLD)))
            else: # means that a file is specified, but must not be valid
                self.uopHeadings.append(
                    Label(headingFrame, text="UoC Data model file: " + os.path.basename(self.UoCDataModelLine.var.get())
                                             + " is invalid", font=tkFont.Font(size=8, weight = tkFont.BOLD, slant = tkFont.ITALIC)))
            self.uopHeadings[-1].pack(side=TOP)

        # valid data model
        else:
            unitsOfPortability = self.dataModel.getUoPs()
            if len(unitsOfPortability) > 0:
                self.uopHeadings[-1].pack(side=TOP)
                self.uopHeadings.append(Label(headingFrame, text="Units of Portability\n(check which UoP(s) should be included in conformance test)"))
                self.uopHeadings[-1].pack(side=TOP)
                self.uopHeadings.append(Dialogs.insertHorizontalLine(headingFrame, width=300))

            else:
                self.uopHeadings.append(
                    Label(headingFrame, text="UoC Data model file: " + os.path.basename(self.UoCDataModelLine.var.get())
                                             + " does not contain any units of portability",
                                        font=tkFont.Font(size=8, weight = tkFont.BOLD, slant = tkFont.ITALIC)))
                self.uopHeadings[-1].pack(side=TOP)

            uopSubFrame = Frame(self.uopFrame.interior)
            self.uopHeadings.append(uopSubFrame)

            for uop in sorted(unitsOfPortability.keys()):
                messages = ''
                logger.debug("uop: " + uop + " datatypes: " + ', '.join(unitsOfPortability[uop]))
                self.uopVar[uop] = IntVar()
                if uop in self.uopsToTest:
                    self.uopVar[uop].set(self.uopsToTest[uop])
                self.uopCheckButtons[uop] = Checkbutton(uopSubFrame, text=uop, variable=self.uopVar[uop])
                self.uopCheckButtons[uop].grid(row=row, column=0, sticky=W)
                self.uopButtons[uop] = self.getUopButton(uopSubFrame, uop)
                self.uopButtons[uop].grid(row=row, column=1)
                row += 1



            self.uopButtons['SPACER_AT_END'] = Label(uopSubFrame, text=' ')
            self.uopButtons['SPACER_AT_END'].grid(row=row, column=0)

            uopSubFrame.pack(side=TOP)



    def _guiToConfiguration(self):
        """ returns an instance of the TestSuite.ConformanceTest class containing the configuration defined on the GUI"""

        config = ConformanceTest.ConformanceTest()
        config.configFile               = self.configurationFilename
        config.baseConfig.projectDirectory = self.projectDirLine.var.get()

        config.baseConfig.osProfile           = self.profileLine.var.get()
        config.baseConfig.osPartition         = self.partitionLine.var.get()
        config.baseConfig.openGLConfiguration = self.openGLLine.var.get()
        config.baseConfig.language            = self.languageLine.var.get()
        config.baseConfig.architecture        = self.architectureLine.var.get()

        config.baseConfig.make          = self.makeLine.var.get()
        # Handle if pymake is to be used....substitute the actual pymake command line'
        if config.baseConfig.make == 'pymake':
            config.baseConfig.make = 'python "' + os.path.join(ConformanceMain.getRootDir(), constants.APP_DIR, 'make.py') + '"'

        config.baseConfig.compiler      = self.compilerLine.var.get()
        config.baseConfig.compilerFlags = self.compilerFlagLine.var.get()
        config.baseConfig.processorSpecificCompilerFlags = self.processorSpecificFlagLine.var.get()

        config.baseConfig.linker        = self.linkerLine.var.get()
        config.baseConfig.linkerFlags   = self.linkerFlagLine.var.get()
        config.baseConfig.binder        = self.binderLine.var.get()
        config.baseConfig.binderFlags   = self.binderFlagLine.var.get()
        config.baseConfig.archiver      = self.archiverLine.var.get()
        config.baseConfig.archiverFlags = self.archiverFlagLine.var.get()
        config.baseConfig.objectSuffix  = self.objectSuffixLine.var.get()
        config.baseConfig.logDirectory  = self.logLine.var.get()
        config.baseConfig.browser       = self.browserLine.var.get()
        config.baseConfig.execSuffix    = self.execSuffixLine.var.get()
        config.baseConfig.addedPathDirs = self.addedPathDirsLine.var.get().split(pathsep)
        config.baseConfig.dataModelJVMOptions =  self.dataModelJVMOptionsLine.var.get()

        config.testPC = self.pcsCheckBox.get()
        config.pcConfig.includePath = self.pcsInfo.includePath.get().split(pathsep)
        config.pcConfig.headerFiles = self.pcsInfo.headerFiles.get().split(pathsep)
        config.pcConfig.libraries   = self.pcsInfo.objectFiles.get().split(pathsep)
        config.pcConfig.frameworkFiles = self.pcsInfo.frameworkFiles.get().split(pathsep)

        config.testTS = self.tssCheckBox.get()
        config.tsConfig.includePath = self.tssInfo.includePath.get().split(pathsep)
        config.tsConfig.headerFiles = self.tssInfo.headerFiles.get().split(pathsep)
        config.tsConfig.libraries   = self.tssInfo.objectFiles.get().split(pathsep)
        config.tsConfig.registerReadCallback = self.tssRegisterReadCallbackCheckBox.get()
        config.tsConfig.frameworkFiles = self.tssInfo.frameworkFiles.get().split(pathsep)

        if constants.getFACEStandardVersion() in [constants.FACE_STANDARD_VERSION_2_1, constants.FACE_STANDARD_VERSION_2_1_1]:
            config.tsConfig.interfaceUnderTest = self.tssInterfaceToTestLine.var.get()

        config.testPSS = self.pssCheckBox.get()
        config.pssConfig.includePath = self.pssInfo.includePath.get().split(pathsep)
        config.pssConfig.headerFiles = self.pssInfo.headerFiles.get().split(pathsep)
        config.pssConfig.libraries   = self.pssInfo.objectFiles.get().split(pathsep)
        config.pssConfig.frameworkFiles = self.pssInfo.frameworkFiles.get().split(pathsep)


        config.testIOS = self.iosCheckBox.get()
        config.ioConfig.includePath = self.iosInfo.includePath.get().split(pathsep)
        config.ioConfig.headerFiles = self.iosInfo.headerFiles.get().split(pathsep)
        config.ioConfig.libraries   = self.iosInfo.objectFiles.get().split(pathsep)
        config.ioConfig.frameworkFiles = self.iosInfo.frameworkFiles.get().split(pathsep)


        # Need to overwrite headerfiles for 2.0, since they are specified in standard
        if constants.getFACEStandardVersion() == constants.FACE_STANDARD_VERSION_2_0:
            # Must use tss.h or tss.hpp for TS configuration (ios.h or ios.hpp for IOS configuration
            if config.baseConfig.language in [TestConfiguration.Language.C, TestConfiguration.Language.CPP]:
                config.tsConfig.headerFiles = [os.path.join('face', 'tss' + config.baseConfig.getLanguageHeaderSuffix())]
                config.ioConfig.headerFiles = [os.path.join('face', 'ios' + config.baseConfig.getLanguageHeaderSuffix())]
            elif config.baseConfig.language == TestConfiguration.Language.Ada:
                config.tsConfig.headerFiles = [os.path.join('face', 'face-ts' + config.baseConfig.getLanguageHeaderSuffix())]
                config.ioConfig.headerFiles = [os.path.join('face', 'face-io' + config.baseConfig.getLanguageHeaderSuffix())]
            else:
                # Java doesn't have header files...
                config.tsConfig.headerFiles = list()
                config.ioConfig.headerFiles = list()


        config.testOS = self.ossCheckBox.get()
        for testName, testValues in config.osConfig.subTests.iteritems():
            testValues.underTest = self.ossInfo.subTests[testName].underTest.get()
            testValues.objectFiles = self.ossInfo.subTests[testName].objectFiles.get().split(pathsep)
            if testName in [TestConfiguration.OsSegmentConfig.SubTests.JAVA_GENERAL,
                            TestConfiguration.OsSegmentConfig.SubTests.JAVA_SAFETY]:
                continue # nothing else to get for Java Tests

            testValues.compilerFlags = self.ossInfo.subTests[testName].compilerFlags.get()
            testValues.includePath = self.ossInfo.subTests[testName].includePath.get().split(pathsep)
            testValues.linkerFlags = self.ossInfo.subTests[testName].linkerFlags.get()
            if testName == TestConfiguration.OsSegmentConfig.SubTests.ARINC_653:
                testValues.headerFiles = self.ossInfo.subTests[testName].headerFiles.get().split(pathsep)

        if self.dataModel is not None:
            self.dataModel.sharedDataModelFilename = self.SharedDataModelLine.var.get()
            self.dataModel.unitOfConformanceDataModelFilename = self.UoCDataModelLine.var.get()
            self.dataModel.jvmCommandLineOptions = config.baseConfig.dataModelJVMOptions


        config.datatypeConfig.dataModel = self.dataModel
        config.datatypeConfig.uopsToTest = dict()
        for key, intVar in self.uopVar.iteritems():
            config.datatypeConfig.uopsToTest[key] = intVar.get()

        config.baseConfig.notes = self.notesText.get(0.0, END).lstrip().rstrip() # strip leading and ending whitespaces

        logger.info(commonTools.toString(config.baseConfig))

        # Add more here
        return config

    def chooseNewProjectDirectory(self, config):
        newProjectLoc = Dialogs.osAgnosticOpenDirectoryDialog()
        if newProjectLoc != '':
            newConfig = copy.deepcopy(config)

            exceptionMsgList = list()
            newConfig.changeProjectDirectory(newProjectLoc, True, exceptionMsgList)
            if len(exceptionMsgList) > 0:
                yesOrNo = Dialogs.YesNoMessageBox(constants.FACE_TESTSUITE_NAME,
                    'There are issues with changing project directory to ' + newProjectLoc + '\n\n' +
                    '\n'.join(exceptionMsgList) +
                    '\n\nYou will need to manually fix these issues if you continue in changing the project ' +
                    'directory.\n\nDo you wish to continue? ',
                    parent=self)
                if yesOrNo == 'no':
                    return config

            self.lastProjectDir = newConfig.baseConfig.projectDirectory
            return newConfig


    def _configurationToGui(self, config):
        """ Sets the GUI based on the configuration passed to this routine

        config - TestSuite.ConformanceTest instance defining the configuration
        """
        try:
        #if True:
            # handle the pymake special case
            if 'make.py' in config.baseConfig.make:
                config.baseConfig.checkMakeValue()
                self.makeLine.var.set('pymake')
            else:
                self.makeLine.var.set(config.baseConfig.make)


            if config.baseConfig.projectDirectory != '' and not os.path.isdir(config.baseConfig.projectDirectory):
                chooseProjectLoc = Dialogs.YesNoMessageBox(constants.FACE_TESTSUITE_NAME, 'Project Directory: ' +
                    config.baseConfig.projectDirectory + ' does not exist, do you wish to choose a new project location?')

                if chooseProjectLoc == 'yes':
                    config = self.chooseNewProjectDirectory(config)

            self.projectDirLine.var.set(config.baseConfig.projectDirectory)
            self.lastProjectDir = config.baseConfig.projectDirectory

            self.compilerLine.var.set(config.baseConfig.compiler)
            self.compilerFlagLine.var.set(config.baseConfig.compilerFlags)
            self.processorSpecificFlagLine.var.set(config.baseConfig.processorSpecificCompilerFlags)
            self.linkerLine.var.set(config.baseConfig.linker)
            self.linkerFlagLine.var.set(config.baseConfig.linkerFlags)
            self.binderLine.var.set(config.baseConfig.binder)
            self.binderFlagLine.var.set(config.baseConfig.binderFlags)
            self.archiverLine.var.set(config.baseConfig.archiver)
            self.archiverFlagLine.var.set(config.baseConfig.archiverFlags)
            self.objectSuffixLine.var.set(config.baseConfig.objectSuffix)
            self.execSuffixLine.var.set(config.baseConfig.execSuffix)
            self.dataModelJVMOptionsLine.var.set(config.baseConfig.dataModelJVMOptions)

            if(config.baseConfig.logDirectory != ""):
                self.logLine.var.set(config.baseConfig.logDirectory)
            else:
                self.logLine.var.set(ConformanceMain.getRootDir() + sep + 'logFiles')
            if (config.baseConfig.browser is None) or (config.baseConfig.browser == ""):
                config.baseConfig.browser = commonTools.getDefaultBrowser()
                if config.baseConfig.browser is None:
                    Dialogs.InfoMessageBox(constants.FACE_TESTSUITE_NAME,
                                          'Test suite could not find browser on your system, press okay to specify one',
                                          parent=self)
                    config.baseConfig.browser = Dialogs.osAgnosticOpenFilenameDialog(title='Please specify browser',
                                                                                         parent=self)

            self.browserLine.var.set(config.baseConfig.browser)

            self.addedPathDirsLine.var.set(listToString(config.baseConfig.addedPathDirs, pathsep))

            self.profileLine.var.set(config.baseConfig.osProfile)
            self.partitionLine.var.set(config.baseConfig.osPartition)
            self.openGLLine.var.set(config.baseConfig.openGLConfiguration)
            self.languageLine.var.set(config.baseConfig.language)
            self.architectureLine.var.set(config.baseConfig.architecture)

            self.pcsCheckBox.set(config.testPC)
            self.pcsInfo.includePath.set(listToString(config.pcConfig.includePath))
            self.pcsInfo.headerFiles.set(listToString(config.pcConfig.headerFiles))
            self.pcsInfo.objectFiles.set(listToString(config.pcConfig.libraries))
            self.pcsInfo.frameworkFiles.set(listToString(config.pcConfig.frameworkFiles))       # Java Approved Framework - July 2019

            self.tssCheckBox.set(config.testTS)
            self.tssInfo.includePath.set(listToString(config.tsConfig.includePath))
            self.tssInfo.headerFiles.set(listToString(config.tsConfig.headerFiles))
            self.tssInfo.objectFiles.set(listToString(config.tsConfig.libraries))
            self.tssRegisterReadCallbackCheckBox.set(config.tsConfig.registerReadCallback)
            if constants.getFACEStandardVersion() in [constants.FACE_STANDARD_VERSION_2_1, constants.FACE_STANDARD_VERSION_2_1_1]:
                self.tssInterfaceToTestLine.var.set(config.tsConfig.interfaceUnderTest)
            self.tssInfo.frameworkFiles.set(listToString(config.tsConfig.frameworkFiles))       # Java Approved Framework - July 2019

            self.pssCheckBox.set(config.testPSS)
            self.pssInfo.includePath.set(listToString(config.pssConfig.includePath))
            self.pssInfo.headerFiles.set(listToString(config.pssConfig.headerFiles))
            self.pssInfo.objectFiles.set(listToString(config.pssConfig.libraries))
            self.pssInfo.frameworkFiles.set(listToString(config.pssConfig.frameworkFiles))       # Java Approved Framework - July 2019

            self.iosCheckBox.set(config.testIOS)
            self.iosInfo.includePath.set(listToString(config.ioConfig.includePath))
            self.iosInfo.headerFiles.set(listToString(config.ioConfig.headerFiles))
            self.iosInfo.objectFiles.set(listToString(config.ioConfig.libraries))
            self.iosInfo.frameworkFiles.set(listToString(config.ioConfig.frameworkFiles))       # Java Approved Framework - July 2019

            self.ossCheckBox.set(config.testOS)
            for testName, testValues in config.osConfig.subTests.iteritems():
                if testName == 'JAVA':
                    logger.warning('Had a JAVA subtest...not JAVA_GENERAL, JAVA_SAFETY')
                    continue # deprecated subTest
                self.ossInfo.subTests[testName].underTest.set(testValues.underTest)
                self.ossInfo.subTests[testName].objectFiles.set(pathsep.join(testValues.objectFiles))
                if testName in ['JAVA', TestConfiguration.OsSegmentConfig.SubTests.JAVA_GENERAL, TestConfiguration.OsSegmentConfig.SubTests.JAVA_SAFETY]:
                    continue # nothing else to set (JAVA is from older configurations

                self.ossInfo.subTests[testName].compilerFlags.set(testValues.compilerFlags)
                self.ossInfo.subTests[testName].linkerFlags.set(testValues.linkerFlags)
                self.ossInfo.subTests[testName].includePath.set(pathsep.join(testValues.includePath))
                if testName == TestConfiguration.OsSegmentConfig.SubTests.ARINC_653:
                    self.ossInfo.subTests[testName].headerFiles.set(pathsep.join(testValues.headerFiles))

            if config.datatypeConfig.dataModel is None:
                self.SharedDataModelLine.var.set("")
                self.UoCDataModelLine.var.set("")
            else:
                self.dataModel = config.datatypeConfig.dataModel
                self.SharedDataModelLine.var.set(self.dataModel.sharedDataModelFilename)
                self.UoCDataModelLine.var.set(self.dataModel.unitOfConformanceDataModelFilename)
                self.uopsToTest = config.datatypeConfig.uopsToTest
                self.populateUoPFrame()
                # the following line is done using the validatecommand in the Entry widget now
                #self.getDataModelUoPs(config.datatypeConfig.dataModel.unitOfConformanceDataModelFilename)

            self.notesText.delete(1.0, END)
            self.notesText.insert(END, config.baseConfig.notes)
            #self.notesText.set(
            #config.baseConfig.notes = self.notesText.get(1.0, END)

            self.configurationFilename = config.configFile

            # enable/disable widgets based on checkbox's state
            self.enableInfoWidgets()
            self._setConfigurationFilenameLabel(self.configurationFilename)

        except AttributeError as ae:

            Dialogs.ErrorMessageBox('Invalid Configuration File',
                                   'Config file incompatible with current version of the Conformance Test Suite: %s'
                                   % ae, parent=self)

    def _loadConfigurationDialog(self):
        """ Prompts user for which configuration file using a dialog window and then sets GUI based on file selected"""

        initialDir = os.path.join(ConformanceMain.getRootDir(), 'configFiles')
        if self.configurationFilename is not None and \
                'initialConfiguration' not in os.path.basename(self.configurationFilename) and \
                len(self.configurationFilename) > 0:
            initialDir = os.path.dirname(self.configurationFilename)

        configurationFile = Dialogs.osAgnosticOpenFilenameDialog(initialdir=initialDir, title='Please select configuration file',
                                                filetypes=[('config files', '.cfg'), ('all files','*')], parent = self)

        if len(configurationFile) > 0:
            self._loadConfiguration(configurationFile)

    def _loadConfiguration(self, configurationFile):
        """ loads configuration in configurationFile into the GUI

        configurationFile - filename containing the pickled instance of the TestSuite.ConformanceTest class
        """
        logger.info('configurationFile: ' + str(configurationFile))
        try:
            self.config = pickle.load( open(configurationFile, "r"))
            self.config.configFile = configurationFile
        except Exception as x:
            errorMsg = 'Could not load configuration file: [' + configurationFile + '], (most likely a deprecated format from a previous version of the test suite): ' + x.message
            logger.error(errorMsg)
            if commonTools.isInteractive():
                Dialogs.ErrorMessageBox('Conformance Test Suite', errorMsg, parent=self)
            return False

        newConfig = ConformanceTest.ConformanceTest()
        missingVars = ConformanceTest.checkConfig(newConfig, self.config)


        if len(missingVars) > 0:
            warningMessage = 'You are reading a configuration file from a previous version of the test suite ' + \
                'with missing values: \n' + '\n'.join(missingVars) + '\nYou will want to save the configuration ' + \
                'file before continuing in the new configuration file format'
            Dialogs.WarningMessageBox(constants.FACE_TESTSUITE_NAME, warningMessage, parent=self)
            logger.warning(warningMessage)

        self.config = newConfig
        self.configurationFilename = configurationFile
        self._configurationToGui(self.config)
        return True


    def _setConfigurationFilenameLabel(self, configurationFile):
        """ Sets the filename label at the top of the configuration window

        configurationFile - Name to put as the filename label
        """
        logger.info('configFile: [' + configurationFile + ']')
        logger.info('default Config File: [' + ConformanceTest.getDefaultConfigurationFile() + ']')
        if configurationFile == ConformanceTest.getDefaultConfigurationFile():
            shortenConfigFilename = ''
            self.saveButton.configure(state=DISABLED)
        else:
            slashLoc = configurationFile.rfind(sep)
            if slashLoc >= 0:
                shortenConfigFilename = configurationFile[slashLoc+len(sep):]
            else:
                shortenConfigFilename = configurationFile

            self.saveButton.configure(state=NORMAL)

        self.configurationFilenameLabel.set(shortenConfigFilename)

    def _saveConfigurationButton(self):
        logger.info('config Filename:' + self.configurationFilename)
        self._saveConfiguration(self.configurationFilename)

    def _saveConfigurationDialog(self):
        """ Prompts user for which configuration file using a dialog window and then saves configuration on GUI to file selected"""
        initialDir = os.path.join(ConformanceMain.getRootDir(), 'configFiles')
        initialFile = constants.DEFAULT_CONFIGURATION_FILENAME
        if len(self.config.configFile) > 0 and not os.path.basename(self.config.configFile).startswith('initialConfiguration'):
            initialDir, initialFile = os.path.split(self.config.configFile)

        configurationFile = Dialogs.osAgnosticSaveAsFilenameDialog(
            initialdir=initialDir, initialfile=initialFile, title='Please select configuration file',
            filetypes=[('config files', '.cfg'), ('all files','*')], parent=self)

        if configurationFile != '':
            self._saveConfiguration(configurationFile)

    def _saveConfiguration(self, configurationFile):
        """ saves configuration in GUI to configurationFile

        configurationFile - filename to contain the pickled instance of the TestSuite.ConformanceTest class
        """

        logger.info('Saving configuration file: ' + str(configurationFile))

        self.config = self._guiToConfiguration()
        self.config.configFile = configurationFile
        #self.config.datatypeConfig.dataModel.ConformanceResults = None
        #self.config.datatypeConfig.dataModel.testResults = None

        pickle.dump( self.config, commonTools.safeOpen(configurationFile, "w"))
        self._setConfigurationFilenameLabel(configurationFile)
        self.configurationFilename = configurationFile

    def _onOkButtonPress(self):
        """
        checks configuration defined in GUI for errors/insufficiencies, prompts user to make corrections.
        if configuration is good, sets config class member to the TestSuite.ConformanceTest instance, destroys the configuration window
        and returns control to the main menu.
        """

        logger.info("Pressed OK")
        # Do the okay stuff here
        config = self._guiToConfiguration()
        if config.verifyConfigurationDialog(parent=self):
            # Need to clear the warnings otherwise it may say configurations are different when they are not.
            config.baseConfig.warnings = list()
            config.pcConfig.warnings = list()
            config.tsConfig.warnings = list()
            config.pssConfig.warnings = list()
            config.ioConfig.warnings = list()
            config.osConfig.warnings = list()
            config.datatypeConfig.warnings = list()
            config.datatypeConfig.errors = list()

            if self.config != config:
                self.config.compare(config)
                saveConfig = Dialogs.YesNoMessageBox('Save Configuration?','Configuration has changed, do you want to save it to a file?', parent=self) == 'yes'
                if saveConfig:
                    self._saveConfigurationDialog()
                    if not self.parent is None:
                        self.parent.configSaved = True
                else:
                    if not self.parent is None:
                        self.parent.configSaved = False

            self.config = config
            self.destroy()
            logger.debug('Destroyed config window')
            commonTools.popParentGUI()

    def _onCancelButtonPress(self):
        """
        sets the config class member to None, destroys the configuration window, and returns control to the main menu.
        """

        logger.info( "Pressed Cancel")
        # do the cancel stuff here
        self.config = None
        self.destroy()
        commonTools.popParentGUI()


    def buttonbox(self):
        """ overridden dialog method for "ok" box at bottom of dialog """
        pass

    def createConfigIoWidgets(self):

        # Begin row 1: configuration file
        frame = Frame(self)
        frame.pack(side=TOP)

        Label(frame, text="Current Configuration File:").pack(side=LEFT)

        self.configurationFilenameLabel = StringVar()
        self.configurationFilenameLabel.set("Undefined")
        configFileFont = tkFont.Font(weight=tkFont.BOLD, slant=tkFont.ITALIC)
        Label(frame, textvariable=self.configurationFilenameLabel, font=configFileFont).pack(side=LEFT)

        # Begin row 2: buttons
        frame = Frame(self)
        frame.pack(side=TOP)

        Button(frame, text="Load", command=self._loadConfigurationDialog).pack(side=LEFT, padx=5, pady=2)
        self.saveButton = Button(frame, text="Save", command=self._saveConfigurationButton)
        self.saveButton.pack(side=LEFT, padx=5, pady=2)
        Button(frame, text="Save As", command=self._saveConfigurationDialog).pack(side=LEFT, padx=5, pady=2)
        Button(frame, text="Ok", command=self._onOkButtonPress).pack(side=LEFT, padx=5, pady=2)
        Button(frame, text="Cancel", command=self._onCancelButtonPress).pack(side=LEFT, padx=5, pady=2)

    def createSegmentInfo(self):
        segmentInfo = namedtuple('info', 'includePath headerFiles objectFiles frameworkFiles')
        segmentInfo.includePath = StringVar()
        segmentInfo.headerFiles = StringVar()
        segmentInfo.objectFiles = StringVar()
        segmentInfo.frameworkFiles = StringVar()
        return segmentInfo

    def envCmd(self):
        """Brings up a file dialog, sets results value to env"""
        envFile = Dialogs.osAgnosticOpenFilenameDialog(title="Please select environment bootstrap script", filetypes=[("Shell scripts", ".sh"), ("Batch scripts", "*.bat"), ("All files", "*")])
        if envFile != '':
            self.envLine.var.set(envFile)

    def _onPosixPartition(self):
        logger.info("_onPosixPartition")
        self.enableInfoWidgets()

    def _onArincPartition(self):
        logger.info("_onArincPartition")
        self.enableInfoWidgets()

    def _onPosixARINCPartition(self):
        logger.info("_onPosixARINCPartition")
        self.enableInfoWidgets()

if __name__ == "__main__":
    logger.info("Running from ConfigurationGUI")

    rootDir = ConformanceMain.getRootDir()
    initConfig = ConformanceTest.getDefaultConfigurationFile()
    configGUI = ConfigurationGUI(None, configFilename=initConfig)
