"""
------------------------------------------------------------------------------------
   Copyright (c) Vanderbilt University, 2020                                       
                                                                                   
   ALL RIGHTS RESERVED, UNLESS OTHERWISE STATED                                    
                                                                                   
   This software, authored by Vanderbilt University under a contract awarded to and
   managed by Alion Science and Technology, was funded by the U.S. Government under
   Contract No. FA8075-14-D-0014 and the U.S. Government has unlimited rights in   
   this software.  An unlimited rights license means that the U.S. Government can  
   use, modify, reproduce, release or disclose computer software in whole or in    
   part, in any manner, and for any purpose whatsoever, and to have or authorize   
   others to do so.                                                                
                                                                                   
   Vanderbilt University disclaims all warranties with regard to this software,    
   including all implied warranties of merchantability and fitness. In no event    
   shall Vanderbilt University be liable for any special, indirect or consequential
   damages or any damages whatsoever resulting from loss of use, data or profits,  
   whether in an action of contract, negligence or other tortious action, arising  
   out of or in connection with the use or performance of this software.           
                                                                                   
   May contain products produced under DoD SENSIAC contract HC104705D4000 under the
   sponsorship of the Defense Technical Information Center,                        
   ATTN: DTIC-AI, 8723 John J. Kingman Rd., Ste 0944, Fort Belvoir, VA 22060-6218. 
   SENSIAC is a DoD Information Analysis Center Sponsored by the Defense Technical 
   Information Center.                                                             
                                                                                   
   Conformance Test Suite Version 2.1.1r9 supporting FACE Standard Version 2.1.1   
                                                                                   
   Release Date: 29-Aug-2023                                                       
------------------------------------------------------------------------------------
"""
"""
---------------------------------------------------------------------------------
Copyright (c) Vanderbilt University, 2019

ALL RIGHTS RESERVED, UNLESS OTHERWISE STATED

This software, authored by Vanderbilt University under a contract awarded to and
managed by Alion Science and Technology, was funded by the U.S. Government under
Contract No. FA8075-14-D-0014 and the U.S. Government has unlimited rights in
this software.  An unlimited rights license means that the U.S. Government can
use, modify, reproduce, release or disclose computer software in whole or in
part, in any manner, and for any purpose whatsoever, and to have or authorize
others to do so.

Vanderbilt University disclaims all warranties with regard to this software,
including all implied warranties of merchantability and fitness. In no event
shall Vanderbilt University be liable for any special, indirect or consequential
damages or any damages whatsoever resulting from loss of use, data or profits,
whether in an action of contract, negligence or other tortious action, arising
out of or in connection with the use or performance of this software.

May contain products produced under DoD SENSIAC contract HC104705D4000 under the
sponsorship of the Defense Technical Information Center,
ATTN: DTIC-AI, 8723 John J. Kingman Rd., Ste 0944, Fort Belvoir, VA 22060-6218.
SENSIAC is a DoD Information Analysis Center Sponsored by the Defense Technical
Information Center.


@author: david.fann@vanderbilt.edu
---------------------------------------------------------------------------------
"""

__author__ = 'm_rushdy@isis.vanderbilt.edu'

import os
import sys
import ConformanceMain
import JavaConformanceTest


egg_path = os.path.join(ConformanceMain.getRootDir(), 'face_conformance_app', 'junitxml-0.7-py2.7.egg')
sys.path.append(egg_path)

import unittest
import junitxml
import fnmatch
import re


def atoi(text):
    return int(text) if text.isdigit() else text


def natural_keys(text):
    """
    alist.sort(key=natural_keys) sorts in human order
    http://nedbatchelder.com/blog/200712/human_sorting.html
    """
    return [atoi(c) for c in re.split('(\d+)', text)]


class ModifiedJUnitXmlClass(junitxml.JUnitXmlResult):

    REPLACE_CHAR = '|'

    def __init__(self, stream):
        junitxml.JUnitXmlResult.__init__(self, stream)
        self.suite_name = " "
        self.t_errors = 0
        self.t_failures = 0
        self.t_unexpected = 0
        self.t_tests_run = 0
        self.all_results = []

    def reset_results(self):
        self.t_errors += len(self.errors)
        self.t_failures += len(self.failures)
        self.t_unexpected += len(getattr(self, 'unexpectedSuccesses', ()))
        self.t_tests_run += self.testsRun
        self.errors = []
        self.failures = []
        self.unexpectedSuccesses = []
        self.testsRun = 0
        self._results = []

    def close_stream(self):
        self._stream.close()

    def stopTestRun(self):
        duration = self._duration(self._run_start)
        self.all_results.append('<testsuite name="%s" errors="%d" failures="%d" tests="%d" time="%0.3f">\n' %
                               (self.suite_name, len(self.errors), len(self.failures) + len(getattr
                               (self, "unexpectedSuccesses", ())), self.testsRun, duration))
        self.all_results.extend(self._results)
        self.all_results.append('</testsuite>\n')
        self.reset_results()

    def write_results(self):
        duration = self._duration(self._run_start)
        self._stream.write('<testsuites errors="%d" failures="%d" tests="%d" time="%0.3f">\n' %
                           (self.t_errors, self.t_failures + self.t_unexpected, self.t_tests_run, duration))
        for i in self.all_results:
            self._stream.write(i)
        self._stream.write('</testsuites>\n')

    def addSuccess(self, test):
        unittest.TestResult.addSuccess(self, test)
        self._test_case_string(test)
        if len(test.std_out.strip()) > 0:
            self.add_standard_out(test.std_out)
            self._results.append('</testcase>\n')
        else:
            self._results.append('/>\n')

    def addFailure(self, test, error):
        unittest.TestResult.addFailure(self, test, error)
        self._test_case_string(test)
        if len(test.std_out.strip()) > 0:
            self.add_standard_out(test.std_out)
        else:
            self._results.append('>\n')
        self._results.append('<failure type="%s"/>\n</testcase>\n' % error[0].__name__)

    def add_standard_out(self, std_out):
        self._results.append('>\n')
        self._results.append('<system-out>\n%s\n</system-out>\n' % ModifiedJUnitXmlClass.
                             _replace_invalid_char(std_out.strip()))

    def _test_case_string(self, test):
        duration = self._duration(self._test_start)
        test_id = test.id()
        class_divide = test_id.split(".")
        length = len(class_divide)
        if test.language.lower() == 'java':
            name = class_divide[length - 1].replace('test ', '').replace(ModifiedJUnitXmlClass.REPLACE_CHAR, '.')
        else:
            name = class_divide[length - 1].replace('test ', '')
        self._results.append('<testcase classname="%s" name="%s" time="%0.3f"' % (test.__class__.__name__ + '.' + test
                             .get_segment_description(), ModifiedJUnitXmlClass._replace_invalid_char(name), duration))

    @staticmethod
    def _replace_invalid_char(string):
        return string.replace('&', '&amp;')\
                     .replace('"', '&quot;')\
                     .replace("'", '&apos;')\
                     .replace('<', '&lt;')\
                     .replace('>', '&gt;')


class JunitXMLWriter():
    def __init__(self, conformance_test, report_directory):
        self.conf_test = conformance_test
        self.report_directory = report_directory
        TestSegment.profile = self.conf_test.baseConfig.osProfile
        TestSegment.language = self.conf_test.baseConfig.language
        TestSegment.log_directory = self.conf_test.baseConfig.logDirectory

    @staticmethod
    def remove_previous_xml_results(path):
        for filename in fnmatch.filter(os.listdir(path), '*.xml'):
            os.remove(os.path.join(path, filename))

    @staticmethod
    def get_junit_obj(results_dir):
        fp = file(results_dir, 'wb')
        result = ModifiedJUnitXmlClass(fp)
        return result

    def create_test_cases(self, result):
        extras = '.' + TestSegment.language + '.' + TestSegment.profile.capitalize()
        handler = TestSuiteHandler(result)
        if self.conf_test.testDataModel or self.conf_test.testPC or self.conf_test.testTS or self.conf_test.testPSS:
            result.suite_name = TestSegment.SEGMENT_DATA_MODEL + extras
            handler.handle(DataModel, self.conf_test.datatypeConfig.dataModel)
        if self.conf_test.testIOS:
            result.suite_name = TestSegment.SEGMENT_IOS + extras
            handler.handle(IOS, self.conf_test.ioConfig)
        if self.conf_test.testOS:
            result.suite_name = TestSegment.SEGMENT_OSS + extras
            handler.handle(OSS, self.conf_test.osConfig)
        if self.conf_test.testPC:
            result.suite_name = TestSegment.SEGMENT_PCS + extras
            handler.handle(PCS, self.conf_test.pcConfig)
        if self.conf_test.testPSS:
            result.suite_name = TestSegment.SEGMENT_PSS + extras
            handler.handle(PSS, self.conf_test.pssConfig)
        if self.conf_test.testTS:
            result.suite_name = TestSegment.SEGMENT_TSS + extras
            handler.handle(TSS, self.conf_test.tsConfig)

    def write_config_results(self):
        self.remove_previous_xml_results(self.report_directory)
        file_name = os.path.join(self.report_directory, 'JUnit_ConformanceReport.xml')
        result = self.get_junit_obj(file_name)
        self.create_test_cases(result)
        result.write_results()
        result.close_stream()


class TestSuiteHandler():
    def __init__(self, result):
        self.result = result

    @staticmethod
    def run_test_suite(result, suite):
        result.startTestRun()
        unittest.TestSuite(suite).run(result)
        result.stopTestRun()

    def handle(self, cls, config):
        # print config.libraries
        # if len(config.libraries) > 0:
        #     self.result.gold_lib_parent_dir = config.libraries[0].split(os.sep)[-2]
        # print self.result.gold_lib_parent_dir
        suite = cls.build_test_suite(config)
        if cls.language.lower() != 'java' and cls.__name__ == 'OSS':
            self.result.suite_name = 'OSS.' + TestSegment.language + '.' + OSS.oss_name + '.' \
                                     + TestSegment.profile.capitalize()
        TestSuiteHandler.run_test_suite(self.result, suite)
        TestSegment.clear_tests(cls)


class TestSegment(unittest.TestCase):
    SEGMENT_OSS = 'OSS'
    SEGMENT_IOS = 'IOS'
    SEGMENT_PSS = 'PSS'
    SEGMENT_PCS = 'PCS'
    SEGMENT_TSS = 'TSS'
    SEGMENT_DATA_MODEL = 'DataModel'

    DEFAULT_LOG_OUTPUT = 'No Log File Found'

    profile = ''
    language = ''
    log_directory = ''

    def __init__(self, std):
        self.std_out = std

    @staticmethod
    def clear_tests(cls):
        for test in unittest.TestLoader().getTestCaseNames(cls):
            delattr(cls, test)

    @staticmethod
    def generate(a, b, err):
        def result_testing(self):
            #_replace_invalid_char(_log_to_string(err))
            self.assertEqual(a, b)
        return result_testing

    @classmethod
    def build_test_suite(cls, segment_config):
        if TestSegment.language.lower() == 'java':
            suite = _java_general_test_case(segment_config, cls)
        else:
            suite = _general_test_case(segment_config, cls)
        return suite

    def get_segment_description(self):
        return TestSegment.language + "." + TestSegment.profile.capitalize()


class DataModel(TestSegment):
    def __init__(self, test_name):
        unittest.TestCase.__init__(self, test_name)
        TestSegment.__init__(self, TestSegment.DEFAULT_LOG_OUTPUT)

    @classmethod
    def build_test_suite(cls, data_model):
        suite = unittest.TestSuite()
        if data_model is not None:
            for t in data_model.testResults:
                if t.score != data_model.TEST_NOT_PERFORMED:
                    test = DataModel.generate(t.score, data_model.TEST_PASSED, '')
                    setattr(DataModel, 'test %s' % t.name, test)
                    dm = DataModel('test %s' % t.name)
                    log = os.path.join(TestSegment.log_directory, data_model.testLogFilenames[t])
                    dm.std_out = _log_to_string(log)
                    suite.addTest(dm)
        return suite


class OSS(TestSegment):
    oss_name = ''

    def __init__(self, test_name, os_name):
        unittest.TestCase.__init__(self, test_name)
        TestSegment.__init__(self, TestSegment.DEFAULT_LOG_OUTPUT)
        self.os_description = os_name

    @classmethod
    def build_test_suite(cls, segment_config):
        if TestSegment.language.lower() == 'java':
            suite = OSS.java_oss_test_suite(segment_config)
        else:
            suite = OSS.oss_test_case(segment_config)
        return suite

    @staticmethod
    def oss_test_case(seg_config):
        suite = unittest.TestSuite()
        for subTest, subTestTargets in seg_config.targets.iteritems():
            parent_dir = ''
            if len(seg_config.subTests[subTest].objectFiles) > 0:
                sep_object_dir = seg_config.subTests[subTest].objectFiles[-1].split(os.sep)
                if len(sep_object_dir) > 1:
                    parent_dir = '.' + sep_object_dir[-2] + '_' + sep_object_dir[-1].replace('.a', '')
                OSS.oss_name = subTest + parent_dir
            for t in sorted(subTestTargets, key=natural_keys):
                name = seg_config.testAssertions[subTest][t].replace('Testing definition of', '').strip(' \t\n\r')
                test = t.split('.')[0].replace(OSS.profile + os.sep, '').replace('Pass' + os.sep, '') + ' - ' + name
                log = seg_config.targetLogfile[subTest][t]
                setattr(OSS, 'test %s' % test, OSS.generate(seg_config.testResult[subTest][t], 0, log))
                oss = OSS('test %s' % test, subTest + parent_dir)
                oss.std_out = _log_to_string(log)
                suite.addTest(oss)
        return suite

    @staticmethod
    def java_oss_test_suite(seg_config):
        suite = unittest.TestSuite()
        for s in seg_config.java_conformance_test_results.java_package_tree.root_node.children:
            suite.addTests(OSS.java_package_suite(s))
        return suite

    @staticmethod
    def java_package_suite(java_package):
        suite = unittest.TestSuite()
        if java_package.subgroup is not None:
            j_package_list = java_package.full_name.split('.', 1)
            if len(j_package_list) > 1:
                first = j_package_list[0]
                sub_full = j_package_list[1]
                for s in java_package.subgroup.subgroups:
                    for r in s.results:
                        test_name = (sub_full + " - " + r.name.replace(' existence test', ''))\
                            .replace('.', ModifiedJUnitXmlClass.REPLACE_CHAR)
                        test = OSS.generate(r.score, JavaConformanceTest.JavaConformanceTestResult.PASS, '')
                        setattr(OSS, 'test %s' % test_name, test)
                        cls_instance = OSS('test %s' % test_name, first + '.' + sub_full.replace('.', ' '))
                        cls_instance.std_out = r.description + '\n' + _get_java_error_description(r.logs)
                        suite.addTest(cls_instance)
        for s in java_package.children:
            suite.addTests(OSS.java_package_suite(s))
        return suite

    def get_segment_description(self):
        if TestSegment.language.lower() != 'java':
            return TestSegment.language + "." + self.os_description + "." + TestSegment.profile.capitalize()
        else:
            return self.os_description


def _general_test_case(segment_config, cls):
    suite = unittest.TestSuite()
    for target in sorted(segment_config.targets, key=natural_keys):
        if target not in segment_config.testResult:
            continue
        if target not in segment_config.testAssertion:
            test_name = target
        else:
            test_assert = segment_config.testAssertion[target].replace('Testing', '').replace(' is conformant', '')
            test_name = target + ' - ' + test_assert.replace(' for conformance', '').strip(' \t\n\r')
        log = segment_config.targetLogfile[target]
        test = cls.generate(segment_config.testResult[target], 0, log)
        setattr(cls, 'test %s' % test_name, test)
        cls_instance = cls('test %s' % test_name)
        cls_instance.std_out = _log_to_string(log)
        suite.addTest(cls_instance)
    return suite


def _java_general_test_case(segment_config, cls):
    suite = unittest.TestSuite()
    for java_subgroup in segment_config.java_conformance_test_results.subgroups:
        suite.addTests(_java_subgroup(java_subgroup, cls))
    return suite


def _java_subgroup(java_subgroup, cls):
    suite = unittest.TestSuite()
    for r in java_subgroup.results:
        test_name = str(java_subgroup.name) + ' - ' + r.name.replace(" ", "")
        test = cls.generate(r.score, JavaConformanceTest.JavaConformanceTestResult.PASS, r.description)
        setattr(cls, 'test %s' % test_name, test)
        cls_instance = cls('test %s' % test_name)
        cls_instance.std_out = r.description + '\n' + _get_java_error_description(r.logs)
        suite.addTest(cls_instance)
    for s in java_subgroup.subgroups:
        suite.addTests(_java_subgroup(s, cls))
    return suite


def _get_java_error_description(logs):
    warning_error_dict = {JavaConformanceTest.JavaConformanceTestResult.WARNING: 'Warning: ',
                          JavaConformanceTest.JavaConformanceTestResult.ERROR: 'Error: '}
    description = ''
    for l in logs:
        description += warning_error_dict[l.level] + l.text + "\n"
    return description


def _log_to_string(log):
    if os.path.exists(log):
        f = open(log, 'r')
        data = f.read()
        f.close()
        return '\n' + data
    else:
        print 'FILE DOES NOT EXIST: ' + log
        return 'No Log File Available'


class IOS(TestSegment):
    def __init__(self, test_name):
        unittest.TestCase.__init__(self, test_name)
        TestSegment.__init__(self, TestSegment.DEFAULT_LOG_OUTPUT)

    @classmethod
    def build_test_suite(cls, segment_config):
        if TestSegment.language.lower() == 'java':
            suite = _java_general_test_case(segment_config, IOS)
        else:
            suite = _general_test_case(segment_config, IOS)
        return suite


class PCS(TestSegment):
    def __init__(self, test_name):
        unittest.TestCase.__init__(self, test_name)
        TestSegment.__init__(self, TestSegment.DEFAULT_LOG_OUTPUT)

    # @classmethod
    # def build_test_suite(cls, segment_config):
    #     if TestSegment.language.lower() == 'java':
    #         suite = _java_test_case(segment_config, PCS)
    #     else:
    #         suite = _test_case(segment_config, PCS)
    #     return suite


class PSS(TestSegment):
    def __init__(self, test_name):
        unittest.TestCase.__init__(self, test_name)
        TestSegment.__init__(self, TestSegment.DEFAULT_LOG_OUTPUT)

    @classmethod
    def build_test_suite(cls, segment_config):
        if TestSegment.language.lower() == 'java':
            suite = _java_general_test_case(segment_config, PSS)
        else:
            suite = _general_test_case(segment_config, PSS)
        return suite


class TSS(TestSegment):
    def __init__(self, test_name):
        unittest.TestCase.__init__(self, test_name)
        TestSegment.__init__(self, TestSegment.DEFAULT_LOG_OUTPUT)

    @classmethod
    def build_test_suite(cls, segment_config):
        if TestSegment.language.lower() == 'java':
            suite = _java_general_test_case(segment_config, TSS)
        else:
            suite = _general_test_case(segment_config, TSS)
        return suite
