#!/usr/bin/env python
"""
------------------------------------------------------------------------------------
   Copyright (c) Vanderbilt University, 2020                                       
                                                                                   
   ALL RIGHTS RESERVED, UNLESS OTHERWISE STATED                                    
                                                                                   
   This software, authored by Vanderbilt University under a contract awarded to and
   managed by Alion Science and Technology, was funded by the U.S. Government under
   Contract No. FA8075-14-D-0014 and the U.S. Government has unlimited rights in   
   this software.  An unlimited rights license means that the U.S. Government can  
   use, modify, reproduce, release or disclose computer software in whole or in    
   part, in any manner, and for any purpose whatsoever, and to have or authorize   
   others to do so.                                                                
                                                                                   
   Vanderbilt University disclaims all warranties with regard to this software,    
   including all implied warranties of merchantability and fitness. In no event    
   shall Vanderbilt University be liable for any special, indirect or consequential
   damages or any damages whatsoever resulting from loss of use, data or profits,  
   whether in an action of contract, negligence or other tortious action, arising  
   out of or in connection with the use or performance of this software.           
                                                                                   
   May contain products produced under DoD SENSIAC contract HC104705D4000 under the
   sponsorship of the Defense Technical Information Center,                        
   ATTN: DTIC-AI, 8723 John J. Kingman Rd., Ste 0944, Fort Belvoir, VA 22060-6218. 
   SENSIAC is a DoD Information Analysis Center Sponsored by the Defense Technical 
   Information Center.                                                             
                                                                                   
   Conformance Test Suite Version 2.1.1r9 supporting FACE Standard Version 2.1.1   
                                                                                   
   Release Date: 29-Aug-2023                                                       
------------------------------------------------------------------------------------
"""

__author__ = 'david.fann@vanderbilt.edu'

import os

import constants
import TestConfiguration
import ConformanceMain
import glob


languages = TestConfiguration.Language.allLanguages
root_dir = ConformanceMain.getRootDir()

face_dm_or_FACE_DM = os.path.join('face', 'dm')
if constants.getFACEStandardVersion() in [constants.FACE_STANDARD_VERSION_2_1, constants.FACE_STANDARD_VERSION_2_1_1]:
    face_dm_or_FACE_DM = os.path.join('FACE', 'DM')


#TODO:  Fix this for 2.0 Conformance Test
tss_type_specific_files = {
    TestConfiguration.Language.C:   [os.path.join(root_dir,
                                                  constants.GOLD_LIB_ROOT,
                                                  TestConfiguration.Language.C,
                                                  'src', 'TSS', 'TypeSpecific_TS.c'),
                                     os.path.join(root_dir,
                                                  constants.GOLD_LIB_ROOT,
                                                  TestConfiguration.Language.C,
                                                  'include', 'FACE', 'TypeSpecific_TS.h')

                                     ],
    TestConfiguration.Language.CPP: [os.path.join(root_dir,
                                                  constants.GOLD_LIB_ROOT,
                                                  TestConfiguration.Language.CPP,
                                                  'src', 'TSS', 'TypeSpecific_TS.cpp'),
                                     os.path.join(root_dir,
                                                  constants.GOLD_LIB_ROOT,
                                                  TestConfiguration.Language.CPP,
                                                  'include', 'FACE', 'TypeSpecific_TS.hpp')

                                     ],
    TestConfiguration.Language.Ada: [os.path.join(root_dir,
                                                  constants.GOLD_LIB_ROOT,
                                                  TestConfiguration.Language.Ada,
                                                  'src', 'TSS', 'face-ts-ts.adb.TYPE_SPECIFIC'),
                                     os.path.join(root_dir,
                                                  constants.GOLD_LIB_ROOT,
                                                  TestConfiguration.Language.CPP,
                                                  'spec', 'FACE', 'face-ts-ts.ads.TYPE_SPECIFIC')

                                     ],

    TestConfiguration.Language.Java: [os.path.join(root_dir,
                                                  constants.GOLD_LIB_ROOT,
                                                  TestConfiguration.Language.Java,
                                                  'tss.javagold'),
                                      os.path.join(root_dir,
                                                  constants.GOLD_LIB_ROOT,
                                                  TestConfiguration.Language.Java,
                                                  'tss.javagoldtext')
                                      ]
}


for l in languages:
    print 'Deleting ' + l + ' Type Specific Files'
    # Delete any gold standard data model data type files out there
    include_or_spec = 'include'
    if l is TestConfiguration.Language.Ada:
        include_or_spec = 'spec'
    if l is TestConfiguration.Language.Java:
        include_or_spec = ''

    dm_directory = os.path.join(root_dir, constants.GOLD_LIB_ROOT, l, include_or_spec, face_dm_or_FACE_DM)

    if os.path.isdir(dm_directory):
        dm_files = os.listdir(dm_directory)
        for f in dm_files:
            os.remove(os.path.join(dm_directory, f))

    # Delete gold standard TypeSpecific TSS files
    for f in tss_type_specific_files[l]:
        if os.path.isfile(f):
            os.remove(f)

    # Delete conformance test TypeSpecific files

    # Java doesn't have anything to delete (was already done in the gold lib section)
    if l is TestConfiguration.Language.Java:
        continue

    conformance_test_dir = os.path.join(root_dir, constants.CONFORMANCE_TEST_ROOT, l, 'TSS', 'FACE_TS')
    conformance_type_specific_files = glob.glob(os.path.join(conformance_test_dir, 'Test_*'))
    for c in conformance_type_specific_files:
        os.remove(c)

