"""
------------------------------------------------------------------------------------
   Copyright (c) Vanderbilt University, 2020                                       
                                                                                   
   ALL RIGHTS RESERVED, UNLESS OTHERWISE STATED                                    
                                                                                   
   This software, authored by Vanderbilt University under a contract awarded to and
   managed by Alion Science and Technology, was funded by the U.S. Government under
   Contract No. FA8075-14-D-0014 and the U.S. Government has unlimited rights in   
   this software.  An unlimited rights license means that the U.S. Government can  
   use, modify, reproduce, release or disclose computer software in whole or in    
   part, in any manner, and for any purpose whatsoever, and to have or authorize   
   others to do so.                                                                
                                                                                   
   Vanderbilt University disclaims all warranties with regard to this software,    
   including all implied warranties of merchantability and fitness. In no event    
   shall Vanderbilt University be liable for any special, indirect or consequential
   damages or any damages whatsoever resulting from loss of use, data or profits,  
   whether in an action of contract, negligence or other tortious action, arising  
   out of or in connection with the use or performance of this software.           
                                                                                   
   May contain products produced under DoD SENSIAC contract HC104705D4000 under the
   sponsorship of the Defense Technical Information Center,                        
   ATTN: DTIC-AI, 8723 John J. Kingman Rd., Ste 0944, Fort Belvoir, VA 22060-6218. 
   SENSIAC is a DoD Information Analysis Center Sponsored by the Defense Technical 
   Information Center.                                                             
                                                                                   
   Conformance Test Suite Version 2.1.1r9 supporting FACE Standard Version 2.1.1   
                                                                                   
   Release Date: 29-Aug-2023                                                       
------------------------------------------------------------------------------------
"""
"""
-------------------------------------------------------------------------------------
Copyright (c) Vanderbilt University, 2019

ALL RIGHTS RESERVED, UNLESS OTHERWISE STATED

This software, authored by Vanderbilt University under a contract awarded to and
managed by Alion Science and Technology, was funded by the U.S. Government under
Contract No. FA8075-14-D-0014 and the U.S. Government has unlimited rights in
this software.  An unlimited rights license means that the U.S. Government can
use, modify, reproduce, release or disclose computer software in whole or in
part, in any manner, and for any purpose whatsoever, and to have or authorize
others to do so.

Vanderbilt University disclaims all warranties with regard to this software,
including all implied warranties of merchantability and fitness. In no event
shall Vanderbilt University be liable for any special, indirect or consequential
damages or any damages whatsoever resulting from loss of use, data or profits,
whether in an action of contract, negligence or other tortious action, arising
out of or in connection with the use or performance of this software.

May contain products produced under DoD SENSIAC contract HC104705D4000 under the
sponsorship of the Defense Technical Information Center,
ATTN: DTIC-AI, 8723 John J. Kingman Rd., Ste 0944, Fort Belvoir, VA 22060-6218.
SENSIAC is a DoD Information Analysis Center Sponsored by the Defense Technical
Information Center.

-------------------------------------------------------------------------------------
"""
"""

face_specified_code_generator.py generates all conformance test and gold standard library
files that are related to the data model

"""
__author__ = 'david.fann@vanderbilt.edu'

import os.path
import logging
import sys
import glob

from ConformanceMain import getRootDir, initializeLogger
if __name__ == "__main__":
    initializeLogger()

egg_path = os.path.join(getRootDir(), 'face_conformance_app', 'stringtemplate3-3.1-py2.7.egg')
sys.path.append(egg_path)

import stringtemplate3

from TestConfiguration import Language, TsSegmentConfig
import constants
import commonTools
import Dialogs

logger = logging.getLogger(__name__)

_face_specified_templates = {
    Language.C: 'TSS_C_DataModelTemplates.stg',
    Language.CPP: 'TSS_C++_DataModelTemplates.stg',
    Language.Ada: 'TSS_Ada_DataModelTemplates.stg'
}


def get_include_directory(language):

    include_dir = os.path.join(getRootDir(), constants.GOLD_LIB_ROOT, language)

    if language in [Language.C, Language.CPP]:
        include_dir = os.path.join(include_dir, "include")
    elif language == Language.Ada:
        include_dir = os.path.join(include_dir, "spec")

    return include_dir


def get_datatype_include_directory(language):
    include_dir = get_include_directory(language)
    dm_include_dir = os.path.join(include_dir, "face", "dm")
    if language == Language.Ada:
            dm_include_dir = os.path.join(include_dir, "face")

    if constants.getFACEStandardVersion() in [constants.FACE_STANDARD_VERSION_2_1, constants.FACE_STANDARD_VERSION_2_1_1]:
        dm_include_dir = os.path.join(include_dir, "FACE", "DM")
        if language == Language.Ada:
            dm_include_dir = os.path.join(include_dir, "FACE")

    return dm_include_dir


def get_gold_source_directory(language):
    return os.path.join(getRootDir(), constants.GOLD_LIB_ROOT, language, 'src', 'TSS')

def get_conformance_directory(language, interface_under_test=None):
    if constants.getFACEStandardVersion() == constants.FACE_STANDARD_VERSION_2_0:
        return os.path.join(getRootDir(), constants.CONFORMANCE_TEST_ROOT, language, 'TSS')
    else:
        valid_interfaces_to_test = [TsSegmentConfig.InterfaceUnderTest.TS_INTERFACE_UoP,
                                    TsSegmentConfig.InterfaceUnderTest.ABSTRACTION_LAYER]

        if not interface_under_test in valid_interfaces_to_test:
            error_msg = 'Interface under test must be one of ' + \
                        ', '.join(valid_interfaces_to_test) + ' got ' + interface_under_test
            logger.error(error_msg)
            raise KeyError(error_msg)

        return os.path.join(getRootDir(), constants.CONFORMANCE_TEST_ROOT, language, 'TSS',
                            TsSegmentConfig.InterfaceUnderTest.interface_test_subdirectory[interface_under_test])


def generate_data_model_data_type_definitions(language, data_model_filename, log_dir):

        datatype_jar_dir = os.path.join(getRootDir(), constants.DM_DATATYPE_GEN_SUBDIR)

        generate_dir = get_include_directory(language)
        if language == Language.Ada:
            generate_dir = get_datatype_include_directory(language)



        language_option_dict = {Language.C: ' -c -o "',
                                Language.CPP: ' -p -o "',
                                Language.Ada: ' -a -o "',
                                Language.Java: ' -j -o "'}

        generate_data_type_definitions_command = \
            'java -jar "' + constants.DM_DATATYPE_GEN_JAR + '"' + language_option_dict[language] + \
            generate_dir + '" -i "' + data_model_filename + '"'

        results = commonTools.subprocessWithSpecifiedPath(generate_data_type_definitions_command, "", datatype_jar_dir)
        commonTools.logSubprocessResults(generate_data_type_definitions_command, results,
                            os.path.join(log_dir, constants.DM_DATATYPE_GEN_JAR + '_results.log'))
        logger.info('command: ' + generate_data_type_definitions_command + '\nreturned code: ' + str(results.returncode)
                    + '\nstd out: ' + results.stdout + '\nstd err: ' + results.stderr)
        if results.returncode != 0:
            error_msg = 'Error in generating data type definitions\nStd Out: ' + \
                       results.stdout + '\nStd Err: ' + results.stderr
            logger.error(error_msg)
            if commonTools.isInteractive():
                Dialogs.ErrorMessageBox(constants.FACE_TESTSUITE_NAME, error_msg, parent=commonTools.getParentGUI())
            raise RuntimeError(error_msg)


def generate_face_specified_gold_standard_library_code(language, data_model_filename, datatype_list, log_dir):

    if not language in Language.allLanguages:
        error_msg = 'language must be in ' + ', '.join(Language.allLanguages) + ' got ' + language
        logger.error(error_msg)
        raise KeyError(error_msg)

    template_filename = os.path.join(getRootDir(), constants.DATAFILES_DIR, _face_specified_templates[language])
    gold_lib_filename = {
        Language.C: 'TypeSpecific_TS',
        Language.CPP: 'TypeSpecific_TS',
        Language.Ada: 'face-ts-ts'
    }
    if constants.getFACEStandardVersion() == constants.FACE_STANDARD_VERSION_2_0:
        gold_lib_filename[Language.Ada] = 'face-ts'
    _delete_face_specified_gold_standard_library_code(language)

    generate_data_model_data_type_definitions(language, data_model_filename, log_dir)
    logger.debug('face_specified_template filename: {0}'.format(template_filename))
    face_specified_template = stringtemplate3.StringTemplateGroup(
            name='face_specified', file=open(template_filename, 'r'))

    face_or_FACE = 'face'
    if constants.getFACEStandardVersion() in [constants.FACE_STANDARD_VERSION_2_1, constants.FACE_STANDARD_VERSION_2_1_1]:
        face_or_FACE = 'FACE'


    spec_dir = os.path.join(get_include_directory(language), face_or_FACE)
    declaration_filename = os.path.join(spec_dir, gold_lib_filename[language] + Language.header_suffix[language])

    imple_dir = get_gold_source_directory(language)
    implementation_filename = os.path.join(imple_dir,
                                           gold_lib_filename[language] + Language.source_suffix[language])


    declaration_template = face_specified_template.getInstanceOf('goldLibHeaderFile')
    implementation_template = face_specified_template.getInstanceOf('goldLibSourceFile')

    unique_id = commonTools.getHeaderDefineConstant(os.path.relpath(declaration_filename, getRootDir()))
    declaration_template['UNIQUE_ID'] = unique_id

    implementation_template['FACE_SPECIFIED_TYPE_HEADER'] = face_or_FACE + '/' + os.path.basename(declaration_filename)

    read_callback_template = None
    if language == Language.Ada:
        read_callback_template = face_specified_template.getInstanceOf('readCallbackDeclarations')

    for datatype in datatype_list:

        header_template = face_specified_template.getInstanceOf('faceSpecifiedHeader')
        header_template['FACE_SPECIFIED_TYPE'] = datatype
        declaration_template['HEADER'] = str(header_template)

        signature_template = face_specified_template.getInstanceOf('faceSpecifiedDeclarations')
        signature_template['FACE_SPECIFIED_TYPE'] = datatype

        declaration_template['BODY'] = str(signature_template)
        if not read_callback_template is None:
            send_event_template = face_specified_template.getInstanceOf('sendEventDeclaration')
            send_event_template['FACE_SPECIFIED_TYPE'] = datatype
            read_callback_template['SEND_EVENT_DECLARATION'] = str(send_event_template)

        function_implementation_template = face_specified_template.getInstanceOf('faceSpecifiedImplementations')
        function_implementation_template['FACE_SPECIFIED_TYPE'] = datatype

        implementation_template['BODY'] = str(function_implementation_template)

    if not read_callback_template is None:
        declaration_template['HEADER'] = str(read_callback_template)

    type_specific_suffix = ''
    if language == Language.Ada:
        type_specific_suffix = '.TYPE_SPECIFIC'

    declaration_file = open(declaration_filename + type_specific_suffix, 'w')
    declaration_file.write(str(declaration_template))
    declaration_file.close()

    implementation_file = open(implementation_filename + type_specific_suffix, 'w')
    implementation_file.write(str(implementation_template))
    implementation_file.close()

    if language == Language.Ada:
        # NEED TO COMBINE THE TYPE INDEPENDENT WITH TYPE SPECIFIC TO MAKE ONE FILE
        ts_ts_spec_filename = declaration_filename
        ts_ts_body_filename = implementation_filename

        type_independent_spec_filename = ts_ts_spec_filename + '.TYPE_INDEPENDENT'
        type_independent_body_filename = ts_ts_body_filename + '.TYPE_INDEPENDENT'

        type_independent_spec_file = open(type_independent_spec_filename, 'r')
        ts_ts_spec_file = open(ts_ts_spec_filename, 'w')

        end_line = 'end FACE.TS.TS;'
        if constants.getFACEStandardVersion() == constants.FACE_STANDARD_VERSION_2_0:
            end_line = 'end FACE.TS;'


        for line in type_independent_spec_file:
            if line.find(end_line) != -1:
                break
            ts_ts_spec_file.write(line)

        type_independent_spec_file.close()
        ts_ts_spec_file.write(str(declaration_template) + '\n' + end_line + '\n')
        ts_ts_spec_file.close()

        type_independent_body_file = open(type_independent_body_filename, 'r')
        ts_ts_body_file = open(ts_ts_body_filename, 'w')

        for line in type_independent_body_file:
            if line.find(end_line) != -1:
                break
            ts_ts_body_file.write(line)

        type_independent_body_file.close()
        ts_ts_body_file.write(str(implementation_template) + '\n' + end_line + '\n')
        ts_ts_body_file.close()



def _delete_face_specified_gold_standard_library_code(language):

    # Need to delete the datatype generator files...the tss files will get overwritten
    # in the generate_gold... method

    subdirectory = get_datatype_include_directory(language)

    if language in [Language.C, Language.CPP]:
        old_gold_files = glob.glob(os.path.join(subdirectory, '*'))
        for o in old_gold_files:
            os.remove(o)



def generate_face_specified_conformance_test_code(language, datatype_list, interface_to_test=None):

    if not language in Language.allLanguages:
        error_msg = 'language must be in ' + ', '.join(Language.allLanguages) + ' got ' + language
        logger.error(error_msg)
        raise KeyError(error_msg)

    if constants.getFACEStandardVersion() in [constants.FACE_STANDARD_VERSION_2_1, constants.FACE_STANDARD_VERSION_2_1_1] and interface_to_test is None:
        error_msg = 'Must specify an TS Interface'
        logger.error(error_msg)
        raise RuntimeError(error_msg)

    template_filename = os.path.join(getRootDir(), constants.DATAFILES_DIR, _face_specified_templates[language])

    _delete_face_specified_conformance_test_code(language, interface_to_test)

    face_specified_template = stringtemplate3.StringTemplateGroup(
            name='face_specified', file=open(template_filename, 'r'))

    conformance_test_templates = {
        'receive_message': 'receiveMessageConformanceTestSourceFile',
        'send_message': 'sendMessageConformanceTestSourceFile',
        'register_callback': 'registerCallbackConformanceTestSourceFile'
    }

    for datatype in datatype_list:

        for test_name, template_name  in conformance_test_templates.iteritems():

            conformance_test_template = face_specified_template.getInstanceOf(template_name)

            conformance_test_template['FACE_SPECIFIED_TYPE'] = datatype
            template_result  = str(conformance_test_template)
            test_prefix = 'Test_'
            if language == Language.Ada:
                test_prefix = 'test_'
            test_filename = os.path.join(get_conformance_directory(language, interface_to_test),
                                         test_prefix + test_name + '_' +  datatype + Language.source_suffix[language])

            test_file = open(test_filename, 'w')
            test_file.write(str(conformance_test_template))
            test_file.close()



def _delete_face_specified_conformance_test_code(language, interface_to_test=None):

    # Need to delete the datatype generated TSS test files
    subdirectory = get_conformance_directory(language, interface_to_test)

    # only the datatype generated files start with 'Test_'
    old_conformance_files = glob.glob(os.path.join(subdirectory, 'Test_*'))
    for o in old_conformance_files:
        os.remove(o)

if __name__ == "__main__":
    logger.debug("Running from " + __file__)

    data_type_list = ['data_type1', 'data_type2']

    languages = Language.allLanguages

    languages = [Language.Ada]

    log_dir = os.path.join(getRootDir(), 'logFiles')
    data_model_filename = os.path.join(getRootDir(), 'sample', 'usm.sample_2.1.face')
    for lang in languages:
        generate_face_specified_gold_standard_library_code(lang, data_model_filename, data_type_list, log_dir)

        generate_face_specified_conformance_test_code(
            lang, data_type_list, TsSegmentConfig.InterfaceUnderTest.TS_INTERFACE_UoP)

