// Product produced under DoD SENSIAC contract HC104705D4000 under
// the sponsorship of the Defense Technical Information Center,
// ATTN: DTIC-AI, 8723 John J. Kingman Rd., Ste 0944, Fort Belvoir, VA
// 22060-6218.  SENSIAC is a DoD Information Analysis Center Sponsored
// by the Defense Technical Information Center.
//
// HANDLING AND DESTRUCTION NOTICE - Comply with distribution statement
// and destroy by any method that will prevent disclosure of
// contents or reconstruction of the document.

/*
 * fixed.h
 *
 *  A Fixed-point Decimal Class.
 *  Created on: Mar 21, 2013
 *  Updated: June 12, 2013
 *      Author: Daniel Ogburn (daniel.ogburn@gtri.gatech.edu)
 */

#ifndef FIXED_H_
#define FIXED_H_
#include<math.h>
#include <stdio.h>
#include<string>
#include<algorithm>
#include "bigint.hpp"
#include <face/types.hpp>

namespace FACE
{

class Fixed
{
    public:
/*****************************
 *         Constructors      *
 *****************************/
        Fixed(FACE::Long val = 0)
        {
			Value = val;
			Scale = 0; // scale always 0 for integral types
			
			// calculate digits
			Digits = 0;
			while (val > 0)
			{
				val /= 10;
				Digits++;
			}
			
            MaxDigits = 31;
            myBitPrecision = calculateBitsNeeded(bigint::powBIGINT((bigint)10,(bigint)MaxDigits))+1;
        }

        Fixed(FACE::UnsignedLong val)
        {
			Value = val;
			Scale = 0; // scale always 0 for integral types
			
			// calculate digits
			Digits = 0;
			while (val > 0)
			{
				val /= 10;
				Digits++;
			}
			
            MaxDigits = 31;
            myBitPrecision = calculateBitsNeeded(bigint::powBIGINT((bigint)10,(bigint)MaxDigits))+1;
        }


        Fixed(FACE::LongLong val )
        {
			Value = val;
			Scale = 0; // scale always 0 for integral types
			
			// calculate digits
			Digits = 0;
			while (val > 0)
			{
				val /= 10;
				Digits++;
			}
			
            MaxDigits = 31;
            myBitPrecision = calculateBitsNeeded(bigint::powBIGINT((bigint)10,(bigint)MaxDigits))+1;
        }

        Fixed(FACE::UnsignedLongLong val)
        {
			Value = val;
			Scale = 0; // scale always 0 for integral types
			
			// calculate digits
			Digits = 0;
			if (val < 0)
				val *= -1;
			while (val > 0)
			{
				val /= 10;
				Digits++;
			}
			
            MaxDigits = 31;
            myBitPrecision = calculateBitsNeeded(bigint::powBIGINT((bigint)10,(bigint)MaxDigits))+1;
        }
		
        Fixed(FACE::Double val)
        {
			// extract integral and fractional parts
			int iPart = (int)val;
			FACE::Double fPart = val - (FACE::Double)iPart;
			
			// calculate digits and scale
			int Digits = 0;
			int Scale = 0;
			if (iPart < 0)
				iPart *= -1;
			// integral contribution to digits
			while (iPart > 1)
			{
				iPart /= 10;
				Digits++;
			}
			// fractional contribution to digits and scale
			while (fPart - (FACE::Double)((int)fPart) != 0)
			{
				fPart *= 10;
				Digits++;
				Scale++;
			}
        }
        Fixed(FACE::LongDouble val)
        {
			// extract integral and fractional parts
			int iPart = (int)val;
			FACE::LongDouble fPart = val - (FACE::LongDouble)iPart;
			
			// calculate digits and scale
			int Digits = 0;
			int Scale = 0;
			if (iPart < 0)
				iPart *= -1;
			// integral contribution to digits
			while (iPart > 1)
			{
				iPart /= 10;
				Digits++;
			}
			// fractional contribution to digits and scale
			while (fPart - (FACE::LongDouble)((int)fPart) != 0)
			{
				fPart *= 10;
				Digits++;
				Scale++;
			}
        }
        Fixed(const Fixed& aFixed)
        {

            Digits = aFixed.Digits;
            Scale = aFixed.Scale;
            Value = aFixed.Value;
        }

        Fixed(const char* CSTRval)
        {
            InitalizeFromString(CSTRval);
        }
        ~Fixed()
        {
            ;
        }
/**************************
 *  Conversion Operators  *
 **************************/

//        operator LongLong() const
//        {
//           return (Value);
//        }
//        operator LongDouble() const
//        {
//            return (Value);
//        }

/******************************************
 *             round                      *
 ******************************************/

        Fixed round(FACE::UnsignedShort newScale) const
        {
            Fixed newFixed;
            bigint newValue;

            if(newScale < Scale)
            {
                newValue = Value / bigint::powBIGINT((bigint)10,(bigint)(Scale-newScale));
                bigint remainder = (Value % bigint::powBIGINT((bigint)10,(bigint)(Scale-newScale))) / bigint::powBIGINT((bigint)10,(bigint)Scale-newScale-1);


                if(remainder >=5 )
                {
                    if(newValue >= 0)
                    {
                      newValue++;
                    }
                    else
                    {
                      newValue--;
                    }
                }

                newFixed.Scale = newScale;
                newFixed.Value = newValue;
                newFixed.Digits = Digits;
            }
            else
            {
                newFixed = *this;
            }

            return (newFixed);

        }
/********************************************
 *            truncate                      *
 ********************************************/
        Fixed truncate(FACE::UnsignedShort newScale) const
        {
            Fixed newFixed;
            bigint newValue;

           if(newScale !=Scale)
           {

               newValue = scaleValue(newScale);


               newFixed.Scale = newScale;
               newFixed.Value = newValue;
               newFixed.Digits = Digits;

           }
           else
           {
              newFixed = *this;
           }

               return (newFixed);

         }
		 
		 
		 
/******************************************
 * to_string() - currently unimplemented  *
 ******************************************/

 
/******************************************
 *                Operators               *
 ******************************************/
        Fixed& operator=(const Fixed& aFixed)
        {
            if(this != &aFixed)
            {
				Digits = aFixed.Digits;
				Scale = aFixed.Scale;
				Value = aFixed.Value;
            }

            return (*this);

        }

//////////////////////////////////////////////////////////////////

        Fixed& operator+=(const Fixed& aFixed)
        {
            *this = *this + aFixed;

            return (*this);
        }

/////////////////////////////////////////////////////////////////

        Fixed& operator-=(const Fixed& aFixed)
        {
            *this = *this - aFixed;

            return (*this);
        }

//////////////////////////////////////////////////////////////////

        Fixed& operator*=(const Fixed& aFixed)
        {
            *this = *this * aFixed;

            return (*this);
        }

///////////////////////////////////////////////////////////////////

        Fixed& operator/=(const Fixed& aFixed)
        {
            *this = *this / aFixed;

             return (*this);
        }

////////////////////////////////////////////////////////////////////

        Fixed& operator++()
        {
            Value++;
            return (*this);
        }
////////////////////////////////////////////////////////////////////

        Fixed& operator++(int)
        {
            ++Value;
            return (*this);
        }

/////////////////////////////////////////////////////////////////////

        Fixed& operator--()
        {
            Value--;
            return (*this);
        }
//////////////////////////////////////////////////////////////////////

        Fixed& operator--(int)
        {
            --Value;

            return (*this);
        }
///////////////////////////////////////////////////////////////////////

        Fixed operator+() const;

///////////////////////////////////////////////////////////////////////

        Fixed operator-() const;

///////////////////////////////////////////////////////////////////////

        FACE::Boolean operator!() const
        {
            return (!(this->Value == 0));
        }

///////////////////////////////////////////////////////////////////////

        // Other member functions
        FACE::UnsignedShort fixed_digits() const
        {
           return (Digits);
        }

///////////////////////////////////////////////////////////////////////

        FACE::UnsignedShort fixed_scale() const
        {
            return (Scale);
        }

///////////////////////////////////////////////////////////////////////////////

       friend Fixed operator + (const Fixed& val1, const Fixed& val2);
       friend Fixed operator - (const Fixed& val1, const Fixed& val2);
       friend Fixed operator * (const Fixed& val1, const Fixed& val2);
       friend Fixed operator / (const Fixed& val1, const Fixed& val2);

       friend FACE::Boolean operator < (const Fixed& val1, const Fixed& val2);
       friend FACE::Boolean operator > (const Fixed& val1, const Fixed& val2);
       friend FACE::Boolean operator >= (const Fixed& val1, const Fixed& val2);
       friend FACE::Boolean operator <= (const Fixed& val1, const Fixed& val2);
       friend FACE::Boolean operator == (const Fixed& val1, const Fixed& val2);
       friend FACE::Boolean operator != (const Fixed& val1, const Fixed& val2);
////////////////////////////////////////////////////////////////////////////////


    private:

        //internally used constructor
        Fixed(FACE::Long someDigits, FACE::Long aScale)
        {
            Value = 0;
            Digits = someDigits;
            Scale = aScale;
        }

/******************************************
 *             InitalizeFromString        *
 ******************************************/
        // all public constructors convert the value passed in to a string using
        // InitalizeFromString
        void InitalizeFromString(const char* CSTRval)
        {
            std::string Stringval = CSTRval;

            FACE::UnsignedLongLong DecimalPointIndex=0;

            //remove all non-integer characters
            for(FACE::UnsignedLongLong i = 0; i < Stringval.size();i++)
            {
                if(Stringval[i] == '-' || Stringval[i]=='d' || Stringval[i]=='D'||Stringval[i]=='+'||Stringval[i]=='.')
                {
                    Stringval.erase(i,1);
                }
            }

            //what's left is the number of digits
            Digits = Stringval.size();

            Stringval = CSTRval;

            //remove all non-integer characters except for the decimal point
            for(FACE::UnsignedLong i = 0; i < Stringval.size();i++)
            {
                if(Stringval[i] == '-' || Stringval[i]=='d' || Stringval[i]=='D'||Stringval[i]=='+')
                {
                    Stringval.erase(i,1);

                }
            }

            //now we determine the scale
            DecimalPointIndex = Stringval.find_first_of(".");


            if(DecimalPointIndex != std::string::npos)
            {
                Scale = (Stringval.size()-1) - DecimalPointIndex;
            }
            else
            {
                Scale = 0;
            }

            //strip all the non-integer characters again except for the negative sign
            Stringval = CSTRval;
            for(FACE::UnsignedLong i = 0; i < Stringval.size();i++)
            {
                if(Stringval[i]=='d' || Stringval[i]=='D'||Stringval[i]=='+'||Stringval[i]=='.')
                {
                    Stringval.erase(i,1);
                }
            }

            Value = Stringval;
            MaxDigits = 31;
            myBitPrecision = calculateBitsNeeded(bigint::powBIGINT((bigint)10,(bigint)MaxDigits))+1;

            if(calculateBitsNeeded(abs(Value)) > myBitPrecision) {}
			// Error constructing Fixed type: Number of Digits exceeds 31
        }

/**********************************************
 *                scaleValue                  *
 **********************************************/
// Adjust the Value by the specificed power of ten
        bigint scaleValue(FACE::UnsignedShort newScale) const
        {
            if(newScale < Scale)
            {

                return (Value / bigint::powBIGINT((bigint)10,(bigint)Scale-newScale));
            }
            else if(newScale > Scale)
            {
                return(Value * bigint::powBIGINT((bigint)10,(bigint)(abs(Scale-newScale))));
            }
            else
            {
                return (Value);
            }
        }
/*******************************************************
 *             calculateBitsNeeded                     *
 *******************************************************/

        bigint calculateBitsNeeded(bigint aValue) const
        {
            if(aValue != 0)
            {
                return (bigint::integerLogTwo(aValue));
            }
            else
            {
                return (0);
            }
        }

/*******************************************************
 *                  abs                                *
 *******************************************************/
        bigint abs(bigint aValue) const
        {
            return (aValue._abs());
        }

       FACE::LongLong abs(FACE::LongLong aValue) const
        {
            if (aValue < 0)
            {
                return (aValue * -1);
            }
            else
            {
                return (aValue);
            }
        }

//////////////Private Members//////////////////////////////////////
        bigint Value;
        FACE::UnsignedShort Digits;
        FACE::UnsignedShort Scale;
        FACE::UnsignedShort MaxDigits;
        bigint myBitPrecision;

};

/***************************
 *    Arithmetic Operators *
 **************************/

inline FACE::Fixed operator + (const Fixed& val1, const Fixed& val2)
{
   FACE::Long newDigits = max((val1.fixed_digits()-val1.fixed_scale()),(val2.fixed_digits()-val2.fixed_scale())) +
                    max(val1.fixed_scale(),val2.fixed_scale()) + 1;
   FACE::Long newScale = max(val1.fixed_scale(),val2.fixed_scale());

    Fixed result(newDigits,newScale);

    if(val1.Scale == val2.Scale)
    {
        result.Value = val1.Value + val2.Value;
    }
    else if(val1.Scale > val2.Scale)
    {
        result.Value = val1.Value + val2.truncate(val1.Scale).Value;
    }
    else
    {
        result.Value = val1.truncate(val2.Scale).Value + val2.Value;
    }

    return (result);
}

///////////////////////////////////////////////////////////////////////

inline Fixed operator - (const Fixed& val1, const Fixed& val2)
{
   FACE::Long newDigits = max((val1.fixed_digits()-val1.fixed_scale()),(val2.fixed_digits()-val2.fixed_scale())) +
                    max(val1.fixed_scale(),val2.fixed_scale()) + 1;
   FACE::Long newScale = max(val1.fixed_scale(),val2.fixed_scale());

    Fixed result(newDigits,newScale);

    if(val1.Scale == val2.Scale)
    {
        result.Value = val1.Value - val2.Value;
    }
    else if(val1.Scale > val2.Scale)
    {
        result.Value = val1.Value - val2.truncate(val1.Scale).Value;
    }
    else
    {
        result.Value = val1.truncate(val2.Scale).Value - val2.Value;
    }

    return (result);
}

///////////////////////////////////////////////////////////////////

inline Fixed operator * (const Fixed& val1, const Fixed& val2)
{
   FACE::Long newDigits = val1.fixed_digits()+val2.fixed_digits();
   FACE::Long newScale  = val1.fixed_scale()+val2.fixed_scale();

    Fixed result(newDigits, newScale);

    result.Value = val1.Value * val2.Value;

    return (result);

}

/////////////////////////////////////////////////////////////////

inline Fixed operator / (const Fixed& val1, const Fixed& val2)
{

   FACE::Long newDigits = (val1.fixed_digits()-val1.fixed_scale())+val2.fixed_scale() +min(val1.fixed_scale(),val2.fixed_scale());
   FACE::Long newScale  = 8; //arbitrary decimal precision

    Fixed result(newDigits, newScale);


    if(val2.Digits-val1.Digits >= 0) // if the divisor has more digits than the dividend
    {

        if(val1.Scale - val2.Scale >= 0)
        {
            //increase the integer dividend(val1) to have enough place values to divide by the divisor (val2)
            result.Value = (val1.Value*bigint::powBIGINT((bigint)10,(bigint)val2.Digits-val1.Digits+1+newScale))/ val2.Value;
            //cut off the extra place values
            result.Value = result.Value / bigint::powBIGINT((bigint)10,(bigint)(val1.Scale-val2.Scale+val2.Digits-val1.Digits+1));
        }
        else
        {
            result.Value = (val1.Value*bigint::powBIGINT((bigint)10,(bigint)val2.Digits-val1.Digits+(abs(val1.Scale - val2.Scale) )+newScale))/ val2.Value;
            result.Value = result.Value/bigint::powBIGINT((bigint)10,(bigint)(val2.Digits-val1.Digits));
        }

        //result.Value = (val1.Value.powBIGINT((bigint)(double)val2.Digits-val1.Digits+1+newScale))/ val2.Value;
    }
    else //dividend has more digits than the divisor
    {
        result.Value = (val1.Value*bigint::powBIGINT((bigint)10,(bigint)newScale))/ val2.Value;

        if(val1.Scale - val2.Scale >= 0)
        {
            result.Value = result.Value / (bigint::powBIGINT((bigint)10,(bigint)val1.Scale-val2.Scale));
        }
        else
        {
            result.Value = result.Value * bigint::powBIGINT((bigint)10,(bigint)(abs(val1.Scale-val2.Scale)));
        }

    }

    return (result);

}

///////////////////////////////////////////////////////////////////////////


/**********************************************
 *        Relational Operators                *
 **********************************************/

inline FACE::Boolean operator > (const Fixed& val1, const Fixed& val2)
{
    if((val1 < val2) == false && (val1!=val2))
    {
        return (true);
    }
    else
    {
        return (false);
    }
}

////////////////////////////////////////////////////////////////////

inline FACE::Boolean operator < (const Fixed& val1, const Fixed& val2)
{
    bool isVal1Negative = false;
    bool isVal2Negative = false;

    if(val1.Value < 0)
    {
        isVal1Negative = true;
    }

    if(val2.Value < 0)
    {
        isVal2Negative = true;
    }

    bigint val1WholePart = val1.Value/ (bigint::powBIGINT((bigint)10,(bigint)val1.Scale));
    bigint val2WholePart = val2.Value/ (bigint::powBIGINT((bigint)10,(bigint)val2.Scale));


    if(val1WholePart < val2WholePart)
    {
        return (true);
    }
    else if(val1WholePart == val2WholePart)
    {
        bigint val1FractionalPart = val1.Value % (val1WholePart * bigint::powBIGINT((bigint)10,(bigint)val1.Scale));
        bigint val2FractionalPart = val2.Value % (val1WholePart * bigint::powBIGINT((bigint)10,(bigint)val2.Scale));

        if(isVal1Negative == true)
        {
            val1FractionalPart = val1FractionalPart * -1;
        }

        if(isVal2Negative == true)
        {
            val2FractionalPart = val2FractionalPart * -1;
        }

        if(val1.Scale < val2.Scale)
        {
            val1FractionalPart = val1FractionalPart * bigint::powBIGINT((bigint)10,(bigint)val2.Scale - val1.Scale);
        }
        else if(val1.Scale > val2.Scale)
        {
            val2FractionalPart = val2FractionalPart * bigint::powBIGINT((bigint)10,(bigint)val1.Scale - val2.Scale);
        }

        if(val1FractionalPart< val2FractionalPart)
        {
            return (true);
        }
        else
        {
            return (false);
        }


    }
    else
    {
        return (false);
    }

}

//////////////////////////////////////////////////////////////////////////

inline FACE::Boolean operator >= (const Fixed& val1, const Fixed& val2)
{
    if((val1 < val2) == false || val1 == val2)
    {
        return (true);
    }
    else
    {
        return (false);
    }
}

//////////////////////////////////////////////////////////////////////////

inline FACE::Boolean operator <= (const Fixed& val1, const Fixed& val2)
{
    if((val1 < val2) == true || val1 == val2)
    {
        return (true);
    }
    else
    {
        return (false);
    }
}

////////////////////////////////////////////////////////////////////////

inline FACE::Boolean operator == (const Fixed& val1, const Fixed& val2)
{
    bool isVal1Negative = false;
    bool isVal2Negative = false;

    if(val1.Value < 0)
    {
        isVal1Negative = true;
    }

    if(val2.Value < 0)
    {
        isVal2Negative = true;
    }

    bigint val1WholePart = val1.Value/ (bigint::powBIGINT((bigint)10,(bigint)val1.Scale));
    bigint val2WholePart = val2.Value/ (bigint::powBIGINT((bigint)10,(bigint)val2.Scale));

    if(val1WholePart == val2WholePart)
    {
        bigint val1FractionalPart = val1.Value % (val1WholePart * bigint::powBIGINT((bigint)10,(bigint)val1.Scale));
        bigint val2FractionalPart = val2.Value % (val1WholePart * bigint::powBIGINT((bigint)10,(bigint)val2.Scale));

        if(isVal1Negative == true)
        {
            val1FractionalPart = val1FractionalPart * -1;
        }

        if(isVal2Negative == true)
        {
            val2FractionalPart = val2FractionalPart * -1;
        }

        if(val1.Scale < val2.Scale)
        {
            val1FractionalPart = val1FractionalPart * bigint::powBIGINT((bigint)10,(bigint)val2.Scale - val1.Scale);
        }
        else if(val1.Scale > val2.Scale)
        {
            val2FractionalPart = val2FractionalPart * bigint::powBIGINT((bigint)10,(bigint)val1.Scale - val2.Scale);
        }

        if(val1FractionalPart == val2FractionalPart)
        {
            return (true);
        }
        else
        {
            return (false);
        }


    }
    else
    {
        return (false);
    }


}

///////////////////////////////////////////////////////////////////////////

inline FACE::Boolean operator != (const Fixed& val1, const Fixed& val2)
{
    if(val1 == val2)
    {
        return (false);
    }
    else
    {
        return (true);
    }
}

} //end FACE namespace
#endif /* FIXED_H_ */
