// Product produced under DoD SENSIAC contract HC104705D4000 under
// the sponsorship of the Defense Technical Information Center,
// ATTN: DTIC-AI, 8723 John J. Kingman Rd., Ste 0944, Fort Belvoir, VA
// 22060-6218.  SENSIAC is a DoD Information Analysis Center Sponsored
// by the Defense Technical Information Center.
//
// HANDLING AND DESTRUCTION NOTICE - Comply with distribution statement
// and destroy by any method that will prevent disclosure of
// contents or reconstruction of the document.

// Arbitrary length integer class
// Implemented using 64bit built in data types
// Chris Loftis - 4/1/2013 (chris.loftis@gtri.gatech.edu)

#ifndef BIGINT_H_
#define BIGINT_H_

#include <math.h>
#include <stdlib.h>
#include <string>
#include <stack>
#include <algorithm>
#include <FACE/types.hpp>

// Number of 64 bit chunks to use to store number
// Must be at least 2
#define NUMSEGMENTS 10

using namespace std;
class bigint
{
public:
	// Constructors
	bigint(FACE::UnsignedLong h, FACE::UnsignedLong l)
	{
		this->High = 0;
		for(int i = 0; i < (NUMSEGMENTS-1); i++)
		{
			this->Low[i] = 0;
		}
		if(NUMSEGMENTS > 2)
		{
			this->Low[1] = h;
		}
		else
		{
			this->High = h;
		}
		this->Low[0] = l;
	}

	bigint()
	{
		this->High = 0;
		for(int i = 0; i < (NUMSEGMENTS-1); i++)
		{
			this->Low[i] = 0;
		}
	}



	bigint(long int l)
	{
        this->High = 0;
        for(int i = 0; i < (NUMSEGMENTS-1); i++)
        {
            this->Low[i] = 0;
        }
        this->Low[0] = abs(l);

        if(l < 0)
        {
            *this=inv(*this);
        }
	}

	bigint(string input)
	{
        bool invert = false;
        if(input[0] == '-')
        {
            // Handle negative sign.
            invert = true;
            // make it 0 so no effect on result
            input[0] = '0';
        }

        // Takes a string of only decimal numbers
        *this = 0UL;

        for(unsigned i = 0; i < input.size(); i++)
        {
            unsigned digit = (input[i]-48);
            unsigned place = input.size()-i-1;
            bigint temp = 1UL;
            // Overflows pow(10,x) for x>5 so do this stuff
            while(place > 5)
            {
                temp = temp * pow((double)10,(double)5);
                place-= 5;
            }
            *this= *this + temp*(digit*pow(10,place));
        }

        if(invert && *this != 0)
        {

            *this = inv(*this);
        }
	}

	// Assignment operators

	bigint& operator=(string input)
	{
        bool invert = false;
        if(input[0] == '-')
        {
            // Handle negative sign.
            invert = true;
            // make it 0 so no effect on result
            input[0] = '0';
        }

        // Takes a string of only decimal numbers
        *this = 0UL;

        for(unsigned i = 0; i < input.size(); i++)
        {
            unsigned digit = (input[i]-48);
            unsigned place = input.size()-i-1;
            bigint temp = 1UL;
            // Overflows pow(10,x) for x>5 so do this stuff
            while(place > 5)
            {
                temp = temp * pow(10,5);
                place-= 5;
            }
            *this= *this + temp*(digit*pow(10,place));
        }

        if(invert&& *this != 0)
        {
            *this = inv(*this);
        }

        return (*this);
	}
	bigint& operator=(const unsigned long & rhs)
	{
		this->High = 0;
		for(int i = (NUMSEGMENTS-2); i >=0; i--)
		{
			this->Low[i] = 0;
		}
		this->Low[0] = (FACE::UnsignedLong)rhs;
		return (*this);
	}

	bigint& operator=(const unsigned long long & rhs)
    {
        this->High = 0;
        for(int i = (NUMSEGMENTS-2); i >=0; i--)
        {
            this->Low[i] = 0;
        }
        this->Low[0] = (FACE::UnsignedLong)rhs;
        return (*this);
    }

	bigint& operator=(const unsigned int & rhs)
    {
        this->High = 0;
        for(int i = (NUMSEGMENTS-2); i >=0; i--)
        {
            this->Low[i] = 0;
        }
        this->Low[0] = (FACE::UnsignedLong)rhs;
        return (*this);
    }
    bigint& operator=(const FACE::Double & rhs)
    {
        this->High = 0;
        for(int i = (NUMSEGMENTS-2); i >=0; i--)
        {
            this->Low[i] = 0;
        }
        this->Low[0] = fabs(rhs);
        if(rhs < 0)
        {
            *this = inv(*this);
        }
        return (*this);
    }

    bigint& operator=(const FACE::LongDouble & rhs)
    {
        this->High = 0;
        for(int i = (NUMSEGMENTS-2); i >=0; i--)
        {
            this->Low[i] = 0;
        }
        this->Low[0] = fabs(rhs);
        if(rhs < 0)
        {
            *this = inv(*this);
        }
        return (*this);
    }

    bigint& operator++()
    {
       *this = *this + 1;
       return (*this);
    }

    bigint& operator++(int)
    {
          *this = *this + 1;
          return (*this);
    }

    bigint& operator--(int)
    {
       *this = *this - 1;
       return (*this);
    }

    bigint& operator--()
    {
        *this = *this - 1;
         return (*this);

    }


    bigint& operator=(const  int & rhs)
    {
        this->High = 0;
        for(int i = (NUMSEGMENTS-2); i >=0; i--)
        {
            this->Low[i] = 0;
        }
        this->Low[0] = abs(rhs);
        if(rhs < 0)
        {
            *this = inv(*this);
        }
        return (*this);
    }

	bigint& operator=(const long int & rhs)
	{
		this->High = 0;
		for(int i = (NUMSEGMENTS-2); i >=0; i--)
		{
			this->Low[i] = 0;
		}
		this->Low[0] = abs(rhs);
		if(rhs < 0)
		{
			*this = inv(*this);
		}
		return (*this);
	}

	bigint& operator= (const long long int &rhs)
	{
        this->High = 0;
        for(int i = (NUMSEGMENTS-2); i >=0; i--)
        {
            this->Low[i] = 0;
        }
        this->Low[0] = abs((long int)rhs);
        if(rhs < 0)
        {
            *this = inv(*this);
        }
        return (*this);
	}

	bigint& operator=(const bigint & rhs)
	{
		this->High = rhs.High;
		for(int i = 0; i < (NUMSEGMENTS-1); i++)
		{
			this->Low[i] = rhs.Low[i];
		}
		return (*this);
	}

	// Output stream - currently unimplemented

	// Comparison operators
	bool operator<(const bigint & rhs) const
	{
		if(this->High == rhs.High)
		{
			for(int i = (NUMSEGMENTS-2); i >=0; i--)
			{
				if(this->Low[i] == rhs.Low[i])
				{
				}
				else if (this->Low[i] < rhs.Low[i])
				{
					return (true);// Less than

				}
				else
				{
					return (false);// Greater than
				}
			}
			return (false);// Gotten here then both equal
		}
		else if(this->High < rhs.High)
		{
			return (true);
		}
		else
		{
			return (false);
		}
	}

	bool operator>(const bigint & rhs) const
	{
		if(this->High == rhs.High)
		{
			for(int i = (NUMSEGMENTS-2); i >=0; i--)
			{
				if(this->Low[i] == rhs.Low[i])
				{
				}
				else if (this->Low[i] > rhs.Low[i])
				{
					return (true);// Greater than
				}
				else
				{
					return (false);// Less than
				}
			}
			return (false);// Gotten here then both equal
		}
		else if(this->High > rhs.High)
		{
			return (true);
		}
		else
		{
			return (false);
		}
	}

	bool operator==(const bigint & rhs) const
	{
		for(int i = 0; i < (NUMSEGMENTS-1); i++)
		{
			if(this->Low[i] != rhs.Low[i])
			{
				return (false);
			}
		}
		if(this->High != rhs.High)
		{
			return (false);
		}
		return (true);
	}

	bool operator>=(const bigint & rhs) const
	{
		return ( (*this == rhs) || (*this > rhs) );
	}

	bool operator<=(const bigint &rhs) const
	{
		return ( (*this == rhs) || (*this < rhs) );
	}

	bool operator!=(const bigint & rhs) const
	{
		return (!(*this == rhs));
	}

	// Misc functions
	bigint _abs(const bigint & value) const
	{
		bigint result = value;
		if(value.High < 0)
		{
			// Taking 2s compliment
			result.High = ~result.High;
			for(int i = 0; i < (NUMSEGMENTS-1); i++)
			{
				result.Low[i] = ~result.Low[i];
			}
			result = result + 1;
		}
		return (result);
	}
    bigint _abs() const
    {
        bigint result = *this;
        if(this->High < 0)
        {
            // Taking 2s compliment
            result.High = ~result.High;
            for(int i = 0; i < (NUMSEGMENTS-1); i++)
            {
                result.Low[i] = ~result.Low[i];
            }
            result = result + 1;
        }
        return (result);
    }

	bigint inv(const bigint & value) const
	{
		bigint result;
		//cout << "Before inv " << value << endl;
		// Take 2s compliment
		result.High = ~value.High;
		for(int i = 0; i < (NUMSEGMENTS-1); i++)
		{
			result.Low[i] = ~value.Low[i];
		}
		result = result + 1;
		//cout << " after inv" << result << endl;
		return (result);
	}

	bool signDiff(const bigint & left, const bigint & right) const
	{
		// Returns true if signs are different. For multiplication, etc
		if( ((left.High >= 0) && (right.High >= 0)) || ((left.High < 0) && (right.High < 0)) )
		{
			return (false);
		}
		else
		{
			return (true);
		}
	}

	bigint operator|(const bigint & rhs) const
	{
		bigint result;

		result.High = this->High | rhs.High;
		for(int i = 0; i < (NUMSEGMENTS-1); i++)
		{
			result.Low[i] = this->Low[i] | rhs.Low[i];
		}
		return (result);
	}

	bigint operator&(const bigint & rhs) const
	{
		bigint result;

		result.High = this->High & rhs.High;
		for(int i = 0; i < (NUMSEGMENTS-1); i++)
		{
			result.Low[i] = this->Low[i] & rhs.Low[i];
		}
		return (result);
	}

	bigint operator<<(const int & rhs) const
	{
		bigint result = *this;
		int shiftnum = rhs;
		int invshiftnum = (8*sizeof(FACE::UnsignedLong)-shiftnum);

		if(shiftnum == 0)
		{
			return (*this);
		}
		else if (shiftnum >= 32)
		{
			while(1)
			{
				// can only shift 63 bits at a time at most
				int shift = min(31,shiftnum);
				result = result << shift;
				shiftnum -= shift;
				invshiftnum = (8*sizeof(FACE::UnsignedLong)-shiftnum);
				if(shiftnum <= 0)
				{
					return (result);
				}
			}
		}
		// mask to get the bits in Low that are being shifted into high
		FACE::UnsignedLong maskHigh = -1;
		FACE::UnsignedLong maskLow;
		maskHigh = maskHigh << invshiftnum;
		maskLow = maskHigh >> invshiftnum;

		FACE::UnsignedLong tempBuff = 0;

		//cout << invshiftnum << endl;

		for(int i = 0; i < NUMSEGMENTS-1; i++)
		{
			tempBuff |= this->Low[i] & maskHigh;
			//cout << "1" << hex << tempBuff << endl;
			result.Low[i] = this->Low[i] << shiftnum;
			result.Low[i] |= tempBuff & maskLow;
			//cout << "2" << tempBuff << endl;
			tempBuff = tempBuff >> invshiftnum;
			//cout << "3" << tempBuff << endl;
		}

		result.High = this->High << shiftnum;
		result.High |= tempBuff & maskLow;

		return (result);

	}

	bigint operator>>(const int & rhs) const
	{
		bigint result = *this;
		int shiftnum = rhs;
		unsigned invshiftnum = (8*sizeof(FACE::UnsignedLong)-shiftnum);

		if(shiftnum == 0)
		{
			return (*this);
		}
		else if (shiftnum >= 32)
		{
			while(1)
			{
				// can only shift 63 bits at a time at most
				int shift = min(31,shiftnum);
				result = result >> shift;
				shiftnum -= shift;
				invshiftnum = (8*sizeof(FACE::UnsignedLong)-shiftnum);
				if(shiftnum <= 0)
				{
					return (result);
				}
			}
		}

		FACE::UnsignedLong maskHigh = -1;
		FACE::UnsignedLong maskLow;
		maskHigh = maskHigh << invshiftnum;
		maskLow = maskHigh >> invshiftnum;


		FACE::UnsignedLong tempBuff = 0;

		tempBuff = this->High & maskLow;
		result.High = (FACE::UnsignedLong)this->High >> shiftnum;
		tempBuff = tempBuff << invshiftnum;

		for(int i = NUMSEGMENTS-2; i >= 0; i--)
		{
			tempBuff |= this->Low[i] & maskLow;
			result.Low[i] = this->Low[i] >> shiftnum;
			result.Low[i] |= tempBuff & maskHigh;
			tempBuff = tempBuff << invshiftnum;
		}

		return (result);
	}

	// Math functions
	bigint operator+(const bigint & rhs) const
	{
		bigint result = 0;
		FACE::UnsignedLong temp;
		bool carry = false;
		for(int i = 0; i < (NUMSEGMENTS-1); i++)
		{

			result.Low[i] = this->Low[i] + rhs.Low[i];

			temp = result.Low[i];
			//cout << i << " " << carry << endl;
			if(carry)
			{
				++result.Low[i];
				if(result.Low[i] < temp)
				{
					// carry from adding in carry
					carry = true;
				}
				else
				{
					carry = false;
				}
			}

			//cout << hex <<" " << result.Low[i];
			if(temp < this->Low[i])
			{
				carry = true;
			}


		}

		result.High = this->High + rhs.High;
		//cout << hex << carry << " " << rhs.High << " " << this->High <<  endl;
		if(carry)
		{
			++result.High;
			//result = inv(result);
		}


		return (result);
	}

	bigint operator+(const unsigned & rhs) const
	{
		bigint result = 0;
		result.High = this->High;
		for(int i = 0; i < NUMSEGMENTS-1; i++)
		{
			result.Low[i] = this->Low[i];
		}
		result.Low[0] = this->Low[0] + rhs;

		//have to propagate overflow up all the way
		int i = 0;
		while(result.Low[i] < this->Low[i])
		{
			++i;
			if( (i - 2) >= NUMSEGMENTS)
			{
				++result.High;
				break;
			}
			else
			{
				++result.Low[i];
			}
		}

		return (result);
	}

	bigint operator+=(const bigint &rhs)
	{
		*this = *this + rhs;
		return (*this);
	}

	bigint operator-(const bigint &rhs) const
	{

		// subtraction = adding inverse
		bigint result = *this + inv(rhs);

		return (result);
	}

	bigint operator-=(const bigint &rhs)
	{
		*this = *this - rhs;
		return (*this);
	}

	bigint operator*(const bigint &rhs) const
	{
		// DO NOT USE COUT HERE
		// Multiplication by shifting and adding
		bigint result;
		bigint larger, smaller;
		int i = 0;

		if(_abs(*this) >= _abs(rhs))
		{
			larger = _abs(*this);
			smaller = _abs(rhs);
		}
		else
		{
			larger = _abs(rhs);
			smaller = _abs(*this);
		}

		// For each bit set in smaller, shift larger left by that much and add it to the total
		while (smaller > 0)
		{
			if(smaller.Low[0] & 0x01)
			{
				result = result + (larger << i);
			}

			i++;
			smaller = smaller >> 1;
		}

		if(signDiff(*this,rhs))
		{
			result = inv(result);
		}
		return (result);
	}

	bigint operator*=(const bigint &rhs)
	{
		*this = *this*rhs;
		return (*this);
	}

	bigint operator/(const bigint &rhs) const
	{
		bigint dividend = _abs(*this);
		bigint divisor = _abs(rhs);
		bigint result = 0;
		bigint current = 1;

		if (dividend == 0)
		{
			return (dividend);
		}
		else if(divisor > dividend)
		{
			result = (unsigned long)0x0UL;
			return (result);
		}
		else if(divisor == dividend)
		{
			result = (unsigned long)0x1UL;
			return (result);
		}

		while(divisor <= dividend)
		{
			divisor = divisor << 1;
			current = current << 1;
		}

		divisor = divisor >> 1;
		current = current >> 1;

		while (current != 0)
		{
			if(dividend >= divisor)
			{
				dividend = dividend - divisor;
				result = result | current;
			}
			current = current >> 1;
			divisor = divisor >> 1;
		}

		if(signDiff(*this,rhs))
		{
			result = inv(result);
		}
		return (result);
	}

	bigint operator/=(const bigint &rhs)
	{
		*this = *this/rhs;
		return (*this);
	}

	bigint operator%(const bigint &rhs) const
	{
		// Could do better here but probably not doing many modulos
		// Doing a couple for every cout. Oh well.
		bigint result;
		bigint abslhs = _abs(*this);
		bigint absrhs = _abs(rhs);

		if(abslhs < absrhs || absrhs == 0)
		{
			return (abslhs);
		}
		result = abslhs / absrhs;

		result = abslhs - (absrhs * result);

		return (result);
	}

	// Log base 2 rounded up.
	// Essentially = place of MSB
	static unsigned integerLogTwo(const bigint &rhs)
	{
		int placeInSegment;
		int segmentNum;
		if(rhs.High != 0)
		{
			placeInSegment = 64-__builtin_clzl((FACE::UnsignedLong)rhs.High);// replace with Thomas' function. Takeout "64-" part.
			segmentNum = NUMSEGMENTS;
		}
		else
		{
			for(int i = NUMSEGMENTS-2; i >= 0; i--)
			{
				if(rhs.Low[i] != 0)
				{
					placeInSegment = 64-__builtin_clzl(rhs.Low[i]);
					segmentNum = i;
					break;
				}
			}
		}
		return (placeInSegment + segmentNum*64);
	}

	// Log base 10.
	static unsigned log(const bigint &rhs)
	{
		if(bigint(0) > rhs)
		{
			return (0);
		}
		unsigned logtwo = integerLogTwo(rhs);
		// This works somehow. The 1233 then shift essentially converts from base 2 to base 10.
		unsigned result = ((logtwo + 1) * 1233) >> 12;
		return (result);
	}

	// Using this for pow instead of bitwise or
	bigint powBIGINT(const bigint &rhs) const
	{
		bigint result = 1;
		bigint exponent = rhs;
		bigint base = *this;
		while(exponent != 0)
		{
			// test bit 0 of exponent
			if(exponent.Low[0] & 0x01)
			{
				result = result * base;
				exponent = exponent - 1;
			}
			base = base * base;
			exponent = exponent/2;
		}
		return (result);
	}

    // Using this for pow instead of bitwise or
    static bigint powBIGINT(const bigint &inputbase, const bigint &rhs)
    {
        bigint result = 1;
        bigint exponent = rhs;
        bigint base = inputbase;
        while(exponent != 0)
        {
            // test bit 0 of exponent
            if(exponent.Low[0] & 0x01)
            {
                result = result * base;
                exponent = exponent - 1;
            }
            base = base * base;
            exponent = exponent/2;
        }
        return (result);
    }

private:

	// Number stored in 64 bit chunks where the most significant is signed to keep the sign information
	FACE::Long High;
	FACE::UnsignedLong Low[NUMSEGMENTS-1];// LSchunk is 0
};

#undef NUMSEGMENTS
#endif
