
------------------------------------------------------------------------------------
-- Copyright (c) Vanderbilt University, 2020                                       
--                                                                                 
-- ALL RIGHTS RESERVED, UNLESS OTHERWISE STATED                                    
--                                                                                 
-- This software, authored by Vanderbilt University under a contract awarded to and
-- managed by Alion Science and Technology, was funded by the U.S. Government under
-- Contract No. FA8075-14-D-0014 and the U.S. Government has unlimited rights in   
-- this software.  An unlimited rights license means that the U.S. Government can  
-- use, modify, reproduce, release or disclose computer software in whole or in    
-- part, in any manner, and for any purpose whatsoever, and to have or authorize   
-- others to do so.                                                                
--                                                                                 
-- Vanderbilt University disclaims all warranties with regard to this software,    
-- including all implied warranties of merchantability and fitness. In no event    
-- shall Vanderbilt University be liable for any special, indirect or consequential
-- damages or any damages whatsoever resulting from loss of use, data or profits,  
-- whether in an action of contract, negligence or other tortious action, arising  
-- out of or in connection with the use or performance of this software.           
--                                                                                 
-- May contain products produced under DoD SENSIAC contract HC104705D4000 under the
-- sponsorship of the Defense Technical Information Center,                        
-- ATTN: DTIC-AI, 8723 John J. Kingman Rd., Ste 0944, Fort Belvoir, VA 22060-6218. 
-- SENSIAC is a DoD Information Analysis Center Sponsored by the Defense Technical 
-- Information Center.                                                             
--                                                                                 
-- Filename: goldStandardLibraries/Ada/spec/FACE/face-hmfm.ads                     
--                                                                                 
-- Conformance Test Suite Version 2.1.1r9 supporting FACE Standard Version 2.1.1   
--                                                                                 
-- This file is generated and may be overwritten.                                  
--                                                                                 
-- Release Date: 29-Aug-2023                                                       
------------------------------------------------------------------------------------
with FACE_Basic_Types;  use FACE_Basic_Types;


package FACE.HMFM is

    type FAULT_MESSAGE_SIZE_TYPE is new FACE_Basic_Types.Long;
    
    type FAULT_MESSAGE_ADDRESS_TYPE is new System.Address;
    
    type THREAD_ID_TYPE is new FACE_Basic_Types.Long;
    
    FAULT_MESSAGE_MAXIMUM_SIZE : constant FACE.HMFM.FAULT_MESSAGE_SIZE_TYPE := 128;
    
    type STACK_SIZE_TYPE is new FACE_Basic_Types.Unsigned_Long;
    
    subtype FAULT_MESSAGE_TYPE is Standard.String(1 .. Standard.Integer(FACE.HMFM.FAULT_MESSAGE_MAXIMUM_SIZE));
    
    type FAULT_CODE_TYPE is
        (
            DEADLINE_MISSED,
            APPLICATION_ERROR,
            NUMERIC_ERROR,
            ILLEGAL_REQUEST,
            STACK_OVERFLOW,
            MEMORY_VIOLATION,
            HARDWARE_FAULT,
            POWER_FAIL
        );
    for FAULT_CODE_TYPE'Size use FACE_Basic_Types.Enum_Size;
    pragma Convention (C, FAULT_CODE_TYPE);
    
    type FAULT_STATUS_TYPE is
        record
                          CODE : FACE.HMFM.FAULT_CODE_TYPE;
                       MESSAGE : FACE.HMFM.FAULT_MESSAGE_TYPE;
                        LENGTH : FACE.HMFM.FAULT_MESSAGE_SIZE_TYPE;
              FAILED_THREAD_ID : FACE.HMFM.THREAD_ID_TYPE;
                FAILED_ADDRESS : FACE.SYSTEM_ADDRESS_TYPE;
    
        end record;
    pragma Convention (C, FAULT_STATUS_TYPE);
    
    type FAULT_HANDLER_ENTRY_TYPE is access procedure
    ;
    pragma Convention(C, FAULT_HANDLER_ENTRY_TYPE);
    
    procedure Initialize
    (
        return_code :    out FACE.RETURN_CODE_TYPE
    );
    
    procedure Create_Fault_Handler
    (
        entry_point : in     FACE.HMFM.FAULT_HANDLER_ENTRY_TYPE;
        stack_size  : in     FACE.HMFM.STACK_SIZE_TYPE;
        return_code :    out FACE.RETURN_CODE_TYPE
    );
    
    procedure Report_Application_Message
    (
        fault       : in     FACE.HMFM.FAULT_MESSAGE_ADDRESS_TYPE;
        length      : in     FACE.HMFM.FAULT_MESSAGE_SIZE_TYPE;
        return_code :    out FACE.RETURN_CODE_TYPE
    );
    
    procedure Get_Fault_Status
    (
        fault       :    out FACE.HMFM.FAULT_STATUS_TYPE;
        return_code :    out FACE.RETURN_CODE_TYPE
    );
    
    procedure Raise_Application_Fault
    (
        code        : in     FACE.HMFM.FAULT_CODE_TYPE;
        message     : in     FACE.HMFM.FAULT_MESSAGE_ADDRESS_TYPE;
        length      : in     FACE.HMFM.FAULT_MESSAGE_SIZE_TYPE;
        return_code :    out FACE.RETURN_CODE_TYPE
    );
    

end FACE.HMFM;

