
------------------------------------------------------------------------------------
-- Copyright (c) Vanderbilt University, 2020                                       
--                                                                                 
-- ALL RIGHTS RESERVED, UNLESS OTHERWISE STATED                                    
--                                                                                 
-- This software, authored by Vanderbilt University under a contract awarded to and
-- managed by Alion Science and Technology, was funded by the U.S. Government under
-- Contract No. FA8075-14-D-0014 and the U.S. Government has unlimited rights in   
-- this software.  An unlimited rights license means that the U.S. Government can  
-- use, modify, reproduce, release or disclose computer software in whole or in    
-- part, in any manner, and for any purpose whatsoever, and to have or authorize   
-- others to do so.                                                                
--                                                                                 
-- Vanderbilt University disclaims all warranties with regard to this software,    
-- including all implied warranties of merchantability and fitness. In no event    
-- shall Vanderbilt University be liable for any special, indirect or consequential
-- damages or any damages whatsoever resulting from loss of use, data or profits,  
-- whether in an action of contract, negligence or other tortious action, arising  
-- out of or in connection with the use or performance of this software.           
--                                                                                 
-- May contain products produced under DoD SENSIAC contract HC104705D4000 under the
-- sponsorship of the Defense Technical Information Center,                        
-- ATTN: DTIC-AI, 8723 John J. Kingman Rd., Ste 0944, Fort Belvoir, VA 22060-6218. 
-- SENSIAC is a DoD Information Analysis Center Sponsored by the Defense Technical 
-- Information Center.                                                             
--                                                                                 
-- Filename: goldStandardLibraries/Ada/spec/FACE/face-ts.ads                       
--                                                                                 
-- Conformance Test Suite Version 2.1.1r9 supporting FACE Standard Version 2.1.1   
--                                                                                 
-- This file is generated and may be overwritten.                                  
--                                                                                 
-- Release Date: 29-Aug-2023                                                       
------------------------------------------------------------------------------------
with FACE_Basic_Types;  use FACE_Basic_Types;
with System; 


package FACE.TS is

    subtype CONNECTION_NAME_TYPE is Standard.String(1 .. 64);
    
    type MESSAGE_SIZE_TYPE is new FACE_Basic_Types.Long;
    
    type MESSAGE_TYPE_GUID is new FACE_Basic_Types.Long_Long;
    
    type CONNECTION_DIRECTION_TYPE is
        (
            SOURCE,
            DESTINATION,
            BI_DIRECTIONAL,
            ONE_WAY_REQUEST_SOURCE,
            ONE_WAY_REQUEST_DESTINATION,
            TWO_WAY_REQUEST_SYNCHRONOUS_SOURCE,
            TWO_WAY_REQUEST_SYNCHRONOUS_DESTINATION,
            TWO_WAY_REQUEST_REPLY_ASYNCHRONOUS_SOURCE,
            TWO_WAY_REQUEST_REPLY_ASYNCHRONOUS_DESTINATION,
            NOT_DEFINED_CONNECTION_DIRECTION_TYPE
        );
    for CONNECTION_DIRECTION_TYPE'Size use FACE_Basic_Types.Enum_Size;
    pragma Convention (C, CONNECTION_DIRECTION_TYPE);
    
    type CONNECTION_ID_TYPE is new FACE_Basic_Types.Long_Long;
    
    type TRANSACTION_ID_TYPE is new FACE_Basic_Types.Long_Long;
    
    type CONNECTION_TYPE is
        (
            SAMPLING_PORT,
            QUEUING_PORT,
            SOCKET,
            MQ,
            SHM,
            CORBA,
            DDS
        );
    for CONNECTION_TYPE'Size use FACE_Basic_Types.Enum_Size;
    pragma Convention (C, CONNECTION_TYPE);
    
    type WAITSET_TYPE is array (0 .. 31) of FACE_Basic_Types.Boolean;
    pragma Convention (C, WAITSET_TYPE);
    
    type QUEUING_DISCIPLINE_TYPE is
        (
            FIFO,
            PRIORITY,
            NOT_DEFINED_QUEUING_DISCIPLINE_TYPE
        );
    for QUEUING_DISCIPLINE_TYPE'Size use FACE_Basic_Types.Enum_Size;
    pragma Convention (C, QUEUING_DISCIPLINE_TYPE);
    
    type CONNECTION_DOMAIN_TYPE is
        (
            UNIX,
            INET,
            NOT_DEFINED_CONNECTION_DOMAIN_TYPE
        );
    for CONNECTION_DOMAIN_TYPE'Size use FACE_Basic_Types.Enum_Size;
    pragma Convention (C, CONNECTION_DOMAIN_TYPE);
    
    type SOCKET_TYPE is
        (
            STREAM,
            DGRAM,
            SEQPACKET,
            NOT_DEFINED_SOCKET_TYPE
        );
    for SOCKET_TYPE'Size use FACE_Basic_Types.Enum_Size;
    pragma Convention (C, SOCKET_TYPE);
    
    type RECEIVE_FLAG_TYPE is
        (
            PEEK,
            OOB_RECEIVE_FLAG_TYPE,
            WAITALL,
            NOT_DEFINED_RECEIVE_FLAG_TYPE
        );
    for RECEIVE_FLAG_TYPE'Size use FACE_Basic_Types.Enum_Size;
    pragma Convention (C, RECEIVE_FLAG_TYPE);
    
    type SEND_FLAG_TYPE is
        (
            EOR,
            OOB_SEND_FLAG_TYPE,
            NOSIGNAL,
            NOT_DEFINED_SEND_FLAG_TYPE
        );
    for SEND_FLAG_TYPE'Size use FACE_Basic_Types.Enum_Size;
    pragma Convention (C, SEND_FLAG_TYPE);
    
    type VALIDITY_TYPE is
        (
            INVALID,
            VALID
        );
    for VALIDITY_TYPE'Size use FACE_Basic_Types.Enum_Size;
    pragma Convention (C, VALIDITY_TYPE);
    
    type MESSAGING_PATTERN_TYPE is
        (
            PUB_SUB,
            CLIENT,
            SERVER
        );
    for MESSAGING_PATTERN_TYPE'Size use FACE_Basic_Types.Enum_Size;
    pragma Convention (C, MESSAGING_PATTERN_TYPE);
    
    type WAITING_RANGE_TYPE is new FACE_Basic_Types.Long;
    
    type TRANSPORT_CONNECTION_STATUS_TYPE is
        record
                                    MESSAGE : FACE.MESSAGE_RANGE_TYPE;
                                MAX_MESSAGE : FACE.MESSAGE_RANGE_TYPE;
                           MAX_MESSAGE_SIZE : FACE.TS.MESSAGE_SIZE_TYPE;
                       CONNECTION_DIRECTION : FACE.TS.CONNECTION_DIRECTION_TYPE;
              WAITING_PROCESSES_OR_MESSAGES : FACE.TS.WAITING_RANGE_TYPE;
                             REFRESH_PERIOD : FACE.SYSTEM_TIME_TYPE;
                          LAST_MSG_VALIDITY : FACE.TS.VALIDITY_TYPE;
    
        end record;
    pragma Convention (C, TRANSPORT_CONNECTION_STATUS_TYPE);
    

end FACE.TS;

