with Interfaces;

with FACE.Exceptions; use FACE.Exceptions;

--------------------
-- FACE.Utilities --
--------------------
package FACE.Utilities is

   -----------
   -- Check --
   -----------
   --
   -- This procedure simply returns when the Return_Code argument has the value No_Error;
   -- otherwise, a unique exception is raised that is tied to the specific Return_Code value.
   --
   -- Note:  Additional information could be included in a String, if this be deemed desirable.
   --
   function Check (Return_Code     : in FACE.Return_Code_Type;
                   Raise_Exception : in Boolean := False)
                   return FACE.Return_Code_Type;

   type User_Character is new Character;

   ---------
   -- "=" --
   ---------
   function "=" (L, R : User_Character) return Standard.Boolean;

   --------------------
   -- User_Char_Hash --
   --------------------
   function User_Char_Hash (C : User_Character) return Natural;

   subtype Usage_Type is Interfaces.Unsigned_64;

   -----------
   -- Usage --
   -----------
   --   This simple protected type can be embedded within other objects
   --   to keep a simple, atomic count of usage, typically for the
   --   allocation/deallocation of the object's memory.
   --
   --   Increment increases the reference count for the object by 1.
   --
   --   Decrement decreases the reference count for the object by 1 and
   --   returns a Boolean value indicating whether the counter has
   --   reached zero.
   --
   --   Current returns the value of the counter at the time Current is
   --   called.
   protected type Usage
   is
      procedure Increment;
      procedure Decrement (Is_Zero : out Boolean);
      function  Current return Usage_Type;
   private
      Count : Usage_Type := 1;
   end Usage;

end FACE.Utilities;
