------------------------------------------------------------------------------------------------------
-- DISTRIBUTION STATEMENT A. Approved for public release; distribution is unlimited.
--
-- Product produced under DoD SENSIAC contract HC104705D4000 under the sponsorship of the Defense
-- Technical Information Center, ATTN: DTIC-AI, 8723 John J. Kingman Rd., Ste 0944, Fort Belvoir, VA
-- 22060-6218.  SENSIAC is a DoD Information Analysis Center Sponsored by the Defense Technical
-- Information Center.
--
-- HANDLING AND DESTRUCTION NOTICE - Comply with distribution statement and destroy by any method that
-- will prevent disclosure of the contents or reconstruction of the document.
------------------------------------------------------------------------------------------------------

-- This spec defines basic types required by the OMG Ada Language Mapping v1.3.
-- Certain types have been modified to be compatible with C.
-- These modifications are marked with a "C interoperability" comment.

with Interfaces;
with Interfaces.C;
with Ada.Strings.Unbounded;
with Ada.Strings.Wide_Unbounded;

package FACE_Basic_Types is

  type    Short              is new Interfaces.Integer_16;
  type    Long               is new Interfaces.Integer_32;
  type    Long_Long          is new Interfaces.Integer_64;
  type    Unsigned_Short     is new Interfaces.Unsigned_16;
  type    Unsigned_Long      is new Interfaces.Unsigned_32;
  type    Unsigned_Long_Long is new Interfaces.Unsigned_64;
  type    Float              is new Interfaces.IEEE_Float_32;
  type    Double             is new Interfaces.IEEE_Float_64;
  type    Long_Double        is new Interfaces.IEEE_Extended_Float;

  subtype Char               is Standard.Character;
  subtype Wchar              is Standard.Wide_Character;
  type    Octet              is new Interfaces.Unsigned_8;

  type Boolean               is new Standard.Boolean;
  for Boolean'Size use 8;  -- C interoperability

  subtype String is Ada.Strings.Unbounded.Unbounded_String;
  function To_FACE_String (Source : Standard.String)
  	return FACE_Basic_Types.String
	renames Ada.Strings.Unbounded.To_Unbounded_String;
  function To_Standard_String (Source : FACE_Basic_Types.String)
  	return Standard.String
	renames Ada.Strings.Unbounded.To_String;
  Null_String : String renames Ada.Strings.Unbounded.Null_Unbounded_String;

  subtype Wide_String is Ada.Strings.Wide_Unbounded.Unbounded_Wide_String;
  function To_FACE_Wide_String (Source : Standard.Wide_String)
  	return FACE_Basic_Types.Wide_String
	renames Ada.Strings.Wide_Unbounded.To_Unbounded_Wide_String;
  function To_Standard_Wide_String (Source : FACE_Basic_Types.Wide_String)
  	return Standard.Wide_String
	renames Ada.Strings.Wide_Unbounded.To_Wide_String;
  Null_Wide_String : Wide_String renames Ada.Strings.Wide_Unbounded.Null_Unbounded_Wide_String;

  Max_String_Length : constant := 256;
  subtype String_Range_Type is Positive range 1 .. Max_String_Length;

  -- C interoperability (C enums have the size of a C int)
  Enum_Size : constant := Interfaces.C.Int'Size;

end FACE_Basic_Types;
