-- -----------------------------------------------------------------------
-- --
-- Logbook system constant and type definitions and management services --
-- --
-- -----------------------------------------------------------------------
package APEX.Logbook_System is
   type Logbook_Id_Type is private;
   Null_Logbook_Id : constant Logbook_Id_Type;
   subtype Logbook_Name_Type is Name_Type;
   type Write_Status_Type is (Aborted, In_Progress, Complete);
   type Logbook_Status_Type is record
      Max_Message_Size : APEX_Integer;
      Max_Nb_Logged_Messages : APEX_Integer;
      Max_Nb_In_Progress_Messages : APEX_Integer;
      Nb_Logged_Messages : APEX_Integer;
      Nb_In_Progress_Messages : APEX_Integer;
      Nb_Aborted_Messages : APEX_Integer;
   end record;
   for Logbook_Status_Type'Alignment use 8;
   procedure Create_Logbook (
                             Logbook_Name : in Logbook_Name_Type;
                             Max_Message_Size : in Message_Size_Type;
                             Max_Nb_Logged_Messages : in Message_Range_Type;
                             Max_Nb_In_Progress_Messages : in Message_Range_Type;
                             Logbook_Id : out Logbook_Id_Type;
                             Return_Code : out Return_Code_Type);
   procedure Write_Logbook (
                            Logbook_Id : in Logbook_Id_Type;
                            Message_Addr : in Message_Addr_Type;
                            Length : in Message_Size_Type;
                            Return_Code : out Return_Code_Type);
   procedure OverWrite_Logbook (
                                Logbook_Id : in Logbook_Id_Type;
                                Message_Addr : in Message_Addr_Type;
                                Length : in Message_Size_Type;
                                Return_Code : out Return_Code_Type);
   procedure Read_Logbook (
                           Logbook_Id : in Logbook_Id_Type;
                           Logbook_Entry : in Message_Range_Type;
                           Message_Addr : in Message_Addr_Type;
                           Length : out Message_Size_Type;
                           Write_Status : out Write_Status_Type;
                           Return_Code : out Return_Code_Type);
   procedure Clear_Logbook (
                            Logbook_Id : in Logbook_Id_Type;
                            Return_Code : out Return_Code_Type);
   procedure Get_Logbook_Id (
                             Logbook_Name : in Logbook_Name_Type;
                             Logbook_Id : out Logbook_Id_Type;
                             Return_Code : out Return_Code_Type);
   procedure Get_Logbook_Status (
                                 Logbook_Id : in Logbook_Id_Type;
                                 Logbook_Status : out Logbook_Status_Type;
                                 Return_Code : out Return_Code_Type);
private
   type Logbook_Id_Type is new APEX_Integer;
   Null_Logbook_Id : constant Logbook_Id_Type := 0;
     pragma Convention (C, Write_Status_Type);
     pragma Convention (C, Logbook_Status_Type);
-- FOR NOW WE ARE GENERATING ADA BODIES FOR THESE...MAY WANT TO REVISIT AND USE C objects instead
--     pragma Import (C, Create_Logbook, "CREATE_LOGBOOK");
--     pragma Import (C, Write_Logbook, "WRITE_LOGBOOK");
--     pragma Import (C, OverWrite_Logbook, "OVERWRITE_LOGBOOK");
--     pragma Import (C, Read_Logbook, "READ_LOGBOOK");
--     pragma Import (C, Clear_Logbook, "CLEAR_LOGBOOK");
--     pragma Import (C, Get_Logbook_Id, "GET_LOGBOOK_ID");
--     pragma Import (C, Get_Logbook_Status, "GET_LOGBOOK_STATUS");
  end APEX.Logbook_System;
-- GNAT DOESN'T LIKE TWO PACKAGES IN ONE SPEC, MAY WANT TO REVISIT AND MAKE SEPARATE ADS FILE FOR THE FOLLOWING
--  -- Package rename for compatibility with Ada 83 binding --
--  with APEX.Logbook_System;
--  package APEX_LOGBOOK_SYSTEM renames APEX.Logbook_System;

