-- --------------------------------------------------------------------------
-- --
-- Sampling port Data Structures constant and type definitions and --
-- management services --
-- --
-- --------------------------------------------------------------------------
--
-- Sampling port data types.
--
-- Note: Data types APEX_BYTE, APEX_INTEGER, APEX_UNSIGNED, and
-- APEX_LONG_INTEGER, are defined in ARINC 653, Part 1. They are
-- considered to be acceptable for use in this context.
--
--
with Interfaces.C;
package APEX.Sampling_Ports.Data is
   type APEX_Char is new Interfaces.C.signed_char;
   type APEX_Short_Integer is new Interfaces.C.short;
   type APEX_Word is new Interfaces.Unsigned_16;
   type APEX_Unsigned_Long is new Interfaces.C.unsigned_long;
   --
   -- Definition of floats is implementation dependent.
   -- APEX_FLOAT must be a 32-bit IEEE 754 compliant single.
   -- APEX_DOUBLE must be a 64-bit IEEE 754 compliant double.
   --
   type APEX_Float is new Interfaces.C.C_float; -- 32-bit float
   type APEX_Double is new Interfaces.C.double; -- 64-bit float
   --
   -- Enumerated values.
   --
   type APEX_Enum8 is new APEX_Byte; -- 8-bit enumeration
   type APEX_Enum16 is new APEX_Word; -- 16-bit enumeration
   type APEX_Enum32 is new APEX_Unsigned; -- 32-bit enumeration
   -- Function Status (FS) state definitions
   -- These values should not be modified.
   type FS_Status is (
                      No_Data,
                      Normal_Operation,
                      Functional_Test,
                      No_Computed_Data);
   for FS_Status use (
                      No_Data => 16#00#,
                      Normal_Operation => 16#03#,
                      Functional_Test => 16#0C#,
                      No_Computed_Data => 16#30#);
   for FS_Status'Size use 8;
   type FSS_Type is record
      Fs1 : FS_Status;
      Fs2 : FS_Status;
      Fs3 : FS_Status;
      Fs4 : FS_Status;
   end record;
   type APEX_CRC is new APEX_Unsigned;
   type Char_Array is array (APEX_Unsigned range <>) of Character;
   type APEX_String( Size : APEX_Unsigned) is record
      Data : Char_Array (1..size);
   end record;
--  private
--     pragma Convention (C, FSS_Type);
--     pragma Convention (C, APEX_String);
end APEX.Sampling_Ports.Data;
-- GNAT DOESN'T LIKE TWO PACKAGES IN ONE SPEC, MAY WANT TO REVISIT AND MAKE SEPARATE ADS FILE FOR THE FOLLOWING
-- Package rename for compatibility with Ada 83 binding --
--  with APEX.Sampling_Ports.Data;
--  package APEX_SAMPLING_PORTS_DATA renames APEX.Sampling_Ports.Data;
