-- -----------------------------------------------------------------------
-- --
-- SAP Ports constant and type definitions and management services --
-- --
-- -----------------------------------------------------------------------
with Interfaces;
with APEX.Processes;

package APEX.SAP_Ports is
   Max_Number_Of_SAP_Ports : constant := 1024; -- Implementation dependent
   -- Partition scope
   type SAP_Port_Name_Type is new Name_Type;
   type SAP_Port_Id_Type is new APEX_Integer;
   type Apex_Unsigned_Short is new Interfaces.Unsigned_16;
   type Addr_Spare_Type is array(1..8) of APEX_Byte;
   type SAP_Address_Type is record
      Addr_Length : APEX_Byte;
      Addr_Type : APEX_Byte;
      UDP_Port_Number : APEX_Unsigned_Short; -- 16 bits
      IP_Address : APEX_Unsigned; -- 32 bits
      Addr_Spare : Addr_Spare_Type;
   end record;
   -- Valid address types
   SAP_ADDR_TYPE_INET : constant := 1;
   SAP_ADDR_TYPE_UNKNOWN : constant := 0;
   type SAP_Port_Status_Type is record
      Nb_Message : Message_Range_Type;
      Max_Nb_Message : Message_Range_Type;
      Max_Message_Size : Message_Size_Type;
      Port_Direction : Port_Direction_Type;
      Waiting_Processes : APEX.Processes.Waiting_Range_Type;
      Current_Dest_Addr : SAP_Address_Type;
      Current_Src_Addr : SAP_Address_Type;
   end record;
   procedure Create_SAP_Port
     (SAP_Port_Name : in SAP_Port_Name_Type;
      Max_Message_Size : in Message_Size_Type;
      Max_Nb_Message : in Message_Range_Type;
      Port_Direction : in Port_Direction_Type;
      Queuing_Discipline : in Queuing_Discipline_Type;
      SAP_Port_Id : out SAP_Port_Id_Type;
      Return_Code : out Return_Code_Type);
   procedure Send_SAP_Message
     (SAP_Port_Id : in SAP_Port_Id_Type;
      Message_Addr : in Message_Addr_Type;
      Length : in Message_Size_Type;
      Time_Out : in System_Time_Type;
      Destination_Addr : in SAP_Address_Type;
      Return_Code : out Apex.Return_Code_Type);
   procedure Receive_SAP_Message
     (SAP_Port_Id : in SAP_Port_Id_Type;
      Time_Out : in System_Time_Type;
      Message_Addr : in Message_Addr_Type;
      -- The message address is passed IN,
      -- although the respective message is passed OUT
      Length : out Message_Size_Type;
      Source_Addr : out SAP_Address_Type;
      Return_Code : out Return_Code_Type);
   procedure Get_SAP_Port_Id
     (SAP_Port_Name : in SAP_Port_Name_Type;
      SAP_Port_Id : out SAP_Port_Id_Type;
      Return_Code : out Return_Code_Type);
   procedure Get_SAP_Port_Status
     (SAP_Port_Id : in SAP_Port_Id_Type;
      SAP_Port_Status : out SAP_Port_Status_Type;
      Return_Code : out Return_Code_Type);
   procedure Send_Extended_SAP_Message
     (SAP_Port_Id : in SAP_Port_Id_Type;
      Message_Addr : in Message_Addr_Type;
      Length : in Message_Size_Type;
      Time_Out : in System_Time_Type;
      Source_Addr : in SAP_Address_Type;
      Destination_Addr : in SAP_Address_Type;
      Return_Code : out Return_Code_Type);
   procedure Receive_Extended_SAP_Message
     (SAP_Port_Id : in SAP_Port_Id_Type;
      Time_Out : in System_Time_Type;
      Message_Addr : in Message_Addr_Type;
      -- The message address is passed IN, although the respective message is
      -- passed OUT
      Length : out Message_Size_Type;
      Source_Addr : out SAP_Address_Type;
      Destination_Addr : out SAP_Address_Type;
      Return_Code : out Return_Code_Type);
--  private
     pragma Convention (C, SAP_Address_Type);
     pragma Convention (C, SAP_Port_Status_Type);
-- FOR NOW WE ARE GENERATING ADA BODIES FOR THESE...MAY WANT TO REVISIT AND USE C objects instead
--     pragma Import (C, Create_SAP_Port, "CREATE_SAP_PORT");
--     pragma Import (C, Send_SAP_Message, "SEND_SAP_MESSAGE");
--     pragma Import (C, Receive_SAP_Message, "RECEIVE_SAP_MESSAGE");
--     pragma Import (C, Get_SAP_Port_Id, "GET_SAP_PORT_ID");
--     pragma Import (C, Get_SAP_Port_Status, "GET_SAP_PORT_STATUS");
--     pragma Import (C, Send_Extended_SAP_Message, "SEND_EXTENDED_SAP_MESSAGE");
--     pragma Import (C, Receive_Extended_SAP_Message,
--                    "RCEIVE_EXTENDED_SAP_MESSAGE");
end APEX.SAP_Ports;
-- GNAT DOESN'T LIKE TWO PACKAGES IN ONE SPEC, MAY WANT TO REVISIT AND MAKE SEPARATE ADS FILE FOR THE FOLLOWING
--  -- Package rename for compatibility with Ada 83 binding --
--  with APEX.SAP_Ports;
--  package APEX_SAP_PORTS renames APEX.SAP_Ports;
