-- -------------------------------------------------------------------- --
-- --
-- Queuing Port List type definitions and management services --
-- --
-- ------------------------------------------------------------------- --
with Interfaces;
with APEX.Queuing_Ports; 
-- Following line not in Standard, but is needed for GNAT to compile...
use APEX.Queuing_Ports;

package APEX.Queuing_Port_List is
    -- Types
    type Port_Action_Type is (Inactive, Active);
    type Reset_List_Type is (Reset, Continue);
    type List_Size_Type is new APEX_Integer
         range 1..System_Limit_Number_Of_Queuing_Ports;
    type Queuing_Port_List_ID_Type is new APEX_Integer;
    type Queuing_Port_List_Name_Type is new Name_Type;

    -- Services

    procedure Create_Queuing_Port_List
        (Queuing_Port_List_Name : in Queuing_Port_List_Name_Type;
         Max_List_Size : in List_Size_Type;
         Queuing_Port_List_ID : out Queuing_Port_List_ID_Type;
         Return_Code : out Return_Code_Type);

    procedure Add_Port_To_Queuing_Port_List
        (Queuing_Port_List_ID : in Queuing_Port_List_ID_Type;
         Queuing_Port_ID : in APEX.Queuing_Ports.Queuing_Port_ID_Type;
         Port_Action : in Port_Action_Type;
         Return_Code : out Return_Code_Type);

    procedure Set_Port_Action_In_Queuing_Port_List
        (Queuing_Port_List_ID : in Queuing_Port_List_ID_Type;
         Queuing_Port_ID : in Queuing_Port_ID_Type;
         Port_Action : in Port_Action_Type;
         Return_Code : out Return_Code_Type);

    procedure Get_Queuing_Port_Action_Status
        (Queuing_Port_List_ID : in Queuing_Port_List_ID_Type;
         Queuing_Port_ID : in Queuing_Port_ID_Type;
         Port_Action_Status : out Port_Action_Type;
         Return_Code : out Return_Code_Type);

    procedure Receive_Message_From_Queuing_Port_List
        (Queuing_Port_List_ID : in Queuing_Port_List_ID_Type;
         Time_Out : in System_Time_Type;
         Message_Addr : in Message_Addr_Type;
         Buffer_Size : in Message_Size_Type;
         Reset_List : in Reset_List_Type;
         Queuing_Port_ID : out Queuing_Port_ID_Type;
         Length : out Message_Size_Type;
         Return_Code : out Return_Code_Type);

    procedure Wait_For_Message_From_Queuing_Port_List
        (Queuing_Port_List_ID : in Queuing_Port_List_ID_Type;
         Time_Out : in System_Time_Type;
         Reset_List : in Reset_List_Type;
         Queuing_Port_ID : out Queuing_Port_ID_Type;
         Return_Code : out Return_Code_Type);

--private
    pragma Convention (C, Port_Action_Type);
    pragma Convention (C, Reset_List_Type);
-- FOR NOW WE ARE GENERATING ADA BODIES FOR THESE...MAY WANT TO REVISIT AND USE C objects instead
--    pragma Import (C, Create_Queuing_Port_List, "CREATE_QUEUING_PORT_LIST");
--    pragma Import (C, Add_Port_To_Queuing_Port_List,
--                    "ADD_PORT_TO_QUEUING_PORT_LIST");
--    pragma Import (C, Set_Port_Action_In_Queuing_Port_List,
--                    "SET_PORT_ACTION_IN_QUEUING_PORT_LIST");
--    pragma Import (C, Get_Queuing_Port_Action_Status,
--                    "GET_QUEUING_PORT_ACTION_STATUS");
--    pragma Import (C, Receive_Message_From_Queuing_Port_List,
--                    "RECEIVE_MESSAGE_FROM_QUEUING_PORT_LIST");
--    pragma Import (C, Wait_For_Message_From_Queuing_Port_List,
--                    "WAIT_FOR_MESSAGE_FROM_QUEUING_PORT_LIST");

end APEX.Queuing_Port_List;

-- GNAT DOESN'T LIKE TWO PACKAGES IN ONE SPEC, MAY WANT TO REVISIT AND MAKE SEPARATE ADS FILE FOR THE FOLLOWING
-- Package rename for compatibility with Ada 83 binding --
--with APEX.Queuing_Port_List;
--package APEX_QUEUING_PORT_LIST renames APEX.Queuing_Port_List;
