#ifndef GOLDSTANDARDLIBRARIES_CPP_INCLUDE_FACE_HMFM_HPP
#define GOLDSTANDARDLIBRARIES_CPP_INCLUDE_FACE_HMFM_HPP

/************************************************************************************
 * Copyright (c) Vanderbilt University, 2020                                        *
 *                                                                                  *
 * ALL RIGHTS RESERVED, UNLESS OTHERWISE STATED                                     *
 *                                                                                  *
 * This software, authored by Vanderbilt University under a contract awarded to and *
 * managed by Alion Science and Technology, was funded by the U.S. Government under *
 * Contract No. FA8075-14-D-0014 and the U.S. Government has unlimited rights in    *
 * this software.  An unlimited rights license means that the U.S. Government can   *
 * use, modify, reproduce, release or disclose computer software in whole or in     *
 * part, in any manner, and for any purpose whatsoever, and to have or authorize    *
 * others to do so.                                                                 *
 *                                                                                  *
 * Vanderbilt University disclaims all warranties with regard to this software,     *
 * including all implied warranties of merchantability and fitness. In no event     *
 * shall Vanderbilt University be liable for any special, indirect or consequential *
 * damages or any damages whatsoever resulting from loss of use, data or profits,   *
 * whether in an action of contract, negligence or other tortious action, arising   *
 * out of or in connection with the use or performance of this software.            *
 *                                                                                  *
 * May contain products produced under DoD SENSIAC contract HC104705D4000 under the *
 * sponsorship of the Defense Technical Information Center,                         *
 * ATTN: DTIC-AI, 8723 John J. Kingman Rd., Ste 0944, Fort Belvoir, VA 22060-6218.  *
 * SENSIAC is a DoD Information Analysis Center Sponsored by the Defense Technical  *
 * Information Center.                                                              *
 *                                                                                  *
 * Filename: goldStandardLibraries/C++/include/FACE/HMFM.hpp                        *
 *                                                                                  *
 * Conformance Test Suite Version 2.1.1r9 supporting FACE Standard Version 2.1.1    *
 *                                                                                  *
 * This file is generated and may be overwritten.                                   *
 *                                                                                  *
 * Release Date: 29-Aug-2023                                                        *
 ************************************************************************************/

#include <FACE/common.hpp>


namespace FACE
{

    namespace HMFM
    {
        typedef FACE::Long FAULT_MESSAGE_SIZE_TYPE;

        typedef void * FAULT_MESSAGE_ADDRESS_TYPE;

        typedef FACE::Long THREAD_ID_TYPE;

        const FACE::HMFM::FAULT_MESSAGE_SIZE_TYPE FAULT_MESSAGE_MAXIMUM_SIZE = 128;

        typedef FACE::UnsignedLong STACK_SIZE_TYPE;

        typedef FACE::Char FAULT_MESSAGE_TYPE[FACE::HMFM::FAULT_MESSAGE_MAXIMUM_SIZE];
        enum FAULT_CODE_TYPE {
            DEADLINE_MISSED,
            APPLICATION_ERROR,
            NUMERIC_ERROR,
            ILLEGAL_REQUEST,
            STACK_OVERFLOW,
            MEMORY_VIOLATION,
            HARDWARE_FAULT,
            POWER_FAIL
        };

        struct FAULT_STATUS_TYPE {
            FACE::HMFM::FAULT_CODE_TYPE CODE;
            FACE::HMFM::FAULT_MESSAGE_TYPE MESSAGE;
            FACE::HMFM::FAULT_MESSAGE_SIZE_TYPE LENGTH;
            FACE::HMFM::THREAD_ID_TYPE FAILED_THREAD_ID;
            FACE::SYSTEM_ADDRESS_TYPE FAILED_ADDRESS;

        };

        typedef void (*FAULT_HANDLER_ENTRY_TYPE) (void);


        void Initialize (
                /*    out */ FACE::RETURN_CODE_TYPE &return_code
            );

        void Create_Fault_Handler (
                /* in     */ FACE::HMFM::FAULT_HANDLER_ENTRY_TYPE entry_point,
                /* in     */ FACE::HMFM::STACK_SIZE_TYPE stack_size,
                /*    out */ FACE::RETURN_CODE_TYPE &return_code
            );

        void Report_Application_Message (
                /* in     */ FACE::HMFM::FAULT_MESSAGE_ADDRESS_TYPE fault,
                /* in     */ FACE::HMFM::FAULT_MESSAGE_SIZE_TYPE length,
                /*    out */ FACE::RETURN_CODE_TYPE &return_code
            );

        void Get_Fault_Status (
                /*    out */ FACE::HMFM::FAULT_STATUS_TYPE &fault,
                /*    out */ FACE::RETURN_CODE_TYPE &return_code
            );

        void Raise_Application_Fault (
                /* in     */ FACE::HMFM::FAULT_CODE_TYPE code,
                /* in     */ FACE::HMFM::FAULT_MESSAGE_ADDRESS_TYPE message,
                /* in     */ FACE::HMFM::FAULT_MESSAGE_SIZE_TYPE length,
                /*    out */ FACE::RETURN_CODE_TYPE &return_code
            );


    } // namespace HMFM


} // namespace FACE



#endif /* #ifndef GOLDSTANDARDLIBRARIES_CPP_INCLUDE_FACE_HMFM_HPP */
