/*------------------------------------------------------------------*/
/*                                                                  */
/* Unless specified as implementation dependent, the values         */
/* specified in this appendix should be implemented as defined.     */
/*                                                                  */
/*------------------------------------------------------------------*/
/*------------------------------------------------------------------*/
/*                                                                  */
/* File System constant and type definitions and management services*/
/*                                                                  */
/*------------------------------------------------------------------*/
#ifdef __cplusplus
extern "C" {
#endif /* __cplusplus */

#ifndef APEX_FILE_SYSTEM
#define APEX_FILE_SYSTEM

#include "APEX_TYPES.h"

/* from APEX_TYPES: */
/* typedef unsigned char APEX_BYTE; */
/* typedef long APEX_INTEGER; */
/* typedef long long APEX_LONG_INTEGER; */
/* typedef enum {NO_ERROR, ...} RETURN_CODE_TYPE; */
/* typedef APEX_BYTE * MESSAGE_ADDR_TYPE; */
/* typedef APEX_INTEGER MESSAGE_SIZE_TYPE; */
/* typedef APEX_LONG_INTEGER SYSTEM_TIME_TYPE; */

#define SYSTEM_LIMIT_FILE_SIZE 0x40000000 /* file system scope */
/* implementation dependent */
#define MAX_FILE_SIZE SYSTEM_LIMIT_FILE_SIZE
#define MAX_FILE_NAME_LENGTH 512 /* 511 char + null */
typedef char FILE_NAME_TYPE [MAX_FILE_NAME_LENGTH];
#define MAX_DIRECTORY_ENTRY_LENGTH 64 /* 63 char + null */
typedef char DIRECTORY_ENTRY_TYPE [MAX_DIRECTORY_ENTRY_LENGTH];
typedef APEX_INTEGER FILE_ID_TYPE;
typedef APEX_INTEGER DIRECTORY_ID_TYPE;
typedef APEX_INTEGER FILE_SIZE_TYPE;
/* 32-bit or 64-bit (APEX_LONG_INTEGER) */
/* implementation dependent */
typedef APEX_INTEGER FILE_ERRNO_TYPE;
typedef enum { READ = 0, READ_WRITE = 1 } FILE_MODE_TYPE;
typedef enum { SEEK_SET = 0, SEEK_CUR = 1, SEEK_END = 2 } FILE_SEEK_TYPE;
typedef
enum {
  FILE_ENTRY = 0,
  DIRECTORY_ENTRY = 1,
  OTHER_ENTRY = 2,
  END_OF_DIRECTORY = 3
} ENTRY_KIND_TYPE;

typedef enum { UNSET = 0, SET = 1 } TIME_SET_TYPE;
typedef enum { VOLATILE = 0, NONVOLATILE = 1, REMOTE = 2} MEDIA_TYPE;
typedef
struct {
  APEX_INTEGER TM_SEC; /* seconds after the minute [0,59] */
  APEX_INTEGER TM_MIN; /* minutes after the hour [0,59] */
  APEX_INTEGER TM_HOUR; /* hours since midnight [0,23] */
  APEX_INTEGER TM_MDAY; /* day of the month [1,31] */
  APEX_INTEGER TM_MON; /* months since January [0,11] */
  APEX_INTEGER TM_YEAR; /* years since 1900 */
  APEX_INTEGER TM_WDAY; /* days since Sunday [0,6] */
  APEX_INTEGER TM_YDAY; /* days since January 1 [0,365] */
  APEX_INTEGER TM_ISDST; /* Daylight Savings Time flag */
  TIME_SET_TYPE TM_IS_SET; /* time has been set */
} COMPOSITE_TIME_TYPE;

typedef
struct {
  COMPOSITE_TIME_TYPE CREATE_TIME;
  COMPOSITE_TIME_TYPE LAST_UPDATE;
  FILE_SIZE_TYPE POSITION;
  FILE_SIZE_TYPE SIZE;
  APEX_INTEGER NB_OF_CHANGES;
  APEX_INTEGER NB_OF_WRITE_ERRORS;
} FILE_STATUS_TYPE;

typedef
struct {
  APEX_LONG_INTEGER TOTAL_BYTES;
  APEX_LONG_INTEGER USED_BYTES; 
  APEX_LONG_INTEGER FREE_BYTES;
  APEX_LONG_INTEGER MAX_ATOMIC_SIZE;
  APEX_INTEGER BLOCK_SIZE;
  FILE_MODE_TYPE ACCESS_RIGHTS;
  MEDIA_TYPE MEDIA;
} VOLUME_STATUS_TYPE;

/* ERRNO Error codes */
/* The values defined by the ERRNO constants are implementation dependent */
#define EPERM 1 /* Operation not permitted */
#define ENOENT 2 /* No such file or directory */
#define EIO 5 /* Input/output error */
#define EBADF 9 /* Bad file descriptor */
#define EACCES 13 /* Permission denied */
#define EBUSY 16 /* Resource busy */
#define EEXIST 17 /* File exists */
#define ENOTDIR 20 /* Not a volume or directory */
#define EISDIR 21 /* Is a directory */
#define EINVAL 22 /* Invalid argument */
#define EMFILE 24 /* Too many open files by the current */
/* partition */
#define EFBIG 27 /* File too large */
#define ENOSPC 28 /* No space left on volume */
#define EROFS 30 /* Storage device containing file is */
/* currently write protected */
#define ENAMETOOLONG 78 /* Filename too long */
#define EOVERFLOW 79 /* Current file position beyond end */
/* of file */
#define ENOTEMPTY 93 /* Directory not empty */
#define ESTALE 151 /* file or directory ID which an */
/* application has been using is no */
/* longer valid */
extern void OPEN_NEW_FILE (
  /*in */ FILE_NAME_TYPE FILE_NAME,
  /*out */ FILE_ID_TYPE *FILE_ID,
  /*out */ RETURN_CODE_TYPE *RETURN_CODE,
  /*inout*/ FILE_ERRNO_TYPE *ERRNO );
extern void OPEN_FILE (
  /*in */ FILE_NAME_TYPE FILE_NAME,
  /*in */ FILE_MODE_TYPE FILE_MODE,
  /*out */ FILE_ID_TYPE *FILE_ID,
  /*out */ RETURN_CODE_TYPE *RETURN_CODE,
  /*inout*/ FILE_ERRNO_TYPE *ERRNO );
extern void CLOSE_FILE (
  /*in */ FILE_ID_TYPE FILE_ID,
  /*out */ RETURN_CODE_TYPE *RETURN_CODE,
  /*inout*/ FILE_ERRNO_TYPE *ERRNO );
extern void READ_FILE (
  /*in */ FILE_ID_TYPE FILE_ID,
  /*in */ MESSAGE_ADDR_TYPE MESSAGE_ADDR, /* by reference */
  /*in */ MESSAGE_SIZE_TYPE IN_LENGTH,
  /*out */ MESSAGE_SIZE_TYPE *OUT_LENGTH,
  /*out */ RETURN_CODE_TYPE *RETURN_CODE,
  /*inout*/ FILE_ERRNO_TYPE *ERRNO );
extern void WRITE_FILE (
  /*in */ FILE_ID_TYPE FILE_ID,
  /*in */ MESSAGE_ADDR_TYPE MESSAGE_ADDR, /* by reference */
  /*in */ MESSAGE_SIZE_TYPE LENGTH,
  /*out */ RETURN_CODE_TYPE *RETURN_CODE,
  /*inout*/ FILE_ERRNO_TYPE *ERRNO );
extern void SEEK_FILE (
  /*in */ FILE_ID_TYPE FILE_ID,
  /*in */ FILE_SIZE_TYPE OFFSET,
  /*in */ FILE_SEEK_TYPE WHENCE,
  /*out */ FILE_SIZE_TYPE *POSITION,
  /*out */ RETURN_CODE_TYPE *RETURN_CODE,
  /*inout*/ FILE_ERRNO_TYPE *ERRNO );
extern void RESIZE_FILE (
  /*in */ FILE_ID_TYPE FILE_ID,
  /*in */ FILE_SIZE_TYPE NEW_SIZE,
  /*out */ RETURN_CODE_TYPE *RETURN_CODE,
  /*inout*/ FILE_ERRNO_TYPE *ERRNO );
extern void SYNC_FILE (
  /*in */ FILE_ID_TYPE FILE_ID,
  /*out */ RETURN_CODE_TYPE *RETURN_CODE,
  /*inout*/ FILE_ERRNO_TYPE *ERRNO );
extern void REMOVE_FILE (
  /*in */ FILE_NAME_TYPE FILE_NAME,
  /*out */ RETURN_CODE_TYPE *RETURN_CODE,
  /*inout*/ FILE_ERRNO_TYPE *ERRNO );
extern void RENAME_FILE (
  /*in */ FILE_NAME_TYPE OLD_FILE_NAME,
  /*in */ FILE_NAME_TYPE NEW_FILE_NAME,
  /*out */ RETURN_CODE_TYPE *RETURN_CODE,
  /*inout*/ FILE_ERRNO_TYPE *ERRNO );
extern void GET_FILE_STATUS (
  /*in */ FILE_ID_TYPE FILE_ID,
  /*out */ FILE_STATUS_TYPE *FILE_STATUS,
  /*out */ RETURN_CODE_TYPE *RETURN_CODE,
  /*inout*/ FILE_ERRNO_TYPE *ERRNO );
extern void GET_VOLUME_STATUS (
  /*in */ FILE_NAME_TYPE FILE_NAME,
  /*out */ VOLUME_STATUS_TYPE *VOLUME_STATUS,
  /*out */ RETURN_CODE_TYPE *RETURN_CODE,
  /*inout*/ FILE_ERRNO_TYPE *ERRNO );
extern void OPEN_DIRECTORY (
  /*in */ FILE_NAME_TYPE DIRECTORY_NAME,
  /*out */ DIRECTORY_ID_TYPE *DIRECTORY_ID,
  /*out */ RETURN_CODE_TYPE *RETURN_CODE,
  /*inout*/ FILE_ERRNO_TYPE *ERRNO );
extern void CLOSE_DIRECTORY (
  /*in */ DIRECTORY_ID_TYPE DIRECTORY_ID,
  /*out */ RETURN_CODE_TYPE *RETURN_CODE,
  /*inout*/ FILE_ERRNO_TYPE *ERRNO );
extern void READ_DIRECTORY (
  /*in */ DIRECTORY_ID_TYPE DIRECTORY_ID,
  /*out */ DIRECTORY_ENTRY_TYPE *ENTRY_NAME,
  /*out */ ENTRY_KIND_TYPE *ENTRY_KIND,
  /*out */ RETURN_CODE_TYPE *RETURN_CODE,
  /*inout*/ FILE_ERRNO_TYPE *ERRNO );
extern void REWIND_DIRECTORY (
  /*in */ DIRECTORY_ID_TYPE DIRECTORY_ID,
  /*out */ RETURN_CODE_TYPE *RETURN_CODE,
  /*inout*/ FILE_ERRNO_TYPE *ERRNO );
extern void MAKE_DIRECTORY (
  /*in */ FILE_NAME_TYPE DIRECTORY_NAME,
  /*out */ RETURN_CODE_TYPE *RETURN_CODE,
  /*inout*/ FILE_ERRNO_TYPE *ERRNO );
extern void REMOVE_DIRECTORY (
  /*in */ FILE_NAME_TYPE DIRECTORY_NAME,
  /*out */ RETURN_CODE_TYPE *RETURN_CODE,
  /*inout*/ FILE_ERRNO_TYPE *ERRNO );
extern void SYNC_DIRECTORY (
  /*in */ DIRECTORY_ID_TYPE DIRECTORY_ID,
  /*out */ RETURN_CODE_TYPE *RETURN_CODE,
  /*inout*/ FILE_ERRNO_TYPE *ERRNO );
extern void RENAME_DIRECTORY (
  /*in */ FILE_NAME_TYPE OLD_DIRECTORY_NAME,
  /*in */ FILE_NAME_TYPE NEW_DIRECTORY_NAME,
  /*out */ RETURN_CODE_TYPE *RETURN_CODE,
  /*inout*/ FILE_ERRNO_TYPE *ERRNO );
#endif
/*------------------------------------------------------------------*/
/*                                                                  */
/* Sampling port data structures constant and type definitions and  */
/* management services                                              */
/*                                                                  */
/*------------------------------------------------------------------*/
#ifndef APEX_SP_DATA
#define APEX_SP_DATA
/* Controls whether to include the header file that defines APEX_TYPES */
#include <APEX_TYPES.h>
/*
** Sampling port data types.
**
** Note: Data types APEX_BYTE, APEX_INTEGER, APEX_UNSIGNED, and
** APEX_LONG_INTEGER, are defined in ARINC 653Part 1. They are
** considered to be acceptable for use in this context.
**
** Implementation dependent:
**
** The type declaration associated with each typdef statement may be
** implementation dependent. Each must define a type that is consistent
** with the standard definition, in size, structure, and behavior.
*/
typedef char APEX_CHAR; /* 8-bit signed integer */
typedef short int APEX_SHORT_INTEGER ; /* 16-bit signed integer */
typedef unsigned short int APEX_WORD ; /* 16-bit unsigned integer */
typedef unsigned long long APEX_UNSIGNED_LONG ; /* 64-bit unsigned integer */
typedef float APEX_FLOAT ; /* 32-bit float */
typedef double APEX_DOUBLE ; /* 64-bit float */
/*
** Enumerated values.
** These use integers to mimic enumerated data
** Legal values are defined by #define statements
** (see FS value declarations)
*/
typedef APEX_CHAR APEX_ENUM8 ; /* 8-bit enumeration */
typedef APEX_SHORT_INTEGER APEX_ENUM16 ; /* 16-bit enumeration */
typedef APEX_INTEGER APEX_ENUM32 ; /* 32-bit enumeration */
typedef APEX_ENUM8 FS_TYPE ; /* Function Status */
typedef
struct
{
FS_TYPE fs1 : 8;
FS_TYPE fs2 : 8;
FS_TYPE fs3 : 8;
FS_TYPE fs4 : 8;
} FSS_TYPE ; /* 32 bits */
/* Function Status (FS) state definitions */
/* These values should not be modified. */
#define NO_DATA 0x00
#define NORMAL_OPERATION 0x03
#define FUNCTIONAL_TEST 0x0C
#define NO_COMPUTED_DATA 0x30
/*
** String data type.
** There is not a generalized form of the APEX string type that is
** expressible as a C typedef. For a string of a given length,
** a structure should be defined similar to this example:
**
** #define MY_STRING_LENGTH 50
** typedef
** struct
** {
** APEX_WORD length;
** APEX_CHAR data[MY_STRING_LENGTH]; // length of specific string
** } MY_APEX_STRING;
*/
#endif
/*------------------------------------------------------------------*/
/*                                                                  */
/* Multiple Module Schedules constant and type definitions          */
/* and management services                                          */
/*                                                                  */
/*------------------------------------------------------------------*/
#ifndef APEX_MULTIPLE_MODULE_SCHEDULES
#define APEX_MULTIPLE_MODULE_SCHEDULES
typedef APEX_INTEGER SCHEDULE_ID_TYPE;
typedef NAME_TYPE SCHEDULE_NAME_TYPE;
typedef
struct {
SYSTEM_TIME_TYPE TIME_OF_LAST_SCHEDULE_SWITCH;
SCHEDULE_ID_TYPE CURRENT_SCHEDULE;
SCHEDULE_ID_TYPE NEXT_SCHEDULE;
} SCHEDULE_STATUS_TYPE;
extern void SET_MODULE_SCHEDULE (
  /*in */ SCHEDULE_ID_TYPE SCHEDULE_ID,
  /*out*/ RETURN_CODE_TYPE *RETURN_CODE);
extern void GET_MODULE_SCHEDULE_STATUS (
  /*out*/ SCHEDULE_STATUS_TYPE *SCHEDULE_STATUS,
  /*out*/ RETURN_CODE_TYPE *RETURN_CODE);
extern void GET_MODULE_SCHEDULE_ID (
  /*in */ SCHEDULE_NAME_TYPE SCHEDULE_NAME,
  /*out*/ SCHEDULE_ID_TYPE *SCHEDULE_ID,
  /*out*/ RETURN_CODE_TYPE *RETURN_CODE);
#endif
/*------------------------------------------------------------------*/
/*                                                                  */
/* Logbook system constant and type definitions and management      */
/* services                                                         */
/*                                                                  */
/*------------------------------------------------------------------*/
#ifndef APEX_LOGBOOK_SYSTEM
#define APEX_LOGBOOK_SYSTEM
typedef APEX_INTEGER LOGBOOK_ID_TYPE;
typedef NAME_TYPE LOGBOOK_NAME_TYPE;
typedef
enum {
ABORTED = 0,
IN_PROGRESS = 1,
COMPLETE = 2
} WRITE_STATUS_TYPE;
typedef
struct {
APEX_INTEGER MAX_MESSAGE_SIZE;
APEX_INTEGER MAX_NB_LOGGED_MESSAGES;
APEX_INTEGER MAX_NB_IN_PROGRESS_MESSAGES;
APEX_INTEGER NB_LOGGED_MESSAGES;
APEX_INTEGER NB_IN_PROGRESS_MESSAGES;
APEX_INTEGER NB_ABORTED_MESSAGES;
} LOGBOOK_STATUS_TYPE;
extern void CREATE_LOGBOOK (
  /*in */ LOGBOOK_NAME_TYPE LOGBOOK_NAME,
  /*in */ MESSAGE_SIZE_TYPE MAX_MESSAGE_SIZE,
  /*in */ MESSAGE_RANGE_TYPE MAX_NB_LOGGED_MESSAGES,
  /*in */ MESSAGE_RANGE_TYPE MAX_NB_IN_PROGRESS_MESSAGES,
  /*out*/ LOGBOOK_ID_TYPE *LOGBOOK_ID,
  /*out*/ RETURN_CODE_TYPE *RETURN_CODE);
extern void WRITE_LOGBOOK (
  /*in */ LOGBOOK_ID_TYPE LOGBOOK_ID,
  /*in */ MESSAGE_ADDR_TYPE MESSAGE_ADDR,
  /*in */ MESSAGE_SIZE_TYPE LENGTH,
  /*out*/ RETURN_CODE_TYPE *RETURN_CODE);
extern void OVERWRITE_LOGBOOK (
  /*in */ LOGBOOK_ID_TYPE LOGBOOK_ID,
  /*in */ MESSAGE_ADDR_TYPE MESSAGE_ADDR,
  /*in */ MESSAGE_SIZE_TYPE LENGTH,
  /*out*/ RETURN_CODE_TYPE *RETURN_CODE);
extern void READ_LOGBOOK (
  /*in */ LOGBOOK_ID_TYPE LOGBOOK_ID,
  /*in */ MESSAGE_RANGE_TYPE LOGBOOK_ENTRY,
  /*in */ MESSAGE_ADDR_TYPE MESSAGE_ADDR,
  /*out*/ MESSAGE_SIZE_TYPE *LENGTH,
  /*out*/ WRITE_STATUS_TYPE *WRITE_STATUS,
  /*out*/ RETURN_CODE_TYPE *RETURN_CODE);
extern void CLEAR_LOGBOOK (
  /*in */ LOGBOOK_ID_TYPE LOGBOOK_ID,
  /*out*/ RETURN_CODE_TYPE *RETURN_CODE);
extern void GET_LOGBOOK_ID (
  /*in */ LOGBOOK_NAME_TYPE LOGBOOK_NAME,
  /*out*/ LOGBOOK_ID_TYPE *LOGBOOK_ID,
  /*out*/ RETURN_CODE_TYPE *RETURN_CODE);
extern void GET_LOGBOOK_STATUS (
  /*in */ LOGBOOK_ID_TYPE LOGBOOK_ID,
  /*out*/ LOGBOOK_STATUS_TYPE *LOGBOOK_STATUS,
  /*out*/ RETURN_CODE_TYPE *RETURN_CODE);
#endif
/*------------------------------------------------------------------*/
/*                                                                  */
/* Sampling Port Extensions constant and type definitions and       */
/* management services                                              */
/*                                                                  */
/*------------------------------------------------------------------*/
#ifndef APEX_SAMPLING_PORT_EXTENSIONS
#define APEX_SAMPLING_PORT_EXTENSIONS
typedef
enum {
EMPTY_PORT = 0,
CONSUMED_MESSAGE = 1,
NEW_MESSAGE = 2
} UPDATED_TYPE;
typedef enum { STALE = 0, FRESH = 1 } AGE_TYPE;
typedef
struct {
SYSTEM_TIME_TYPE REFRESH_PERIOD;
SYSTEM_TIME_TYPE TIME_STAMP;
MESSAGE_SIZE_TYPE MAX_MESSAGE_SIZE;
PORT_DIRECTION_TYPE PORT_DIRECTION;
AGE_TYPE MESSAGE_AGE;
UPDATED_TYPE UPDATED;
} SAMPLING_PORT_CURRENT_STATUS_TYPE;
extern void READ_UPDATED_SAMPLING_MESSAGE (
  /*in */ SAMPLING_PORT_ID_TYPE SAMPLING_PORT_ID,
  /*in */ MESSAGE_ADDR_TYPE MESSAGE_ADDR,
  /*out*/ MESSAGE_SIZE_TYPE *LENGTH,
  /*out*/ UPDATED_TYPE *UPDATED,
  /*out*/ RETURN_CODE_TYPE *RETURN_CODE);
extern void GET_SAMPLING_PORT_CURRENT_STATUS (
  /*in */ SAMPLING_PORT_ID_TYPE SAMPLING_PORT_ID,
  /*out*/ SAMPLING_PORT_CURRENT_STATUS_TYPE
*SAMPLING_PORT_CURRENT_STATUS,
  /*out*/ RETURN_CODE_TYPE *RETURN_CODE);
extern void READ_SAMPLING_MESSAGE_CONDITIONAL (
  /*in */ SAMPLING_PORT_ID_TYPE SAMPLING_PORT_ID,
  /*in */ SYSTEM_TIME_TYPE REF_TIME_STAMP,
  /*in */ MESSAGE_ADDR_TYPE MESSAGE_ADDR,
  /*out*/ MESSAGE_SIZE_TYPE *LENGTH,
  /*out*/ SYSTEM_TIME_TYPE *TIME_STAMP,
  /*out*/ RETURN_CODE_TYPE *RETURN_CODE);
#endif
/*------------------------------------------------------------------*/
/*                                                                  */
/* SAP Ports constant and type definitions and management services  */
/*                                                                  */
/*------------------------------------------------------------------*/
#ifndef APEX_SAP_PORTS
#define APEX_SAP_PORTS
#define MAX_NUMBER_OF_SAP_PORTS 1024 /* Implementation dependent */
/* Partition scope */
typedef NAME_TYPE SAP_PORT_NAME_TYPE;
typedef APEX_INTEGER SAP_PORT_ID_TYPE;
#ifndef APEX_UNSIGNED_SHORT
/* Implementation dependent*/
typedef unsigned short int APEX_UNSIGNED_SHORT;/* 16-bit unsigned integer */
#endif
typedef struct SAP_ADDRESS_TYPE
{
APEX_BYTE Addr_Length; /* structure size */
APEX_BYTE Addr_Type; /* address type */
APEX_UNSIGNED_SHORT UDP_Port_Number; /* src/dst UDP port number */
APEX_UNSIGNED IP_Address; /* src/dst IP address */
APEX_BYTE Addr_Spare[8]; /* structure padding */
} SAP_ADDRESS_TYPE;
#define SAP_ADDR_TYPE_UNKNOWN 0 /* unknown address type */
#define SAP_ADDR_TYPE_INET 1 /* type is IP address type */
/* structure returned by SAP port status query */
typedef struct SAP_PORT_STATUS_TYPE
{
MESSAGE_RANGE_TYPE NB_MESSAGE;
MESSAGE_RANGE_TYPE MAX_NB_MESSAGE;
MESSAGE_SIZE_TYPE MAX_MESSAGE_SIZE;
PORT_DIRECTION_TYPE PORT_DIRECTION;
WAITING_RANGE_TYPE WAITING_PROCESSES;
SAP_ADDRESS_TYPE CURRENT_DEST_ADDR;
SAP_ADDRESS_TYPE CURRENT_SRC_ADDR;
} SAP_PORT_STATUS_TYPE;
extern void CREATE_SAP_PORT (
  /*in */ SAP_PORT_NAME_TYPE SAP_PORT_NAME,
  /*in */ MESSAGE_SIZE_TYPE MAX_MESSAGE_SIZE,
  /*in */ MESSAGE_RANGE_TYPE MAX_NB_MESSAGE,
  /*in */ PORT_DIRECTION_TYPE PORT_DIRECTION,
  /*in */ QUEUING_DISCIPLINE_TYPE QUEUING_DISCIPLINE,
  /*out*/ SAP_PORT_ID_TYPE *SAP_PORT_ID,
  /*out*/ RETURN_CODE_TYPE *RETURN_CODE );
extern void SEND_SAP_MESSAGE (
  /*in */ SAP_PORT_ID_TYPE SAP_PORT_ID,
  /*in */ MESSAGE_ADDR_TYPE MESSAGE_ADDR, /* by reference */
  /*in */ MESSAGE_SIZE_TYPE LENGTH,
  /*in */ SYSTEM_TIME_TYPE TIME_OUT,
  /*in */ SAP_ADDRESS_TYPE DESTINATION_ADDR,
  /*out*/ RETURN_CODE_TYPE *RETURN_CODE );
extern void RECEIVE_SAP_MESSAGE (
  /*in */ SAP_PORT_ID_TYPE SAP_PORT_ID,
  /*in */ SYSTEM_TIME_TYPE TIME_OUT,
  /*out*/ MESSAGE_ADDR_TYPE MESSAGE_ADDR,
  /*out*/ MESSAGE_SIZE_TYPE *LENGTH,
  /*out*/ SAP_ADDRESS_TYPE *SOURCE_ADDR,
  /*out*/ RETURN_CODE_TYPE *RETURN_CODE );
extern void GET_SAP_PORT_ID (
  /*in */ SAP_PORT_NAME_TYPE SAP_PORT_NAME,
  /*out*/ SAP_PORT_ID_TYPE *SAP_PORT_ID,
  /*out*/ RETURN_CODE_TYPE *RETURN_CODE);
extern void GET_SAP_PORT_STATUS (
  /*in */ SAP_PORT_ID_TYPE SAP_PORT_ID,
  /*in */ SAP_PORT_STATUS_TYPE *SAP_PORT_STATUS,
  /*in */ RETURN_CODE_TYPE *RETURN_CODE );
extern void SEND_EXTENDED_SAP_MESSAGE(
  /*in */ SAP_PORT_ID_TYPE SAP_PORT_ID,
  /*in */ MESSAGE_ADDR_TYPE MESSAGE_ADDR, /* by reference */
  /*in */ MESSAGE_SIZE_TYPE LENGTH,
  /*in */ SYSTEM_TIME_TYPE TIME_OUT,
  /*in */ SAP_ADDRESS_TYPE SOURCE_ADDR,
  /*in */ SAP_ADDRESS_TYPE DESTINATION_ADDR,
  /*out*/ RETURN_CODE_TYPE *RETURN_CODE );
extern void RECEIVE_EXTENDED_SAP_MESSAGE(
  /*in */ SAP_PORT_ID_TYPE SAP_PORT_ID,
  /*in */ SYSTEM_TIME_TYPE TIME_OUT,
  /*out*/ MESSAGE_ADDR_TYPE MESSAGE_ADDR,
  /*out*/ MESSAGE_SIZE_TYPE *LENGTH,
  /*out*/ SAP_ADDRESS_TYPE *SOURCE_ADDR,
  /*out*/ SAP_ADDRESS_TYPE *DESTINATION_ADDR,
  /*out*/ RETURN_CODE_TYPE *RETURN_CODE );
#endif
/*--------------------------------------------------------------------*/
/*                                                                    */
/* Name Service constant and type definitions and management services */
/*                                                                    */
/*--------------------------------------------------------------------*/
#ifndef APEX_NAME_SERVICE
#define APEX_NAME_SERVICE
typedef APEX_INTEGER NAME_SERVICE_ID_TYPE;
typedef NAME_TYPE NAME_SERVICE_NAME_TYPE;
typedef APEX_BYTE * NAME_SERVICE_ADDR_TYPE;
typedef APEX_INTEGER NAME_SERVICE_LENGTH_TYPE;

/* Following line is in standard, but conflicts with QUEUING_DISCIPLINE_TYPE
   DEFINED IN APEX_TYPES.h
enum {FIFO = 0, PRIORITY = 1) NAME_SERVICE_QUEUING_DISCIPLINE_TYPE ; */
#include "APEX_TYPES.h"
typedef QUEUING_DISCIPLINE_TYPE NAME_SERVICE_QUEUING_DISCIPLINE_TYPE;

extern void CREATE_NAME_SERVICE (
  /*in */ NAME_SERVICE_NAME_TYPE NAME_SERVICE_NAME,
  /*in */ NAME_SERVICE_QUEUING_DISCIPLINE_TYPE QUEUING_DISCIPLINE,
  /*out*/ NAME_SERVICE_ID_TYPE *SERVICE_ID,
  /*out*/ RETURN_CODE_TYPE *RETURN_CODE );

 /* HAD TO CHANGE NAME FROM MAX_NAME_LENGTH TO MAX_NAME_LENGTH_VAR to avoid confile in APEX_TYPES.h */
extern void GET_NAME (
  /*in */ NAME_SERVICE_ID_TYPE SERVICE_ID,
  /*in */ SAP_ADDRESS_TYPE SAP_ADDRESS,
  /*in */ SYSTEM_TIME_TYPE TIME_OUT,
  /*in */ NAME_SERVICE_ADDR_TYPE NAME_ADDR,
  /*in */ NAME_SERVICE_LENGTH_TYPE MAX_NAME_LENGTH_VAR,
  /*out*/ NAME_SERVICE_LENGTH_TYPE *NAME_LENGTH,
  /*out*/ RETURN_CODE_TYPE *RETURN_CODE );

extern void GET_ADDRESS (
  /*in */ NAME_SERVICE_ID_TYPE SERVICE_ID,
  /*in */ NAME_SERVICE_ADDR_TYPE NAME,
  /*in */ NAME_SERVICE_LENGTH_TYPE *NAME_LENGTH,
  /*in */ SYSTEM_TIME_TYPE TIME_OUT,
  /*out */ SAP_ADDRESS_TYPE *SAP_ADDRESS,
  /*out*/ RETURN_CODE_TYPE *RETURN_CODE );
#endif
/*--------------------------------------------------------------------*/
/*                                                                    */
/* Memory Block constant and type definitions and management services */
/*                                                                    */
/*--------------------------------------------------------------------*/
#ifndef APEX_MEMORY_BLOCK
#define APEX_MEMORY_BLOCK
typedef NAME_TYPE MEMORY_BLOCK_NAME_TYPE;

/* FOLLOWING ENUMERATION IS A DUPLICATE from the FILE SYSTEM header...using typedef instead
typedef enum {READ = 0, READ_WRITE=1} MEMORY_BLOCK_MODE_TYPE; */
#include "APEX_FILE_SYSTEM.h"
typedef FILE_MODE_TYPE MEMORY_BLOCK_MODE_TYPE;


typedef APEX_INTEGER MEMORY_BLOCK_SIZE_TYPE;
typedef APEX_BYTE * MEMORY_BLOCK_ADDR_TYPE;
typedef struct MEMORY_BLOCK_STATUS_TYPE
{
MEMORY_BLOCK_ADDR_TYPE MEMORY_BLOCK_ADDR;
MEMORY_BLOCK_MODE_TYPE MEMORY_BLOCK_MODE;
MEMORY_BLOCK_SIZE_TYPE MEMORY_BLOCK_SIZE; /* Number of bytes */
} MEMORY_BLOCK_STATUS_TYPE;
extern void GET_MEMORY_BLOCK_STATUS (
/* in */ MEMORY_BLOCK_NAME_TYPE MEMORY_BLOCK_NAME,
/* out */ MEMORY_BLOCK_STATUS_TYPE *MEMORY_BLOCK_STATUS,
/* out */ RETURN_CODE_TYPE *RETURN_CODE);
#endif
/*==================================================================*/
/*--------------------------------------*/
/*                                      */
/* Health Monitoring Extension services */
/*                                      */
/*--------------------------------------*/
#ifndef APEX_HEALTH_MONITORING_EXTENSIONS
#define APEX_HEALTH_MONITORING_EXTENSIONS
extern void ENABLE_ERROR_HANDLER (
  /*out*/ RETURN_CODE_TYPE *RETURN_CODE);

#endif

/*-----------------------------------------------------------------------*/
/*                                                                       */
/* Queuing Port List constants, type definitions and management services */
/*                                                                       */
/*-----------------------------------------------------------------------*/
#ifndef APEX_QUEUING_PORT_LIST
#define APEX_QUEUING_PORT_LIST
/* Types */
typedef enum {INACTIVE, ACTIVE} PORT_ACTION_TYPE;
typedef enum {RESET, CONTINUE} RESET_LIST_TYPE;
typedef APEX_INTEGER LIST_SIZE_TYPE;
typedef APEX_INTEGER QUEUING_PORT_LIST_ID_TYPE;
typedef NAME_TYPE QUEUING_PORT_LIST_NAME_TYPE;
/* Services */
extern void CREATE_QUEUING_PORT_LIST (
  /*in */ QUEUING_PORT_LIST_NAME_TYPE QUEUING_PORT_LIST_NAME,
  /*in */ LIST_SIZE_TYPE MAX_LIST_SIZE,
  /*out*/ QUEUING_PORT_LIST_ID_TYPE *QUEUING_PORT_LIST_ID,
  /*out*/ RETURN_CODE_TYPE *RETURN_CODE);
extern void ADD_PORT_TO_QUEUING_PORT_LIST (
  /*in */ QUEUING_PORT_LIST_ID_TYPE QUEUING_PORT_LIST_ID,
  /*in */ QUEUING_PORT_ID_TYPE QUEUING_PORT_ID,
  /*in */ PORT_ACTION_TYPE PORT_ACTION,
  /*out*/ RETURN_CODE_TYPE *RETURN_CODE);
extern void SET_PORT_ACTION_IN_QUEUING_PORT_LIST (
  /*in */ QUEUING_PORT_LIST_ID_TYPE QUEUING_PORT_LIST_ID,
  /*in */ QUEUING_PORT_ID_TYPE QUEUING_PORT_ID,
  /*in */ PORT_ACTION_TYPE PORT_ACTION,
  /*out*/ RETURN_CODE_TYPE *RETURN_CODE);
extern void GET_QUEUING_PORT_ACTION_STATUS (
  /*in */ QUEUING_PORT_LIST_ID_TYPE QUEUING_PORT_LIST_ID,
  /*in */ QUEUING_PORT_ID_TYPE QUEUING_PORT_ID,
  /*out*/ PORT_ACTION_TYPE *PORT_ACTION_STATUS,
  /*out*/ RETURN_CODE_TYPE *RETURN_CODE);
extern void RECEIVE_MESSAGE_FROM_QUEUING_PORT_LIST (
  /*in */ QUEUING_PORT_LIST_ID_TYPE QUEUING_PORT_LIST_ID,
  /*in */ SYSTEM_TIME_TYPE TIME_OUT,
  /*in */ MESSAGE_ADDR_TYPE MESSAGE_ADDR,
  /*in */ MESSAGE_SIZE_TYPE BUFFER_SIZE,
  /*in */ RESET_LIST_TYPE RESET_LIST,
  /*out*/ QUEUING_PORT_ID_TYPE *QUEUING_PORT_ID,
  /*out*/ MESSAGE_SIZE_TYPE *LENGTH,
  /*out*/ RETURN_CODE_TYPE *RETURN_CODE);
extern void WAIT_FOR_MESSAGE_FROM_QUEUING_PORT_LIST (
  /*in */ QUEUING_PORT_LIST_ID_TYPE QUEUING_PORT_LIST_ID,
  /*in */ SYSTEM_TIME_TYPE TIME_OUT,
  /*in */ RESET_LIST_TYPE RESET_LIST,
  /*out*/ QUEUING_PORT_ID_TYPE *QUEUING_PORT_ID,
  /*out*/ RETURN_CODE_TYPE *RETURN_CODE);

#endif

#ifdef __cplusplus
}
#endif /* __cplusplus */


