/*------------------------------------------------------------------*/
/*                                                                  */
/* SAP Ports constant and type definitions and management services  */
/*                                                                  */
/*------------------------------------------------------------------*/
#ifndef APEX_SAP_PORTS
#define APEX_SAP_PORTS

#ifdef __cplusplus
extern "C" {
#endif /* __cplusplus */

#include "APEX_TYPES.h"
#include "APEX_PROCESS.h"


#define MAX_NUMBER_OF_SAP_PORTS 1024 /* Implementation dependent */
/* Partition scope */
typedef NAME_TYPE SAP_PORT_NAME_TYPE;
typedef APEX_INTEGER SAP_PORT_ID_TYPE;
//#ifndef APEX_UNSIGNED_SHORT
/* Implementation dependent*/
typedef unsigned short int APEX_UNSIGNED_SHORT;/* 16-bit unsigned integer */
//#endif
typedef struct SAP_ADDRESS_TYPE
{
APEX_BYTE Addr_Length; /* structure size */
APEX_BYTE Addr_Type; /* address type */
APEX_UNSIGNED_SHORT UDP_Port_Number; /* src/dst UDP port number */
APEX_UNSIGNED IP_Address; /* src/dst IP address */
APEX_BYTE Addr_Spare[8]; /* structure padding */
} SAP_ADDRESS_TYPE;
#define SAP_ADDR_TYPE_UNKNOWN 0 /* unknown address type */
#define SAP_ADDR_TYPE_INET 1 /* type is IP address type */
/* structure returned by SAP port status query */
typedef struct SAP_PORT_STATUS_TYPE
{
MESSAGE_RANGE_TYPE NB_MESSAGE;
MESSAGE_RANGE_TYPE MAX_NB_MESSAGE;
MESSAGE_SIZE_TYPE MAX_MESSAGE_SIZE;
PORT_DIRECTION_TYPE PORT_DIRECTION;
WAITING_RANGE_TYPE WAITING_PROCESSES;
SAP_ADDRESS_TYPE CURRENT_DEST_ADDR;
SAP_ADDRESS_TYPE CURRENT_SRC_ADDR;
} SAP_PORT_STATUS_TYPE;
extern void CREATE_SAP_PORT (
  /*in */ SAP_PORT_NAME_TYPE SAP_PORT_NAME,
  /*in */ MESSAGE_SIZE_TYPE MAX_MESSAGE_SIZE,
  /*in */ MESSAGE_RANGE_TYPE MAX_NB_MESSAGE,
  /*in */ PORT_DIRECTION_TYPE PORT_DIRECTION,
  /*in */ QUEUING_DISCIPLINE_TYPE QUEUING_DISCIPLINE,
  /*out*/ SAP_PORT_ID_TYPE *SAP_PORT_ID,
  /*out*/ RETURN_CODE_TYPE *RETURN_CODE );
extern void SEND_SAP_MESSAGE (
  /*in */ SAP_PORT_ID_TYPE SAP_PORT_ID,
  /*in */ MESSAGE_ADDR_TYPE MESSAGE_ADDR,
  /*in */ MESSAGE_SIZE_TYPE LENGTH,
  /*in */ SYSTEM_TIME_TYPE TIME_OUT,
  /*in */ SAP_ADDRESS_TYPE DESTINATION_ADDR,
  /*out*/ RETURN_CODE_TYPE *RETURN_CODE );
extern void RECEIVE_SAP_MESSAGE (
  /*in */ SAP_PORT_ID_TYPE SAP_PORT_ID,
  /*in */ SYSTEM_TIME_TYPE TIME_OUT,
  /*out*/ MESSAGE_ADDR_TYPE MESSAGE_ADDR,
  /*out*/ MESSAGE_SIZE_TYPE *LENGTH,
  /*out*/ SAP_ADDRESS_TYPE *SOURCE_ADDR,
  /*out*/ RETURN_CODE_TYPE *RETURN_CODE );
extern void GET_SAP_PORT_ID (
  /*in */ SAP_PORT_NAME_TYPE SAP_PORT_NAME,
  /*out*/ SAP_PORT_ID_TYPE *SAP_PORT_ID,
  /*out*/ RETURN_CODE_TYPE *RETURN_CODE);
extern void GET_SAP_PORT_STATUS (
  /*in */ SAP_PORT_ID_TYPE SAP_PORT_ID,
  /*in */ SAP_PORT_STATUS_TYPE *SAP_PORT_STATUS,
  /*in */ RETURN_CODE_TYPE *RETURN_CODE );
extern void SEND_EXTENDED_SAP_MESSAGE(
  /*in */ SAP_PORT_ID_TYPE SAP_PORT_ID,
  /*in */ MESSAGE_ADDR_TYPE MESSAGE_ADDR,
  /*in */ MESSAGE_SIZE_TYPE LENGTH,
  /*in */ SYSTEM_TIME_TYPE TIME_OUT,
  /*in */ SAP_ADDRESS_TYPE SOURCE_ADDR,
  /*in */ SAP_ADDRESS_TYPE DESTINATION_ADDR,
  /*out*/ RETURN_CODE_TYPE *RETURN_CODE );
extern void RECEIVE_EXTENDED_SAP_MESSAGE(
  /*in */ SAP_PORT_ID_TYPE SAP_PORT_ID,
  /*in */ SYSTEM_TIME_TYPE TIME_OUT,
  /*out*/ MESSAGE_ADDR_TYPE MESSAGE_ADDR,
  /*out*/ MESSAGE_SIZE_TYPE *LENGTH,
  /*out*/ SAP_ADDRESS_TYPE *SOURCE_ADDR,
  /*out*/ SAP_ADDRESS_TYPE *DESTINATION_ADDR,
  /*out*/ RETURN_CODE_TYPE *RETURN_CODE );

#ifdef __cplusplus
}
#endif /* __cplusplus */

#endif
