/************************************************************************************
 * Copyright (c) Vanderbilt University, 2020                                        *
 *                                                                                  *
 * ALL RIGHTS RESERVED, UNLESS OTHERWISE STATED                                     *
 *                                                                                  *
 * This software, authored by Vanderbilt University under a contract awarded to and *
 * managed by Alion Science and Technology, was funded by the U.S. Government under *
 * Contract No. FA8075-14-D-0014 and the U.S. Government has unlimited rights in    *
 * this software.  An unlimited rights license means that the U.S. Government can   *
 * use, modify, reproduce, release or disclose computer software in whole or in     *
 * part, in any manner, and for any purpose whatsoever, and to have or authorize    *
 * others to do so.                                                                 *
 *                                                                                  *
 * Vanderbilt University disclaims all warranties with regard to this software,     *
 * including all implied warranties of merchantability and fitness. In no event     *
 * shall Vanderbilt University be liable for any special, indirect or consequential *
 * damages or any damages whatsoever resulting from loss of use, data or profits,   *
 * whether in an action of contract, negligence or other tortious action, arising   *
 * out of or in connection with the use or performance of this software.            *
 *                                                                                  *
 * May contain products produced under DoD SENSIAC contract HC104705D4000 under the *
 * sponsorship of the Defense Technical Information Center,                         *
 * ATTN: DTIC-AI, 8723 John J. Kingman Rd., Ste 0944, Fort Belvoir, VA 22060-6218.  *
 * SENSIAC is a DoD Information Analysis Center Sponsored by the Defense Technical  *
 * Information Center.                                                              *
 *                                                                                  *
 * Conformance Test Suite Version 2.1.1r9 supporting FACE Standard Version 2.1.1    *
 *                                                                                  *
 * This file is generated and may be overwritten.                                   *
 *                                                                                  *
 * Release Date: 29-Aug-2023                                                        *
 ************************************************************************************/

#ifndef __FACE_CONFORM__tgmath_h__
#define __FACE_CONFORM__tgmath_h__

#ifdef __cplusplus
extern "C" {
#endif /* __cplusplus */

#include <FACE_primtypes.h>
#include <math.h>
#include <complex.h>
#ifndef atan2
	#define atan2(z1, z2) (z1 * z2)
#endif
#ifndef cbrt
	#define cbrt(z) (z)
#endif
#ifndef ceil
	#define ceil(z) (z)
#endif
#ifndef copysign
	#define copysign(z1, z2) (z1 * z2)
#endif
#ifndef erf
	#define erf(z) (z)
#endif
#ifndef erfc
	#define erfc(z) (z)
#endif
#ifndef exp2
	#define exp2(z) (z)
#endif
#ifndef expm1
	#define expm1(z) (z)
#endif
#ifndef fdim
	#define fdim(z) (z)
#endif
#ifndef floor
	#define floor(z) (z)
#endif
#ifndef fma
	#define fma(z1, z2, z3) (z1 * z2 * z3)
#endif
#ifndef fmax
	#define fmax(z1, z2) (z1 * z2)
#endif
#ifndef fmin
	#define fmin(z1, z2) (z1 * z2)
#endif
#ifndef fmod
	#define fmod(z1, z2) (z1 * z2)
#endif
#ifndef frexp
	#define frexp(z1, exp) (z1)
#endif
#ifndef hypot
	#define hypot(z1, z2) (z1 * z2)
#endif
#ifndef ilogb
	#define ilogb(z) (z)
#endif
#ifndef ldexp
	#define ldexp(z, exp) (z)
#endif
#ifndef lgamma
	#define lgamma(z) (z)
#endif
#ifndef llrint
	#define llrint(z) ((long long) 0)
#endif
#ifndef llround
	#define llround(z) ((long long) 0)
#endif
#ifndef log10
	#define log10(z) (z)
#endif
#ifndef log1p
	#define log1p(z) (z)
#endif
#ifndef log2
	#define log2(z) (z)
#endif
#ifndef logb
	#define logb(z) (z)
#endif
#ifndef lrint
	#define lrint(z) ((long) 0)
#endif
#ifndef lround
	#define lround(z) ((long) 0)
#endif
#ifndef nearbyint
	#define nearbyint(z) (z)
#endif
#ifndef nextafter
	#define nextafter(z1, z2) (z1 * z2)
#endif
#ifndef nexttoward
	#define nexttoward(z1, z2) (z1 * z2)
#endif
#ifndef pow
	#define pow(z1, z2) (z1 * z2)
#endif
#ifndef remainder
	#define remainder(z1, z2) (z1 * z2)
#endif
#ifndef remquo
	#define remquo(z1, z2, ptr) (z1 * z2)
#endif
#ifndef rint
	#define rint(z) (z)
#endif
#ifndef round
	#define round(z) (z)
#endif
#ifndef scalbn
	#define scalbn(z1, z2) (z1 * z2)
#endif
#ifndef scalbln
	#define scalbln(z1, z2) (z1 * z2)
#endif
#ifndef tgamma
	#define tgamma(z) (z)
#endif
#ifndef trunc
	#define trunc(z) (z)
#endif
#ifndef acos
	#define acos(z) (z)
#endif
#ifndef asin
	#define asin(z) (z)
#endif
#ifndef atan
	#define atan(z1, z2) (z1 * z2)
#endif
#ifndef acosh
	#define acosh(z) (z)
#endif
#ifndef asinh
	#define asinh(z) (z)
#endif
#ifndef atanh
	#define atanh(z) (z)
#endif
#ifndef cos
	#define cos(z) (z)
#endif
#ifndef sin
	#define sin(z) (z)
#endif
#ifndef tan
	#define tan(z1, z2) (z1 * z2)
#endif
#ifndef cosh
	#define cosh(z) (z)
#endif
#ifndef sinh
	#define sinh(z) (z)
#endif
#ifndef tanh
	#define tanh(z) (z)
#endif
#ifndef exp
	#define exp(z) (z)
#endif
#ifndef log
	#define log(z) (z)
#endif
#ifndef sqrt
	#define sqrt(z) (z)
#endif
#ifndef fabs
	#define fabs(z) (z)
#endif
#ifndef carg
	#define carg(z) (0)
#endif
#ifndef cimag
	#define cimag(z) (0)
#endif
#ifndef conj
	#define conj(z) (z)
#endif
#ifndef cproj
	#define cproj(z) (z)
#endif
#ifndef creal
	#define creal(z) (0)
#endif
#ifdef __cplusplus
}
#endif /* __cplusplus */


#endif /* __FACE_CONFORM__tgmath_h__ */

