// Product produced under Contract No. FA8075-14-D-0014 and the U.S. Government
// has unlimited rights in this software.
//
// Vanderbilt University disclaims all warranties with regard to this
// software, including all implied warranties of merchantability and fitness.
// In no event shall Vanderbilt University be liable for any special,
// indirect or consequential damages or any damages whatsoever resulting
// from loss of use, data or profits, whether in an action of contract,
// negligence or other tortious action, arising out of or in connection
// with the use or performance of this software.
//
// May contain product produced under DoD SENSIAC contract HC104705D4000 under
// the sponsorship of the Defense Technical Information Center,
// ATTN: DTIC-AI, 8723 John J. Kingman Rd., Ste 0944, Fort Belvoir, VA
// 22060-6218.  SENSIAC is a DoD Information Analysis Center Sponsored
// by the Defense Technical Information Center.

package edu.vu.isis.FACE.util;


import java.io.DataInputStream;
import java.io.DataOutputStream;
import java.io.IOException;
import java.net.ServerSocket;
import java.net.Socket;
import java.net.SocketException;
import java.net.UnknownHostException;
import java.util.HashMap;


public class TCPSocket {
	
	protected class ListenThread extends Thread
	{
		public void run()
		{
			while (true)
			{
				try 
				{
					if (isServer && connected == false)
					{
						if (serverCommSock == null)
						{
							serverCommSock = serverSock.accept();
							connected = true;
						}
					}
					else if (connected == false) // client
					{
						clientSock = new Socket(server_host, server_port);
						clientInputStream = new DataInputStream(clientSock.getInputStream());
						connected = true;
					}
				}
				catch (IOException e)
				{
					connected = false;
				}
				
				try {
					Thread.sleep(500);
				}
				catch (InterruptedException e)
				{
					break;
				}
			}
		}
	}
	
	public ServerSocket serverSock = null;
	public Socket serverCommSock;
	public Socket clientSock = null;
	public boolean isServer = false;
	public boolean connected = false;
	private ListenThread listenThread;
	private int server_port;
	private String server_host;
	private DataInputStream clientInputStream;
	
	public void createServerSocket(int serverPort) throws IOException {
		serverSock = new ServerSocket(serverPort);
		listenThread = new ListenThread();
		connected = false;
		isServer = true;
		listenThread.start();
	}
	
	public void receive(byte buffer[]) throws IOException {
		if (connected)
		{
			clientInputStream.read(buffer);
		}
       	else
       	{

       		throw new SocketException("Socket not connected.");
       	}
	}

    public void createClientSocket(String server_host, int server_port) throws UnknownHostException, IOException {
		listenThread = new ListenThread();
		this.server_host = server_host;
		this.server_port = server_port;
		connected = false;
		isServer = false;
		listenThread.start();
	}
    
   
    public void send(byte data[]) throws IOException, SocketException {
      	if (connected)
   		{
       		if (serverCommSock != null)
          	{
           		DataOutputStream out_put = new DataOutputStream(serverCommSock.getOutputStream());
           		out_put.write(data);
          	}
   		}
       	else
       	{
       		throw new SocketException("Socket not connected.");
       	}
	}
   
    public void close() throws Exception {
    	if((serverSock!=null) && (!serverSock.isClosed())) {
    		serverSock.close();
    		connected = false;
    	}
    	if((serverCommSock!=null) && (!serverCommSock.isClosed())) {
    		serverCommSock.close();
    		connected = false;
    	}         
    	if((clientSock!=null) && (!clientSock.isClosed())) {
    		clientSock.close();
    		connected = false;
    	}
    }
}
