// Product produced under Contract No. FA8075-14-D-0014 and the U.S. Government
// has unlimited rights in this software.
//
// Vanderbilt University disclaims all warranties with regard to this
// software, including all implied warranties of merchantability and fitness.
// In no event shall Vanderbilt University be liable for any special,
// indirect or consequential damages or any damages whatsoever resulting
// from loss of use, data or profits, whether in an action of contract,
// negligence or other tortious action, arising out of or in connection
// with the use or performance of this software.
//
// May contain product produced under DoD SENSIAC contract HC104705D4000 under
// the sponsorship of the Defense Technical Information Center,
// ATTN: DTIC-AI, 8723 John J. Kingman Rd., Ste 0944, Fort Belvoir, VA
// 22060-6218.  SENSIAC is a DoD Information Analysis Center Sponsored
// by the Defense Technical Information Center.

package edu.vu.isis.FACE.util;

import java.io.IOException;
import java.net.DatagramPacket;
import java.net.DatagramSocket;
import java.net.MulticastSocket;
import java.net.InetAddress;
import java.net.SocketException;
import java.net.UnknownHostException;
import java.net.SocketTimeoutException;
import java.net.PortUnreachableException;
import java.nio.channels.IllegalBlockingModeException;
import java.util.concurrent.atomic.AtomicLong;
import java.util.HashMap;
import java.util.Iterator;

public class UDPSocket {
    
    DatagramSocket client=null;
    DatagramSocket server=null;
        
    public void createServerSocket() throws SocketException, UnknownHostException {
        server = new DatagramSocket();
    }

    public void createClientSocket(String client_host, int client_port) throws SocketException, UnknownHostException, IOException {
        InetAddress address = InetAddress.getByName(client_host);

        if (!address.isMulticastAddress())
            client = new DatagramSocket(client_port, address);
        else // multicast
        {
            try {
                MulticastSocket client_multi = new MulticastSocket(client_port);
                client = new MulticastSocket(client_port);
                ((MulticastSocket)client).joinGroup(address);
            }
            catch (IOException e) {
                throw e;
            }
        }
    }

    public void send(byte data[],String client_host, int client_port) throws SocketException, UnknownHostException, IOException {
        DatagramPacket data_sent=null;
        data_sent = new DatagramPacket(data,data.length,InetAddress.getByName(client_host),client_port);
        server.send(data_sent);
    }
    
    public void receive(byte buffer[],long timeout) throws SocketTimeoutException, PortUnreachableException, IllegalBlockingModeException, IOException {
        DatagramPacket data_received = new DatagramPacket(buffer,buffer.length);
        client.setSoTimeout((int)timeout);
        client.receive(data_received);
    }
    
    public void close() {
        if((server!=null) && (!server.isClosed())) {
              server.close();
        }
        if((client!=null) && (!client.isClosed())) {
              client.close();
        }
     }
}
