/*
 * DISTRIBUTION STATEMENT A. Approved for public release; distribution is unlimited.
 *
 * Product produced under DoD SENSIAC contract HC104705D4000 under the sponsorship of the Defense
 * Technical Information Center, ATTN: DTIC-AI, 8723 John J. Kingman Rd., Ste 0944, Fort Belvoir, VA
 * 22060-6218.  SENSIAC is a DoD Information Analysis Center Sponsored by the Defense Technical
 * Information Center.
 * 
 * HANDLING AND DESTRUCTION NOTICE - Comply with distribution statement and destroy by any method that
 * will prevent disclosure of the contents or reconstruction of the document.
 */

#include "edu_vu_isis_FACE_posix_PosixMQ.h"
#include "JNI_Errors.h"

#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <errno.h>
#include <sys/types.h>
#include <fcntl.h>
#include <sys/stat.h>
#include <mqueue.h>
#include <map>


std::map<int, int> errno_map;


extern "C"
{

JNIEXPORT void JNICALL Java_edu_vu_isis_FACE_posix_PosixMQ_init_1native
  (JNIEnv *env, jobject thisObj, jlong instanceID)
{
	errno_map[instanceID];
}

JNIEXPORT void JNICALL Java_edu_vu_isis_FACE_posix_PosixMQ_finalize_1native
  (JNIEnv *env, jobject thisObj, jlong instanceID)
{
	errno_map.erase(instanceID);
}

/*******************************
*  mq_open
********************************/
// mq_open flag symbols
JNIEXPORT jint JNICALL Java_edu_vu_isis_FACE_posix_PosixMQ_getO_1RDONLY(JNIEnv *env, jclass thisClass)  { return O_RDONLY;  }
JNIEXPORT jint JNICALL Java_edu_vu_isis_FACE_posix_PosixMQ_get0_1WRONLY(JNIEnv *env, jclass thisClass)  { return O_WRONLY;  }
JNIEXPORT jint JNICALL Java_edu_vu_isis_FACE_posix_PosixMQ_getO_1RDWR(JNIEnv *env, jclass thisClass)    { return O_RDWR;    }
JNIEXPORT jint JNICALL Java_edu_vu_isis_FACE_posix_PosixMQ_getO_1CREAT(JNIEnv *env, jclass thisClass)   { return O_CREAT;   }
JNIEXPORT jint JNICALL Java_edu_vu_isis_FACE_posix_PosixMQ_getO_1EXCL(JNIEnv *env, jclass thisClass)    { return O_EXCL;    }
JNIEXPORT jint JNICALL Java_edu_vu_isis_FACE_posix_PosixMQ_getO_1NONBLOCK(JNIEnv *env, jclass thisClass){ return O_NONBLOCK;}

JNIEXPORT jlong JNICALL Java_edu_vu_isis_FACE_posix_PosixMQ_mq_1open_1native__JLjava_lang_String_2I
  (JNIEnv *env, jobject thisObj, jlong instanceID, jstring name_jstring, jint oflag)
{
	// Convert name to c-string
	const char* name = env->GetStringUTFChars(name_jstring, NULL);
	if (name == NULL)
		return JNI_JSTRING_CSTRING_ERR;

	// Create message queue
 	mqd_t desc;
 	desc = mq_open(name, oflag);
 	errno_map[instanceID] = errno; // capture any error for later retrieval

 	env->ReleaseStringUTFChars(name_jstring, name);

 	return (intptr_t)desc;
}

JNIEXPORT jlong JNICALL Java_edu_vu_isis_FACE_posix_PosixMQ_mq_1open_1native__JLjava_lang_String_2IIII
  (JNIEnv *env, jobject thisObj, jlong instanceID, jstring name_jstring, jint oflag, jint mode, jint length, jint msgsize)
{
	// Convert name to c-string
	const char* name = env->GetStringUTFChars(name_jstring, NULL);
	if (name == NULL)
		return JNI_JSTRING_CSTRING_ERR;

	// Create message queue
    struct mq_attr attr;
    attr.mq_maxmsg = length;
    attr.mq_msgsize = msgsize;
    // mq_flags and mq_curmsgs not used by mq_open()

 	mqd_t desc = mq_open(name, oflag, mode, &attr);
 	errno_map[instanceID] = errno; // capture any error for later retrieval

 	env->ReleaseStringUTFChars(name_jstring, name);

 	return (intptr_t)desc;
}


/*******************************
*  mq_send
********************************/
JNIEXPORT jlong JNICALL Java_edu_vu_isis_FACE_posix_PosixMQ_mq_1send_1native
  (JNIEnv *env, jobject thisObj, jlong instanceID, jlong descriptor, jbyteArray bytes_jarray)
{
	if (bytes_jarray == NULL)
		return JNI_NULL_ARG;

	// get C-pointer to and length of java byte array
	jbyte* bytes_carray = env->GetByteArrayElements(bytes_jarray, 0);
	if (bytes_carray == NULL)
		return JNI_JARRAY_CARRAY_ERR;
	jsize bytes_length = env->GetArrayLength(bytes_jarray);

	// size of jbyte is platform-dependent, so msg_len needs to be calculated
	int retVal = mq_send((mqd_t)descriptor, (char*)bytes_carray, bytes_length * sizeof(jbyte), 0);
	errno_map[instanceID] = errno;

	env->ReleaseByteArrayElements(bytes_jarray, bytes_carray, 0);

	return retVal;
}


/*******************************
*  mq_receive
********************************/
JNIEXPORT jlong JNICALL Java_edu_vu_isis_FACE_posix_PosixMQ_mq_1receive_1native
  (JNIEnv *env, jobject thisObj, jlong instanceID, jlong descriptor, jbyteArray bytes_jarray)
{
	if (bytes_jarray == NULL)
		return JNI_NULL_ARG;

	jbyte* bytes_carray = env->GetByteArrayElements(bytes_jarray, 0);
	if (bytes_carray == NULL)
		return JNI_JARRAY_CARRAY_ERR;
	jsize bytes_length = env->GetArrayLength(bytes_jarray);

	// size of jbyte is platform-dependent, so msg_len needs to be calculated
	size_t retVal = mq_receive((mqd_t)descriptor, (char*)bytes_carray, bytes_length * sizeof(jbyte), NULL);
	errno_map[instanceID] = errno;
	if (retVal != -1)
		env->SetByteArrayRegion(bytes_jarray, 0, bytes_length, bytes_carray);

	env->ReleaseByteArrayElements(bytes_jarray, bytes_carray, 0);
	return retVal;
}


/*******************************
*  mq_close
********************************/
JNIEXPORT jlong JNICALL Java_edu_vu_isis_FACE_posix_PosixMQ_mq_1close_1native
  (JNIEnv *env, jobject thisObj, jlong instanceID, jlong descriptor)
{
	int retVal = mq_close((mqd_t)descriptor);
 	errno_map[instanceID] = errno; // capture any error for later retrieval

 	return retVal;
}


/*************************
* Custom JNI Error symbols
**************************/
JNIEXPORT jint JNICALL Java_edu_vu_isis_FACE_posix_PosixMQ_getJNI_1JSTRING_1CSTRING_1ERR(JNIEnv*, jclass) { return JNI_JSTRING_CSTRING_ERR; }
JNIEXPORT jint JNICALL Java_edu_vu_isis_FACE_posix_PosixMQ_getJNI_1JARRAY_1CARRAY_1ERR(JNIEnv*, jclass)   { return JNI_JARRAY_CARRAY_ERR;   }
JNIEXPORT jint JNICALL Java_edu_vu_isis_FACE_posix_PosixMQ_getJNI_1NULL_1ARG(JNIEnv*, jclass)             { return JNI_NULL_ARG;            }


/**********************
* POSIX Error symbols
***********************/
JNIEXPORT jint JNICALL Java_edu_vu_isis_FACE_posix_PosixMQ_getEACCES(JNIEnv*, jclass)      { return EACCES;  }
JNIEXPORT jint JNICALL Java_edu_vu_isis_FACE_posix_PosixMQ_getEAGAIN(JNIEnv*, jclass)      { return EAGAIN;  }
JNIEXPORT jint JNICALL Java_edu_vu_isis_FACE_posix_PosixMQ_getEBADF(JNIEnv*, jclass)       { return EBADF;   }
JNIEXPORT jint JNICALL Java_edu_vu_isis_FACE_posix_PosixMQ_getEBADMSG(JNIEnv*, jclass)     { return EBADMSG; }
JNIEXPORT jint JNICALL Java_edu_vu_isis_FACE_posix_PosixMQ_getEEXIST(JNIEnv*, jclass)      { return EEXIST;  }
JNIEXPORT jint JNICALL Java_edu_vu_isis_FACE_posix_PosixMQ_getEINTR(JNIEnv*, jclass)       { return EINTR;   }
JNIEXPORT jint JNICALL Java_edu_vu_isis_FACE_posix_PosixMQ_getEINVAL(JNIEnv*, jclass)      { return EINVAL;  }
JNIEXPORT jint JNICALL Java_edu_vu_isis_FACE_posix_PosixMQ_getEMFILE(JNIEnv*, jclass)      { return EMFILE;  }
JNIEXPORT jint JNICALL Java_edu_vu_isis_FACE_posix_PosixMQ_getEMSGSIZE(JNIEnv*, jclass)    { return EMSGSIZE;}
JNIEXPORT jint JNICALL Java_edu_vu_isis_FACE_posix_PosixMQ_getENAMETOOLONG(JNIEnv*, jclass){ return ENAMETOOLONG; }
JNIEXPORT jint JNICALL Java_edu_vu_isis_FACE_posix_PosixMQ_getENFILE(JNIEnv*, jclass)      { return ENFILE;  }
JNIEXPORT jint JNICALL Java_edu_vu_isis_FACE_posix_PosixMQ_getENOENT(JNIEnv*, jclass)      { return ENOENT;  }
JNIEXPORT jint JNICALL Java_edu_vu_isis_FACE_posix_PosixMQ_getENOSPC(JNIEnv*, jclass)      { return ENOSPC;  }
//JNIEXPORT jint JNICALL Java_edu_vu_isis_FACE_posix_PosixMQ_getETIMEDOUT(JNIEnv*, jclass)   { return ETIMEDOUT; }


JNIEXPORT jlong JNICALL Java_edu_vu_isis_FACE_posix_PosixMQ_getErrno_1native
  (JNIEnv *env, jobject thisObj, jlong instanceID)
{
	return errno_map[instanceID];
}

JNIEXPORT jstring JNICALL Java_edu_vu_isis_FACE_posix_PosixMQ_getStrerror_1native
  (JNIEnv *env, jobject thisObj, jlong instanceID)
{
	return env->NewStringUTF(strerror(errno_map[instanceID]));
}

} // extern "C"
