/*
 * DISTRIBUTION STATEMENT A. Approved for public release; distribution is unlimited.
 *
 * Product produced under DoD SENSIAC contract HC104705D4000 under the sponsorship of the Defense
 * Technical Information Center, ATTN: DTIC-AI, 8723 John J. Kingman Rd., Ste 0944, Fort Belvoir, VA
 * 22060-6218.  SENSIAC is a DoD Information Analysis Center Sponsored by the Defense Technical
 * Information Center.
 * 
 * HANDLING AND DESTRUCTION NOTICE - Comply with distribution statement and destroy by any method that
 * will prevent disclosure of the contents or reconstruction of the document.
 */

#include "edu_vu_isis_FACE_posix_PosixSHM.h"
#include "JNI_Errors.h"

#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <errno.h>
#include <sys/types.h>
#include <fcntl.h>
#include <sys/mman.h>
#include <map>


std::map<int, int> errno_map;


extern "C"
{

JNIEXPORT void JNICALL Java_edu_vu_isis_FACE_posix_PosixSHM_init_1native
  (JNIEnv *env, jobject thisObj, jlong instanceID)
{
	errno_map[instanceID];
}

JNIEXPORT void JNICALL Java_edu_vu_isis_FACE_posix_PosixMQ_finalize_1native
  (JNIEnv *env, jobject thisObj, jlong instanceID)
{
	errno_map.erase(instanceID);
}

/*******************************
*  shm_open
********************************/
// shm_open flag symbols
JNIEXPORT jint JNICALL Java_edu_vu_isis_FACE_posix_PosixSHM_getO_1RDONLY(JNIEnv*, jclass) { return O_RDONLY; }
JNIEXPORT jint JNICALL Java_edu_vu_isis_FACE_posix_PosixSHM_getO_1RDWR  (JNIEnv*, jclass) { return O_RDWR;   }
JNIEXPORT jint JNICALL Java_edu_vu_isis_FACE_posix_PosixSHM_getO_1CREAT (JNIEnv*, jclass) { return O_CREAT;  }
JNIEXPORT jint JNICALL Java_edu_vu_isis_FACE_posix_PosixSHM_getO_1EXCL  (JNIEnv*, jclass) { return O_EXCL;   }
JNIEXPORT jint JNICALL Java_edu_vu_isis_FACE_posix_PosixSHM_getO_1TRUNC (JNIEnv*, jclass) { return O_TRUNC;  }

JNIEXPORT jlong JNICALL Java_edu_vu_isis_FACE_posix_PosixSHM_shm_1open_1native
  (JNIEnv *env, jobject thisObj, jlong instanceID, jstring name_jstring, jlong oflag, jlong mode)
{
	// Convert name to c-string
	const char* name = env->GetStringUTFChars(name_jstring, NULL);
	if (name == NULL)
		return JNI_JSTRING_CSTRING_ERR;

	// Create shared memory
 	int shm_fd;
 	shm_fd = shm_open(name, oflag, (mode_t)mode);
 	errno_map[instanceID] = errno; // capture any error for later retrieval

 	env->ReleaseStringUTFChars(name_jstring, name);

 	return (intptr_t)shm_fd;
}


/*******************************
*  ftruncate
********************************/
JNIEXPORT jlong JNICALL Java_edu_vu_isis_FACE_posix_PosixSHM_ftruncate_1native
  (JNIEnv *env, jobject thisObj, jlong instanceID, jlong fildes, jlong length)
{
	int retVal = ftruncate(fildes, (off_t)length);
	errno_map[instanceID] = errno;

	return retVal;
}


/*******************************
*  mmap
********************************/
// mmap return symbols
JNIEXPORT jlong JNICALL Java_edu_vu_isis_FACE_posix_PosixSHM_getMAP_1FAILED(JNIEnv*, jclass) { return (jlong)MAP_FAILED;  }
// mmap prot symbols 
JNIEXPORT jint JNICALL Java_edu_vu_isis_FACE_posix_PosixSHM_getPROT_1READ  (JNIEnv*, jclass) { return PROT_READ;   }
JNIEXPORT jint JNICALL Java_edu_vu_isis_FACE_posix_PosixSHM_getPROT_1WRITE (JNIEnv*, jclass) { return PROT_WRITE;  }
JNIEXPORT jint JNICALL Java_edu_vu_isis_FACE_posix_PosixSHM_getPROT_1EXEC  (JNIEnv*, jclass) { return PROT_EXEC;   }
JNIEXPORT jint JNICALL Java_edu_vu_isis_FACE_posix_PosixSHM_getPROT_1NONE  (JNIEnv*, jclass) { return PROT_NONE;   }
// mmap flag symbols 
JNIEXPORT jint JNICALL Java_edu_vu_isis_FACE_posix_PosixSHM_getMAP_1SHARED (JNIEnv*, jclass) { return MAP_SHARED;  }
JNIEXPORT jint JNICALL Java_edu_vu_isis_FACE_posix_PosixSHM_getMAP_1PRIVATE(JNIEnv*, jclass) { return MAP_PRIVATE; }
JNIEXPORT jint JNICALL Java_edu_vu_isis_FACE_posix_PosixSHM_getMAP_1FIXED  (JNIEnv*, jclass) { return MAP_FIXED;   }

JNIEXPORT jlong JNICALL Java_edu_vu_isis_FACE_posix_PosixSHM_mmap_1native
  (JNIEnv *env, jobject thisObj, jlong instanceID, jlong len, jlong prot, jlong flags, jlong fildes)
{
	void* retAddress = mmap((void*)0, (size_t)len, prot, flags, fildes, (off_t)0);
	errno_map[instanceID] = errno;

	return (jlong)retAddress;
}


/*******************************
*  memcpy
********************************/
JNIEXPORT jlong JNICALL Java_edu_vu_isis_FACE_posix_PosixSHM_memcpy_1native
  (JNIEnv *env, jobject thisObj, jboolean writeToAddress, jlong address, jbyteArray bytes_jarray)
{
	if (bytes_jarray == NULL)
		return JNI_NULL_ARG;

	// get C-pointer to and length of java byte array
	jbyte* bytes_carray = env->GetByteArrayElements(bytes_jarray, 0);
	if (bytes_carray == NULL)
		return JNI_JARRAY_CARRAY_ERR;
	jsize bytes_length = env->GetArrayLength(bytes_jarray);

	// either copy from byte array to address or vice-versa, depending on writeToAddress
	if (writeToAddress == JNI_TRUE)
	{
		memcpy((void*)address, (void*)bytes_carray, (size_t)bytes_length);

		env->ReleaseByteArrayElements(bytes_jarray, bytes_carray, 0);
		return (jlong)0;
	}
	else // JNI_FALSE
	{
		memcpy((void*)bytes_carray, (void*)address, (size_t)bytes_length);
		env->SetByteArrayRegion(bytes_jarray, 0, bytes_length, bytes_carray);

		env->ReleaseByteArrayElements(bytes_jarray, bytes_carray, 0);
		return (jlong)0;
	}
}


/*******************************
*  munmap
********************************/
JNIEXPORT jlong JNICALL Java_edu_vu_isis_FACE_posix_PosixSHM_munmap_1native
  (JNIEnv *env, jobject thisObj, jlong instanceID, jlong addr, jlong len)
{
	int retVal = munmap((void*)addr, (size_t)len);
 	errno_map[instanceID] = errno; // capture any error for later retrieval

 	return retVal;
}


/*******************************
*  close
********************************/
JNIEXPORT jlong JNICALL Java_edu_vu_isis_FACE_posix_PosixSHM_close_1native
  (JNIEnv *env, jobject thisObj, jlong instanceID, jlong fildes)
{
	int retVal = close(fildes);
 	errno_map[instanceID] = errno; // capture any error for later retrieval

 	return retVal;
}


/*************************
* Custom JNI Error symbols
**************************/
JNIEXPORT jint JNICALL Java_edu_vu_isis_FACE_posix_PosixSHM_getJNI_1JSTRING_1CSTRING_1ERR(JNIEnv*, jclass) { return JNI_JSTRING_CSTRING_ERR; }
JNIEXPORT jint JNICALL Java_edu_vu_isis_FACE_posix_PosixSHM_getJNI_1JARRAY_1CARRAY_1ERR(JNIEnv*, jclass)   { return JNI_JARRAY_CARRAY_ERR;   }
JNIEXPORT jint JNICALL Java_edu_vu_isis_FACE_posix_PosixSHM_getJNI_1NULL_1ARG(JNIEnv*, jclass)             { return JNI_NULL_ARG;            }


/**********************
* POSIX Error symbols
***********************/
JNIEXPORT jint JNICALL Java_edu_vu_isis_FACE_posix_PosixSHM_getEACCES      (JNIEnv*, jclass) { return EACCES;       }
JNIEXPORT jint JNICALL Java_edu_vu_isis_FACE_posix_PosixSHM_getEAGAIN      (JNIEnv*, jclass) { return EAGAIN;       }
JNIEXPORT jint JNICALL Java_edu_vu_isis_FACE_posix_PosixSHM_getEBADF       (JNIEnv*, jclass) { return EBADF;        }
JNIEXPORT jint JNICALL Java_edu_vu_isis_FACE_posix_PosixSHM_getEBADMSG     (JNIEnv*, jclass) { return EBADMSG;      }
JNIEXPORT jint JNICALL Java_edu_vu_isis_FACE_posix_PosixSHM_getEEXIST      (JNIEnv*, jclass) { return EEXIST;       }
JNIEXPORT jint JNICALL Java_edu_vu_isis_FACE_posix_PosixSHM_getEFBIG       (JNIEnv*, jclass) { return EFBIG;        }
JNIEXPORT jint JNICALL Java_edu_vu_isis_FACE_posix_PosixSHM_getEINTR       (JNIEnv*, jclass) { return EINTR;        }
JNIEXPORT jint JNICALL Java_edu_vu_isis_FACE_posix_PosixSHM_getEINVAL      (JNIEnv*, jclass) { return EINVAL;       }
JNIEXPORT jint JNICALL Java_edu_vu_isis_FACE_posix_PosixSHM_getEIO         (JNIEnv*, jclass) { return EIO;          }
JNIEXPORT jint JNICALL Java_edu_vu_isis_FACE_posix_PosixSHM_getEMFILE      (JNIEnv*, jclass) { return EMFILE;       }
JNIEXPORT jint JNICALL Java_edu_vu_isis_FACE_posix_PosixSHM_getENAMETOOLONG(JNIEnv*, jclass) { return ENAMETOOLONG; }
JNIEXPORT jint JNICALL Java_edu_vu_isis_FACE_posix_PosixSHM_getENFILE      (JNIEnv*, jclass) { return ENFILE;       }
JNIEXPORT jint JNICALL Java_edu_vu_isis_FACE_posix_PosixSHM_getENODEV      (JNIEnv*, jclass) { return ENODEV;       }
JNIEXPORT jint JNICALL Java_edu_vu_isis_FACE_posix_PosixSHM_getENOENT      (JNIEnv*, jclass) { return ENOENT;       }
JNIEXPORT jint JNICALL Java_edu_vu_isis_FACE_posix_PosixSHM_getENOMEM      (JNIEnv*, jclass) { return ENOMEM;       }
JNIEXPORT jint JNICALL Java_edu_vu_isis_FACE_posix_PosixSHM_getENOSPC      (JNIEnv*, jclass) { return ENOSPC;       }
JNIEXPORT jint JNICALL Java_edu_vu_isis_FACE_posix_PosixSHM_getENOTSUP     (JNIEnv*, jclass) { return ENOTSUP;      }
JNIEXPORT jint JNICALL Java_edu_vu_isis_FACE_posix_PosixSHM_getENXIO       (JNIEnv*, jclass) { return ENXIO;        }
JNIEXPORT jint JNICALL Java_edu_vu_isis_FACE_posix_PosixSHM_getEOVERFLOW   (JNIEnv*, jclass) { return EOVERFLOW;    }


JNIEXPORT jlong JNICALL Java_edu_vu_isis_FACE_posix_PosixSHM_getErrno_1native
  (JNIEnv *env, jobject thisObj, jlong instanceID)
{
	return errno_map[instanceID];
}

JNIEXPORT jstring JNICALL Java_edu_vu_isis_FACE_posix_PosixSHM_getStrerror_1native
  (JNIEnv *env, jobject thisObj, jlong instanceID)
{
	return env->NewStringUTF(strerror(errno_map[instanceID]));
}

}