@echo off

if "%~1" == "-h" GOTO usage
if "%~1" == "--help" GOTO usage

GOTO pythonCheck

:usage
    echo Usage: runConformanceTest.bat [options] [config_file1] [config_file2] ...
    echo:
    echo If config file(s) is(are) supplied, conformance test will run directly (no
    echo GUI). If no config_file is supplied, the test suite GUI will launch for
    echo interactive control of conformance test.
    echo:
    echo Multiple configuration files can be passed to run by test suite, but it
	echo is important to have different log directories in each configuration
	echo file, otherwise the test results would be overwritten by subsequent test
	echo:
	echo Options:
    echo   -h, --help            show this help message and exit
    echo   -d new_project_directory, --projectDir=new_project_directory
    echo                         Run test suite using config file, but changing the
    echo                         project directory to new_project_directory. Changing
    echo                         project directory only affects segment related files,
    echo                         not build files. A configuration file must be
    echo                         specified, and the original project directory must be
    echo                         specified in the configuration file.  Multiple
    echo                         configuration files may be specified.
    echo   -m, --model           Run data model test only.
    echo   -o new_log_directory, --logDir=new_log_directory
    echo                        Run test suite using config file, but changing the log
    echo                         directory to new_log_directory
    echo   -g, --gui             Run test suite with GUI even if a configuration file
    echo                         is supplied
    echo   -v, --version         Echo test suite version and FACE Standard version
    echo                         conformance is checked against
    echo   -l generated_library_directory, --gsl=generated_library_directory
    echo                         Generate Gold Standard Libraries needed by segments
    echo                         specified in the configuration file and store in the
    echo                         generated_library_directory. A configuration file must
    echo                         be specified, but subsequent configuration files will
    echo                         be ignored

	exit /b 0
} 

:pythonCheck
rem Check if Python is installed
echo Checking for Python ...

python -c "import sys ; sys.exit(0)" >nul
if %ERRORLEVEL% NEQ 0 (
  echo ------------------- ERROR -------------------
  echo Conformance Test Suite requires Python 2.5+ installed on the host system
  echo Install Python and add it to your PATH.
  exit /b 1
)

echo Python found

rem Make sure Python is 2.6 or later
echo Checking Python version ...

python -c "import sys ; sys.exit( 0 if sys.version_info > (2, 6) else 1 )"
if %ERRORLEVEL% NEQ 0 (
  echo ------------------- ERROR -------------------
  echo Conformance Test Suite requires Python 2.6+ installed on the host system
  echo Current Python version:
  python -V
  pause
  exit /b 1
)

echo Python version is useable

rem Checking version...don't need a GUI, skip tkinterCheck
if "%~1" == "-v" GOTO conformanceTestLaunch
if "%~1" == "--version" GOTO conformanceTestLaunch

if "%~1" == "" GOTO tkinterCheck
if "%~1" == "-g" GOTO tkinterCheck
if "%~1" == "--gui" GOTO tkinterCheck

rem Must be a configuration file...don't need a GUI, skip tkinterCheck
GOTO conformanceTestLaunch

:tkinterCheck
rem Make sure TKinter is installed
echo Checking for Tkinter ...
python face_conformance_app\check_tkinter.py
if %ERRORLEVEL% NEQ 0 (
  echo ------------------- ERROR -------------------
  echo Conformance Test Suite requires Tkinter installed on the host system
  echo Please installed Python with Tkinter
  pause
  exit /b 1
)
echo Tkinter found

:javaCheck
rem Make sure Java can be called from Python
echo Checking for Java ...
python face_conformance_app\check_java.py
if %ERRORLEVEL% NEQ 0 (
  echo ------------------- ERROR -------------------
  echo Conformance Test Suite requires Java 1.7 or greater is installed on the host system
  pause
  exit /b 1
)
echo Java found

:conformanceTestLaunch
rem Run the tool
echo Starting Conformance Test Suite in the face_conformance_app subdirectory...
pushd face_conformance_app
echo python ConformanceMain.py %*
python ConformanceMain.py %*
set returnValue=%ERRORLEVEL%
popd

exit /b %returnValue%

