-- Product produced under DoD SENSIAC contract HC104705D4000 under
-- the sponsorship of the Defense Technical Information Center,
-- ATTN: DTIC-AI, 8723 John J. Kingman Rd., Ste 0944, Fort Belvoir, VA
-- 22060-6218.  SENSIAC is a DoD Information Analysis Center Sponsored
-- by the Defense Technical Information Center.
--
-- HANDLING AND DESTRUCTION NOTICE - Comply with distribution statement
-- and destroy by any method that will prevent disclosure of
-- contents or reconstruction of the document.

with FACE_Basic_Types; use FACE_Basic_Types;
with FACE.Sequences.Unbounded;

package FACE.DM is

	type RollRadian is new FACE_Basic_Types.Double;

	type Longitude is new FACE_Basic_Types.Double;

	type Latitude is new FACE_Basic_Types.Double;

	type PitchDegree is new FACE_Basic_Types.Double;

	type Altitude is new FACE_Basic_Types.Double;

	type HeadingRadian is new FACE_Basic_Types.Double;

	type Pitch is new FACE_Basic_Types.Float;

	type HeadingDegree is new FACE_Basic_Types.Double;

	type PitchRadian is new FACE_Basic_Types.Double;

	type Roll is new FACE_Basic_Types.Float;

	type RollDegree is new FACE_Basic_Types.Double;

	type Yaw is new FACE_Basic_Types.Float;

	type Validity is (
		Valid,
		Invalid);
	Null_Validity : constant Validity := Validity'First;

	type AttitudeBodyFrame is record
		null;
	end record;
	Null_AttitudeBodyFrame : constant AttitudeBodyFrame := (null record);

	type Position is record
		latitude : FACE.DM.Latitude;
		longitude : FACE.DM.Longitude;
		altitude : FACE.DM.Altitude;
	end record;
	Null_Position : constant Position := (
		latitude => FACE.DM.Latitude(0),
		longitude => FACE.DM.Longitude(0),
		altitude => FACE.DM.Altitude(0));

	type Nav_Data is record
		pos : FACE.DM.Position;
		att : FACE.DM.AttitudeBodyFrame;
	end record;
	Null_Nav_Data : constant Nav_Data := (
		pos => Null_Position,
		att => Null_AttitudeBodyFrame);

	type IMU_Data is record
		att : FACE.DM.AttitudeBodyFrame;
	end record;
	Null_IMU_Data : constant IMU_Data := (
		att => Null_AttitudeBodyFrame);

	type InertialMeasurementUnit is record
		attitude : FACE.DM.AttitudeBodyFrame;
	end record;
	Null_InertialMeasurementUnit : constant InertialMeasurementUnit := (
		attitude => Null_AttitudeBodyFrame);

	package IDL_SEQUENCE_InertialMeasurementUnit_Package is new FACE.Sequences.Unbounded(InertialMeasurementUnit, NUll_InertialMeasurementUnit);
	subtype IDL_SEQUENCE_InertialMeasurementUnit is IDL_SEQUENCE_InertialMeasurementUnit_Package.Sequence;
	Null_IDL_SEQUENCE_InertialMeasurementUnit : constant IDL_SEQUENCE_InertialMeasurementUnit := IDL_SEQUENCE_InertialMeasurementUnit_Package.Null_Sequence;

	type AttitudeRadian is record
		roll : FACE.DM.RollRadian;
		pitch : FACE.DM.PitchRadian;
		yaw : FACE.DM.HeadingRadian;
	end record;
	Null_AttitudeRadian : constant AttitudeRadian := (
		roll => FACE.DM.RollRadian(0),
		pitch => FACE.DM.PitchRadian(0),
		yaw => FACE.DM.HeadingRadian(0));

	type EGI1 is record
		attitude : FACE.DM.AttitudeBodyFrame;
		position : FACE.DM.Position;
	end record;
	Null_EGI1 : constant EGI1 := (
		attitude => Null_AttitudeBodyFrame,
		position => Null_Position);

	package IDL_SEQUENCE_EGI1_Package is new FACE.Sequences.Unbounded(EGI1, NUll_EGI1);
	subtype IDL_SEQUENCE_EGI1 is IDL_SEQUENCE_EGI1_Package.Sequence;
	Null_IDL_SEQUENCE_EGI1 : constant IDL_SEQUENCE_EGI1 := IDL_SEQUENCE_EGI1_Package.Null_Sequence;

	type EGI_Data is record
		pos : FACE.DM.Position;
	end record;
	Null_EGI_Data : constant EGI_Data := (
		pos => Null_Position);

	type NavManagementFunction is record
		attitude : FACE.DM.AttitudeBodyFrame;
		position : FACE.DM.Position;
		imuNavSource : IDL_SEQUENCE_InertialMeasurementUnit;
		egiNavSource : IDL_SEQUENCE_EGI1;
	end record;
	Null_NavManagementFunction : constant NavManagementFunction := (
		attitude => Null_AttitudeBodyFrame,
		position => Null_Position,
		imuNavSource => Null_IDL_SEQUENCE_InertialMeasurementUnit,
		egiNavSource => Null_IDL_SEQUENCE_EGI1);

	type AttitudeDegree is record
		roll : FACE.DM.RollDegree;
		pitch : FACE.DM.PitchDegree;
		yaw : FACE.DM.HeadingDegree;
	end record;
	Null_AttitudeDegree : constant AttitudeDegree := (
		roll => FACE.DM.RollDegree(0),
		pitch => FACE.DM.PitchDegree(0),
		yaw => FACE.DM.HeadingDegree(0));

end FACE.DM;