
-------------------------------------------------------------------------------------
-- DISTRIBUTION STATEMENT A. Approved for public release; distribution is unlimited.
--                                                                                  
-- Product produced under DoD SENSIAC contract HC104705D4000 under the sponsorship  
-- of the Defense Technical Information Center,                                     
-- ATTN: DTIC-AI, 8723 John J. Kingman Rd., Ste 0944, Fort Belvoir, VA 22060-6218.  
-- SENSIAC is a DoD Information Analysis Center Sponsored by the Defense Technical  
-- Information Center.                                                              
--                                                                                  
-- HANDLING AND DESTRUCTION NOTICE - Comply with distribution statement and destroy 
-- by any method that will prevent disclosure of the contents or reconstruction of  
-- the document.                                                                    
--                                                                                  
-- Copyright (c) Vanderbilt University, 2016                                        
--                                                                                  
-- ALL RIGHTS RESERVED, UNLESS OTHERWISE STATED                                     
--                                                                                  
-- Developed under contract for the U.S. Department of Defense                      
-- by the Institute for Software Integrated Systems (ISIS), Vanderbilt University   
--                                                                                  
-- The FACE Consortium has approved results from this release for                   
-- certifying software components as FACE Conformant                                
--                                                                                  
-- Vanderbilt University disclaims all warranties with regard to this software,     
-- including all implied warranties of merchantability and fitness.  In no event    
-- shall Vanderbilt University be liable for any special, indirect or               
-- consequential damages or any damages whatsoever resulting from loss of use,      
-- data or profits, whether in an action of contract, negligence or other tortious  
-- action, arising out of or in connection with the use or performance of this      
-- software.                                                                        
--                                                                                  
-- Filename: goldStandardLibraries/Ada/spec/FACE/face-hmfm.ads                      
--                                                                                  
-- Conformance Test Suite Version 2.1.0r5 supporting FACE Standard Version 2.1      
--                                                                                  
-- This file is generated and may be overwritten.                                   
--                                                                                  
-- Release Date: 16-Jan-2017                                                        
-------------------------------------------------------------------------------------
with FACE_Basic_Types;  use FACE_Basic_Types;


package FACE.HMFM is

    type FAULT_MESSAGE_SIZE_TYPE is new FACE_Basic_Types.Long;
    
    type FAULT_MESSAGE_ADDRESS_TYPE is new System.Address;
    
    type THREAD_ID_TYPE is new FACE_Basic_Types.Long;
    
    FAULT_MESSAGE_MAXIMUM_SIZE : constant FACE.HMFM.FAULT_MESSAGE_SIZE_TYPE := 128;
    
    type STACK_SIZE_TYPE is new FACE_Basic_Types.Unsigned_Long;
    
    subtype FAULT_MESSAGE_TYPE is Standard.String(1 .. Standard.Integer(FACE.HMFM.FAULT_MESSAGE_MAXIMUM_SIZE));
    
    type FAULT_CODE_TYPE is
        (
            DEADLINE_MISSED,
            APPLICATION_ERROR,
            NUMERIC_ERROR,
            ILLEGAL_REQUEST,
            STACK_OVERFLOW,
            MEMORY_VIOLATION,
            HARDWARE_FAULT,
            POWER_FAIL
        );
    for FAULT_CODE_TYPE'Size use FACE_Basic_Types.Enum_Size;
    pragma Convention (C, FAULT_CODE_TYPE);
    
    type FAULT_STATUS_TYPE is
        record
                          CODE : FACE.HMFM.FAULT_CODE_TYPE;
                       MESSAGE : FACE.HMFM.FAULT_MESSAGE_TYPE;
                        LENGTH : FACE.HMFM.FAULT_MESSAGE_SIZE_TYPE;
              FAILED_THREAD_ID : FACE.HMFM.THREAD_ID_TYPE;
                FAILED_ADDRESS : FACE.SYSTEM_ADDRESS_TYPE;
    
        end record;
    pragma Convention (C, FAULT_STATUS_TYPE);
    
    type FAULT_HANDLER_ENTRY_TYPE is access procedure
    ;
    pragma Convention(C, FAULT_HANDLER_ENTRY_TYPE);
    
    procedure Initialize
    (
        return_code :    out FACE.RETURN_CODE_TYPE
    );
    
    procedure Create_Fault_Handler
    (
        entry_point : in     FACE.HMFM.FAULT_HANDLER_ENTRY_TYPE;
        stack_size  : in     FACE.HMFM.STACK_SIZE_TYPE;
        return_code :    out FACE.RETURN_CODE_TYPE
    );
    
    procedure Report_Application_Message
    (
        fault       : in     FACE.HMFM.FAULT_MESSAGE_ADDRESS_TYPE;
        length      : in     FACE.HMFM.FAULT_MESSAGE_SIZE_TYPE;
        return_code :    out FACE.RETURN_CODE_TYPE
    );
    
    procedure Get_Fault_Status
    (
        fault       :    out FACE.HMFM.FAULT_STATUS_TYPE;
        return_code :    out FACE.RETURN_CODE_TYPE
    );
    
    procedure Raise_Application_Fault
    (
        code        : in     FACE.HMFM.FAULT_CODE_TYPE;
        message     : in     FACE.HMFM.FAULT_MESSAGE_ADDRESS_TYPE;
        length      : in     FACE.HMFM.FAULT_MESSAGE_SIZE_TYPE;
        return_code :    out FACE.RETURN_CODE_TYPE
    );
    

end FACE.HMFM;

