with Ada.Characters.Handling;

--------------------
-- FACE.Utilities --
--------------------
package body FACE.Utilities is

   -----------
   -- Check --
   -----------
   --
   -- This procedure simply returns when the Return_Code argument has the value No_Error;
   -- otherwise, a unique exception is raised that is tied to the specific Return_Code value.
   --
   -- Note:  Additional information could be included in a String, if this be deemed desirable.
   --
   function Check (Return_Code     : in FACE.Return_Code_Type;
                   Raise_Exception : in Boolean := False)
                   return FACE.Return_Code_Type
   is
   begin
      if Raise_Exception then
         case Return_Code is
            when FACE.No_Error               => null;
            when FACE.No_Action              => raise FACE_No_Action;
            when FACE.Not_Available          => raise FACE_Not_Available;
            when FACE.Addr_In_Use            => raise FACE_Addr_In_Use;
            when FACE.Invalid_Param          => raise FACE_Invalid_Param;
            when FACE.Invalid_Config         => raise FACE_Invalid_Config;
            when FACE.Permission_Denied      => raise FACE_Permission_Denied;
            when FACE.Invalid_Mode           => raise FACE_Invalid_Mode;
            when FACE.Timed_Out              => raise FACE_Timed_Out;
            when FACE.Message_Stale          => raise FACE_Message_Stale;
            when FACE.Connection_In_Progress => raise FACE_Connection_In_Progress;
            when FACE.Connection_Closed      => raise FACE_Connection_Closed;
            when FACE.Data_Buffer_Too_Small  => raise FACE_Data_Buffer_Too_Small;
         end case;
      end if;  -- Raise_Exception
      return Return_Code;  -- not raising an exception
   end Check;

   package Ach renames Ada.Characters.Handling;

   ---------
   -- "=" --
   ---------
   function "=" (L, R : User_Character) return Standard.Boolean is
   begin
      return ACH.To_Upper (Character (L)) = ACH.To_Upper (Character (R));
   end "=";

   --------------------
   -- User_Char_Hash --
   --------------------
   function User_Char_Hash (C : User_Character) return Natural is
   begin
      return Character'Pos (ACH.To_Upper (Character (C)));
   end User_Char_Hash;

   -----------
   -- Usage --
   -----------

   protected body Usage
   is

      ---------------
      -- Increment --
      ---------------

      procedure Increment
      is
         use Interfaces;
      begin
         Count := Count + 1;
      end Increment;

      ---------------
      -- Decrement --
      ---------------

      procedure Decrement (Is_Zero : out Boolean)
      is
          use Interfaces;
     begin
         Count   := Count - 1;
         Is_Zero := Boolean (Count = 0);
      end Decrement;

      -------------
      -- Current --
      -------------

      function Current return Usage_Type
      is
      begin
         return Count;
      end Current;

   end Usage;

end FACE.Utilities;
