// Product produced under DoD SENSIAC contract HC104705D4000 under
// the sponsorship of the Defense Technical Information Center,
// ATTN: DTIC-AI, 8723 John J. Kingman Rd., Ste 0944, Fort Belvoir, VA
// 22060-6218.  SENSIAC is a DoD Information Analysis Center Sponsored
// by the Defense Technical Information Center.
//
// HANDLING AND DESTRUCTION NOTICE - Comply with distribution statement
// and destroy by any method that will prevent disclosure of
// contents or reconstruction of the document.

#ifndef FACE_DM_InertialMeasurementUnit_UnboundedSequence_defined
#define FACE_DM_InertialMeasurementUnit_UnboundedSequence_defined
#include <stdlib.h>
#include <string.h>
#include <FACE/types.hpp>
#include "InertialMeasurementUnit.hpp"
namespace FACE
{
    namespace DM
    {
        class InertialMeasurementUnit_UnboundedSequence
        {
            //Unbounded sequence
            public:
                InertialMeasurementUnit_UnboundedSequence()
                {
                    myLength = 0;
                    myBuffer = new InertialMeasurementUnit[1];
                    myMaximum = 0;
                    myRelease = true;
                }

                InertialMeasurementUnit_UnboundedSequence(FACE::UnsignedLong max,FACE::UnsignedLong length, InertialMeasurementUnit *data, FACE::Boolean release = false)
                {
                    myMaximum = max;
                    myLength = length;
                    myBuffer = new InertialMeasurementUnit[length];
                    myRelease = release;
                    if(data != NULL)
                    {
                        memcpy(myBuffer, data, myLength*sizeof(InertialMeasurementUnit));
                    }
                }

                InertialMeasurementUnit_UnboundedSequence(const InertialMeasurementUnit_UnboundedSequence& aSequence)
                {
                    myMaximum = aSequence.maximum();
                    myLength = aSequence.length();
                    if(myLength > 0)
                    {
                        myBuffer = new InertialMeasurementUnit[myLength];
                        memcpy(myBuffer,aSequence.get_buffer(),myLength*sizeof(InertialMeasurementUnit));
                    }
                    else
                    {
                        myLength = 0;
                        myBuffer = new InertialMeasurementUnit[1];
                    }
                }

                ~InertialMeasurementUnit_UnboundedSequence()
                {
                    if(myRelease == true && myBuffer != NULL)
                    {
                        delete[] myBuffer;
                    }
                }

                InertialMeasurementUnit_UnboundedSequence& operator=(const InertialMeasurementUnit_UnboundedSequence& aSequence)
                {
                    myMaximum = aSequence.maximum();-1;
                    myLength = aSequence.length();
                    if(myLength > 0)
                    {
                        myBuffer = new InertialMeasurementUnit[myLength];
                        memcpy(myBuffer,aSequence.get_buffer(),myLength*sizeof(InertialMeasurementUnit));
                    }
                    else
                    {
                        myLength = 0;
                        myBuffer = new InertialMeasurementUnit[1];
                    }
                    return (*this);
                }

                FACE::UnsignedLong maximum() const
                {
                    return (myMaximum);
                }

                void length(FACE::UnsignedLong length)
                {
                    // if requested length is greater than bound, do nothing
                    if(length > myMaximum || length < 0)
                        return;

                    InertialMeasurementUnit* tempBuffer = myBuffer;
                    FACE::UnsignedLong tempLength = myLength;
                    if(length > myMaximum)
                    {
                        myMaximum = length;
                    }
                    myLength = length;
                    if(length == 0 || length == 1)
                    {
                           myBuffer = new InertialMeasurementUnit[1];
                    }
                    else if(tempLength >= length)
                    {
                        myBuffer = new InertialMeasurementUnit[length];
                        memcpy(myBuffer,tempBuffer,length*sizeof(InertialMeasurementUnit));
                        delete[] tempBuffer;
                    }
                    else
                    {
                        myBuffer = new InertialMeasurementUnit[length];
                        memcpy(myBuffer,tempBuffer,tempLength*sizeof(InertialMeasurementUnit));
                        delete[] tempBuffer;
                    }
                }

                FACE::UnsignedLong length() const
                {
                    return (myLength);
                }

                InertialMeasurementUnit &operator[](FACE::UnsignedLong index)
                {
                    return (myBuffer[index]);
                }

                const InertialMeasurementUnit &operator[](FACE::UnsignedLong index) const
                {
                    return (myBuffer[index]);
                }
                FACE::Boolean release() const
                {
                    return (myRelease);
                }

                void replace(FACE::UnsignedLong max,FACE::UnsignedLong length, InertialMeasurementUnit *data,FACE::Boolean release = false)
                {
                    myMaximum = max;
                    myLength = length;
                    myBuffer = new InertialMeasurementUnit[length];
                    myRelease = release;
                    if(data != NULL)
                    {
                        memcpy(myBuffer, data, myLength*sizeof(InertialMeasurementUnit));
                    }
                }

                InertialMeasurementUnit* get_buffer(FACE::Boolean orphan = false)
                {
                    if(orphan == true)
                    {
                        InertialMeasurementUnit * tempBuffer = new InertialMeasurementUnit[myLength];
                        memcpy(tempBuffer, myBuffer, myLength);
                        delete[] myBuffer;
                        return (tempBuffer);
                    }
                    else
                    {
                        return (myBuffer);
                    }
                }

                const InertialMeasurementUnit* get_buffer() const
                {
                    return (myBuffer);
                }

                private:
                    InertialMeasurementUnit * myBuffer;
                    FACE::UnsignedLong myLength;
                    FACE::UnsignedLong myMaximum;
                    FACE::Boolean myRelease;
            };
    }
}
#endif /* FACE_DM_InertialMeasurementUnit_UnboundedSequence_defined */ 
#ifndef FACE_DM_EGI1_UnboundedSequence_defined
#define FACE_DM_EGI1_UnboundedSequence_defined
#include <stdlib.h>
#include <string.h>
#include <FACE/types.hpp>
#include "EGI1.hpp"
namespace FACE
{
    namespace DM
    {
        class EGI1_UnboundedSequence
        {
            //Unbounded sequence
            public:
                EGI1_UnboundedSequence()
                {
                    myLength = 0;
                    myBuffer = new EGI1[1];
                    myMaximum = 0;
                    myRelease = true;
                }

                EGI1_UnboundedSequence(FACE::UnsignedLong max,FACE::UnsignedLong length, EGI1 *data, FACE::Boolean release = false)
                {
                    myMaximum = max;
                    myLength = length;
                    myBuffer = new EGI1[length];
                    myRelease = release;
                    if(data != NULL)
                    {
                        memcpy(myBuffer, data, myLength*sizeof(EGI1));
                    }
                }

                EGI1_UnboundedSequence(const EGI1_UnboundedSequence& aSequence)
                {
                    myMaximum = aSequence.maximum();
                    myLength = aSequence.length();
                    if(myLength > 0)
                    {
                        myBuffer = new EGI1[myLength];
                        memcpy(myBuffer,aSequence.get_buffer(),myLength*sizeof(EGI1));
                    }
                    else
                    {
                        myLength = 0;
                        myBuffer = new EGI1[1];
                    }
                }

                ~EGI1_UnboundedSequence()
                {
                    if(myRelease == true && myBuffer != NULL)
                    {
                        delete[] myBuffer;
                    }
                }

                EGI1_UnboundedSequence& operator=(const EGI1_UnboundedSequence& aSequence)
                {
                    myMaximum = aSequence.maximum();-1;
                    myLength = aSequence.length();
                    if(myLength > 0)
                    {
                        myBuffer = new EGI1[myLength];
                        memcpy(myBuffer,aSequence.get_buffer(),myLength*sizeof(EGI1));
                    }
                    else
                    {
                        myLength = 0;
                        myBuffer = new EGI1[1];
                    }
                    return (*this);
                }

                FACE::UnsignedLong maximum() const
                {
                    return (myMaximum);
                }

                void length(FACE::UnsignedLong length)
                {
                    // if requested length is greater than bound, do nothing
                    if(length > myMaximum || length < 0)
                        return;

                    EGI1* tempBuffer = myBuffer;
                    FACE::UnsignedLong tempLength = myLength;
                    if(length > myMaximum)
                    {
                        myMaximum = length;
                    }
                    myLength = length;
                    if(length == 0 || length == 1)
                    {
                           myBuffer = new EGI1[1];
                    }
                    else if(tempLength >= length)
                    {
                        myBuffer = new EGI1[length];
                        memcpy(myBuffer,tempBuffer,length*sizeof(EGI1));
                        delete[] tempBuffer;
                    }
                    else
                    {
                        myBuffer = new EGI1[length];
                        memcpy(myBuffer,tempBuffer,tempLength*sizeof(EGI1));
                        delete[] tempBuffer;
                    }
                }

                FACE::UnsignedLong length() const
                {
                    return (myLength);
                }

                EGI1 &operator[](FACE::UnsignedLong index)
                {
                    return (myBuffer[index]);
                }

                const EGI1 &operator[](FACE::UnsignedLong index) const
                {
                    return (myBuffer[index]);
                }
                FACE::Boolean release() const
                {
                    return (myRelease);
                }

                void replace(FACE::UnsignedLong max,FACE::UnsignedLong length, EGI1 *data,FACE::Boolean release = false)
                {
                    myMaximum = max;
                    myLength = length;
                    myBuffer = new EGI1[length];
                    myRelease = release;
                    if(data != NULL)
                    {
                        memcpy(myBuffer, data, myLength*sizeof(EGI1));
                    }
                }

                EGI1* get_buffer(FACE::Boolean orphan = false)
                {
                    if(orphan == true)
                    {
                        EGI1 * tempBuffer = new EGI1[myLength];
                        memcpy(tempBuffer, myBuffer, myLength);
                        delete[] myBuffer;
                        return (tempBuffer);
                    }
                    else
                    {
                        return (myBuffer);
                    }
                }

                const EGI1* get_buffer() const
                {
                    return (myBuffer);
                }

                private:
                    EGI1 * myBuffer;
                    FACE::UnsignedLong myLength;
                    FACE::UnsignedLong myMaximum;
                    FACE::Boolean myRelease;
            };
    }
}
#endif /* FACE_DM_EGI1_UnboundedSequence_defined */ 


#ifndef _NAVMANAGEMENTFUNCTION_HPP
#define _NAVMANAGEMENTFUNCTION_HPP

#include "AttitudeBodyFrame.hpp"
#include "Position.hpp"

namespace FACE
{
    namespace DM
    {
        struct NavManagementFunction
        {
            FACE::DM::AttitudeBodyFrame attitude;
            FACE::DM::Position position;
            FACE::DM::InertialMeasurementUnit_UnboundedSequence imuNavSource;
            FACE::DM::EGI1_UnboundedSequence egiNavSource;

        };
    }
}

#endif	/* _NAVMANAGEMENTFUNCTION_HPP*/	