// Product produced under Contract No. FA8075-14-D-0014 and the U.S. Government
// has unlimited rights in this software.
//
// Vanderbilt University disclaims all warranties with regard to this
// software, including all implied warranties of merchantability and fitness.
// In no event shall Vanderbilt University be liable for any special,
// indirect or consequential damages or any damages whatsoever resulting
// from loss of use, data or profits, whether in an action of contract,
// negligence or other tortious action, arising out of or in connection
// with the use or performance of this software.
//
// May contain products produced under DoD SENSIAC contract HC104705D4000 under
// the sponsorship of the Defense Technical Information Center,
// ATTN: DTIC-AI, 8723 John J. Kingman Rd., Ste 0944, Fort Belvoir, VA
// 22060-6218.  SENSIAC is a DoD Information Analysis Center Sponsored
// by the Defense Technical Information Center.

/*
   This file contains type definitions for types given in 
     Tables 141, 143, and 145 of the FACE 2.1 Standard.
   Relevant information from these tables is replicated below:
 
   C++ Type:                  Size (bytes):    Range of Values:
   --------                   ------------     ---------------
   FACE::Boolean              1                0 to (2^1 - 1)
   FACE::Char                 1                -2^7 to (2^7 - 1)
   FACE::WChar                unspecified      unknown
   FACE::Octet                1                0 to (2^8 - 1)
   FACE::Double               8                IEEE double precision floating point
   FACE::LongDouble           10               IEEE extended double precision floating point
   FACE::Float                4                IEEE single precision floating point
   FACE::Short                2                -2^15 to (2^15 - 1)
   FACE::Long                 4                -2^31 to (2^31 - 1)
   FACE::LongLong             8                -2^63 to (2^63 - 1)
   FACE::UnsignedShort        2                0 to (2^16 - 1)
   FACE::UnsignedLong         4                0 to (2^32 - 1)
   FACE::UnsignedLongLong     8                0 to (2^64 - 1)
   
   Default definitions for these types are provided below,
     but they should be edited for your particular system.
*/

/* types.hpp is a file should be distributed by your Operating System.
 * However, for the conformance test, we are using the values
 * from the CONFORMANCE_TEST_FACE_EXACT_TYPES.h file to define
 * the FACE C++ Data Types.
 * CONFORMANCE_TEST_FACE_EXACT_TYPES.h is modified by the tester to
 * be correct for the O/S under test.  Please see the manual for more information
 */


#ifndef _FACE_TYPES_HPP_
#define _FACE_TYPES_HPP_

#include "CONFORMANCE_TEST_FACE_EXACT_TYPES.h"

namespace FACE
{
	// These are defined exactly by C and C++ standards 
	typedef bool                Boolean;
	typedef wchar_t             WChar;
	typedef double              Double;
	typedef long double         LongDouble;
	typedef float               Float;

	// These are defined by CONFORMANCE_TEST_FACE_EXACT_TYPES.h 
	typedef FACE_int8_t         Char;
	typedef FACE_uint8_t        Octet;
	typedef FACE_int16_t        Short;
	typedef FACE_int32_t        Long;
	typedef FACE_int64_t        LongLong;
	typedef FACE_uint16_t       UnsignedShort;
	typedef FACE_uint32_t       UnsignedLong;
	typedef FACE_uint64_t       UnsignedLongLong;
}

#endif /* _FACE_TYPES_HPP_ */
