#!/usr/bin/env python

import struct
import sys

J2SE_8  = 52
J2SE_7  = 51
J2SE_6  = 50
J2SE_5  = 49
JDK_1_4 = 48
JDK_1_3 = 47
JDK_1_2 = 46
JDK_1_1 = 45

def versionToString(ver):
  if ver == J2SE_8:
    return "8"
  if ver == J2SE_7:
    return "7"
  if ver == J2SE_6:
    return "6"
  if ver == J2SE_5:
    return "5"
  if ver == JDK_1_4:
    return "1.4"
  if ver == JDK_1_3:
    return "1.3"
  if ver == JDK_1_2:
    return "1.2"
  if ver == JDK_1_1:
    return "1.1"
  return "INVALID"

def getVersionCode(classfile):
  if len(classfile) < 8:
    print >> sys.stderr, "Not a class file"
    sys.exit(1)

  return struct.unpack(">IHH", classfile[0:8])[2]

def main():
  if len(sys.argv) != 2:
    print >> sys.stderr, "Usage: {0} CLASSFILE".format(sys.argv[0])
    sys.exit(1)

  classfilename = sys.argv[1]
  classfile = None

  print "Checking java compiler's output for version {0}. (MUST be version {0}.)".format(versionToString(J2SE_8)).format(versionToString(J2SE_8))

  try:
    with open(classfilename) as f:
      classfile = f.read()
  except IOError as e:
    print >> sys.stderr, "I/O Error: {0}".format(e.strerror)
    sys.exit(1)

  version = getVersionCode(classfile)
  print "Found version {0}".format(versionToString(version))
  if version > J2SE_8:
    sys.exit(1)

if __name__ == "__main__":
  main()

