////////////////////////////////////////////////////////////////////////////////
// DISTRIBUTION STATEMENT A. Approved for public release; distribution is
// unlimited.
//
// Product produced under DoD SENSIAC contract HC104705D4000 under the
// sponsorship of the Defense Technical Information Center, ATTN: DTIC-AI, 8723
// John J. Kingman Rd., Ste 0944, Fort Belvoir, VA 22060-6218.  SENSIAC is a
// DoD Information Analysis Center Sponsored by the Defense Technical
// Information Center.
//
// HANDLING AND DESTRUCTION NOTICE - Comply with distribution statement and
// destroy by any method that will prevent disclosure of the contents or
// reconstruction of the document.
//
// Future Airborne Capability Environment (FACE) Reference Architecture,
// copyright 2012 The Open Group. FACE is a trademark of The Open Group in the
// United States and other countries.
//
// Developed under contract for the U.S. Department of Defense by the Institute
// for Software Integrated Systems (ISIS), Vanderbilt University
//
// Vanderbilt University disclaims all warranties with regard to this software,
// including all implied warranties of merchantability and fitness.  In no
// event shall Vanderbilt University be liable for any special, indirect or
// consequential damages or any damages whatsoever resulting from loss of
// use,data or profits, whether in an action of contract, negligence or other
// tortious action, arising out of or in connection with the use or performance
// of this software.
//
// Supported FACE Technical Standard: 2.1.0
//
// FACE Conformance Test Suite Release: 2.1.0r5
//
// The FACE Consortium will currently accept any results from this release
// for certifying software components as FACE Conformant
//
// Copyright 2017 Vanderbilt University
//
//
// Filename: CompilerSpecific.c
//
// Created: 2016-05-18T13:32:24.954000
//
////////////////////////////////////////////////////////////////////////////////
/*
 * Place any required function stubs for the compiler/linker used for test
 *  i.e. __main(),_start, etc...
 *
 * Be careful to honor profiles. 
 */


/*
 *   Profile specific MACROs
 */
#if !defined (FACE_GENERAL_PURPOSE_PROFILE) && !defined(FACE_SAFETY_EXTENDED_PROFILE) && !defined(FACE_SAFETY_BASE_PROFILE) && !defined(FACE_SECURITY_PROFILE)
#error "No FACE profile defined"
#endif


/*
 * These are known symbols for GCC.
 */
#if defined(__GNUC__)

/*
*   Function stub for _start() to compensate for -nodefaultlibs -nostartfiles. 
*   Without the stub, linker gives "warning: cannot find entry symbol _start; 
*   defaulting to 0000000000400144"
*/
void _start() {}

/*
 * This is invoked by GCC when stack checking is enabled.
 */
void __stack_chk_fail(void)
{
}


 #if defined(__CYGWIN__)
    /*
     *  This symbol is implicitly generated by GCC on some platforms.
     *  It initializes the run-time and may invoke global constructors.
     */
    int __main(int argc, const char **argv) {}

 #endif  /* __CYGWIN__ */




/*
 * Integer Arithmetic Functions in GCC low-level runtime library that a target processor cannot perform directly.
 * c99 - always allows long long
 */
#if defined(ALLOWABLE_USE_OF_LONG_LONG)
// GCC Internals - 4.1.1 Arithmetic Functions
void __lshrdi3() { }
void __ashrdi3() { }
void __ashldi3() { }
void __muldi3() { }
void __divdi3() { }
void __moddi3() { }
void __udivdi3() { }
void __umoddi3() { }

// GCC Internals - 4.1.3 Trapping Arithmetic Functions
void __addvdi3() { }
void __subvdi3() { }
void __mulvdi3() { }
void __negdi2() { }
void __udivmoddi4() { }
#endif /*ALLOWABLE_USE_OF_LONG_LONG*/


#else
  #error "Compiler is unknown. Please add Compiler Specific support."
#endif  /* __GNUC__ */

