////////////////////////////////////////////////////////////////////////////////
// DISTRIBUTION STATEMENT A. Approved for public release; distribution is
// unlimited.
//
// Product produced under DoD SENSIAC contract HC104705D4000 under the
// sponsorship of the Defense Technical Information Center, ATTN: DTIC-AI, 8723
// John J. Kingman Rd., Ste 0944, Fort Belvoir, VA 22060-6218.  SENSIAC is a
// DoD Information Analysis Center Sponsored by the Defense Technical
// Information Center.
//
// HANDLING AND DESTRUCTION NOTICE - Comply with distribution statement and
// destroy by any method that will prevent disclosure of the contents or
// reconstruction of the document.
//
// Future Airborne Capability Environment (FACE) Reference Architecture,
// copyright 2012 The Open Group. FACE is a trademark of The Open Group in the
// United States and other countries.
//
// Developed under contract for the U.S. Department of Defense by the Institute
// for Software Integrated Systems (ISIS), Vanderbilt University
//
// Vanderbilt University disclaims all warranties with regard to this software,
// including all implied warranties of merchantability and fitness.  In no
// event shall Vanderbilt University be liable for any special, indirect or
// consequential damages or any damages whatsoever resulting from loss of
// use,data or profits, whether in an action of contract, negligence or other
// tortious action, arising out of or in connection with the use or performance
// of this software.
//
// Supported FACE Technical Standard: 2.1.0
//
// FACE Conformance Test Suite Release: 2.1.0r5
//
// The FACE Consortium will currently accept any results from this release
// for certifying software components as FACE Conformant
//
// Copyright 2017 Vanderbilt University
//
//
// Filename: CompilerSpecific.cpp
//
// Created: 2016-05-18T13:32:24.954000
//
////////////////////////////////////////////////////////////////////////////////



/*
 * Place any required function stubs for the compiler/linker used for test
 *  i.e. __main(),_start, etc...
 *
 * Be careful to honor profiles.
 */



/*
 *   Profile specific MACROs
 */
#if !defined (FACE_GENERAL_PURPOSE_PROFILE) && !defined(FACE_SAFETY_EXTENDED_PROFILE) && !defined(FACE_SAFETY_BASE_PROFILE) && !defined(FACE_SECURITY_PROFILE)
#error "No FACE profile defined"
#endif



/*
 * These are known symbols for GCC.
 */
#if defined(__GNUC__)


/*
*   Function stub for _start() to compensate for -nodefaultlibs -nostartfiles. 
*   Without the stub, linker gives "warning: cannot find entry symbol _start; 
*   defaulting to 0000000000400144"
*/
extern "C" {
	void _start() {}
}

/*
 * FACE Security and Safety Profiles place restrictions on C++ source code.
 * Exceptions and RTTI may NOT be used. The following symbols are generated
 * in support of exceptions and RTTI. Conformant General Purpose code may
 * reference them. Any compiled object claiming Security or Safety
 * Profile conformance should NEVER reference these symbols.
 *
 * http://wiki.osdev.org/C++ is a good reference along with the GCC source
 * code and documentation.
 */
#if defined(FACE_GENERAL_PURPOSE_PROFILE) || defined(FACE_SAFETY_EXTENDED_PROFILE)
extern "C" {

	/*
	 * Variable indicates the type of exception and methods are implicitly invoked by GCC in support of the exception.
	 * https://stackoverflow.com/questions/17967597/obtaining-current-gcc-exception-model
	*/
	#if defined (__USING_SJLJ_EXCEPTIONS__)
		 int __gxx_personality_sj0;
		 int _Unwind_SjLj_Register(void) {}
		 int _Unwind_SjLj_Unregister(void) {}
		 int _Unwind_SjLj_Resume(void) {}
	#elif defined(__CYGWIN__) || defined(__MINGW32__) || defined(__MINGW64__)
		int __gxx_personality_seh0;
	  void _Unwind_Resume(void) {}  

	#else
		int __gxx_personality_v0;
	  void _Unwind_Resume(void) {}
	#endif


  /*
   *  These support global C++ objects including execution of destructors
   *  as a side-effect of calling exit().
   */
  void *__dso_handle;

  int __cxa_atexit(void (*destructor) (void *), void *arg, void *__dso_handle)
  {
    return 0;
  }

  void __cxa_finalize(void *f);
  

  /*
   * These are part of the ARM EABI and references to these may be generated
   * when compiling C++ source written to any FACE Profile targeting the ARM
   * architecture.
   *
   * Reference:
   * http://infocenter.arm.com/help/topic/com.arm.doc.ihi0041d/IHI0041D_cppabi.pdf
   */
   #if defined(__arm__)
	int __aeabi_unwind_cpp_pr1;
	int __aeabi_atexit;
   #endif
}
#endif

/*
 * These may be generated by GCC when compiling conformant source code
 * written to any FACE Profile targeting any architecture.
 * Only invoked when pure virtual methods can't be resolved.
 * 
 * NOTE: Usage is unavoidable due to FACE interfaces definitions' usage of pure virtual functions.
 */
#if defined(FACE_GENERAL_PURPOSE_PROFILE) || defined(FACE_SAFETY_EXTENDED_PROFILE) || defined(FACE_SAFETY_BASE_PROFILE) || defined(FACE_SECURITY_PROFILE)
int __cxa_pure_virtual;
#endif
#if defined(FACE_GENERAL_PURPOSE_PROFILE) || defined(FACE_SAFETY_EXTENDED_PROFILE)
int __cxa_deleted_virtual;
#endif


namespace __cxxabiv1 {
    class __class_type_info {
    public:
      virtual ~__class_type_info() { }
      void operator delete(void *var0) { }	
    };
    __class_type_info __my_class_type_info;

    class __si_class_type_info : public __class_type_info {
    public:
      virtual ~__si_class_type_info() { }
    };
    __si_class_type_info __my_si_class_type_info;

    class __vmi_class_type_info : public __class_type_info {
    public:
      ~__vmi_class_type_info () { }
    };
    __vmi_class_type_info __my__vmi_class_type_info;
}

extern "C" {
  /*
   * This is invoked by GCC when stack checking is enabled.
   */
  void __stack_chk_fail(void) {}

  #if defined(__PPC__)
    /*
     * This symbol is part of the Embedded Application Programming Interface
     * (EABI) and assumed to exist.
     */
    void __eabi(void) {}
  #endif

  #if defined(__CYGWIN__) || defined(__MINGW32__) || defined(__MINGW64__)
    /*
     *  This symbol is implicitly generated by GCC on some platforms.
     *  It initializes the run-time and may invoke global constructors.
     */
    int __main(int argc, const char **argv) {}

    /*
     *  Cygwin wraps the operators new, delete, etc.  We are only adding
     *  to this list as we run into undefined errors.  The full list is
     *  contained in the thread at:
     *  https://cygwin.com/ml/cygwin-developers/2009-06/msg00007.html
     *
     *  HINT: Echo the "_XXX" part and pipe it into c++filt to learn the
     *  C++ name.
     */
    void __wrap_ZdlPv(void) {}
    void _ZdlPv(void) {}
    int atexit (void (*func)(void)) {}

  #endif
} /* end of extern "C" */


/*
 * Integer Arithmetic Functions in GCC low-level runtime library that a target processor cannot perform directly.
 * NOTE: If it is C++03 and uses either long long or %ll in printf, it is NON-CONFORMANT.
 */
#if defined(ALLOWABLE_USE_OF_LONG_LONG)
extern "C" {
  // GCC Internals - 4.1.1 Arithmetic Functions
  void __lshrdi3() { }
  void __ashrdi3() { }
  void __ashldi3() { }
  void __muldi3() { }
  void __divdi3() { }
  void __moddi3() { }
  void __udivdi3() { }
  void __umoddi3() { }

  // GCC Internals - 4.1.3 Trapping Arithmetic Functions
  void __addvdi3() { }
  void __subvdi3() { }
  void __mulvdi3() { }
  void __negdi2() { }
  void __udivmoddi4() { }
} /* end of extern "C" */

#endif /*ALLOWABLE_USE_OF_LONG_LONG*/


#else
  #error "Compiler is unknown. Please add Compiler Specific support."
#endif

