/*
DISTRIBUTION STATEMENT D. Distribution authorized to the Department of Defense and U.S. DoD contractors only.

Product produced under DoD SENSIAC contract HC104705D4000 under the sponsorship of the Defense Technical Information Center, 
ATTN: DTIC-AI, 8723 John J. Kingman Rd., Ste 0944, Fort Belvoir, VA 22060-6218.	
SENSIAC is a DoD Information Analysis Center Sponsored by the Defense Technical Information Center.

HANDLING AND DESTRUCTION NOTICE - Comply with distribution statement and destroy by any method that will prevent disclosure 
of the contents or reconstruction of the document.

Copyright (c) Vanderbilt University, 2013

ALL RIGHTS RESERVED, UNLESS OTHERWISE STATED

Developed under contract for the U.S. Department of Defense
by the Institute for Software Integrated Systems (ISIS), Vanderbilt University

Vanderbilt University disclaims all warranties with regard to this software, 
including all implied warranties of merchantability and fitness.	In no event 
shall Vanderbilt University be liable for any special, indirect or 
consequential damages or any damages whatsoever resulting from loss of use, 
data or profits, whether in an action of contract, negligence or other tortious
action, arising out of or in connection with the use or performance of this 
software.
 */
package edu.vanderbilt.isis.face.v21.datamodel.common.ldmgen;

import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.Status;
import org.eclipse.emf.common.notify.Notifier;
import org.eclipse.emf.common.util.TreeIterator;
import org.eclipse.emf.ecore.resource.Resource;
import org.eclipse.emf.ecore.resource.ResourceSet;
import org.eclipse.m2m.qvt.oml.util.Log;

import edu.vanderbilt.isis.face.v21.datamodel.common.Activator;
import face.v21.face_ext.Face_extFactory;

@SuppressWarnings("restriction")
public class FACEMapGenerator {
	
	public static IStatus createInitialMap(Resource faceResource, Resource faceExtResource) {
		
		face.v21.face_ext.Face_extFactory ff = Face_extFactory.eINSTANCE;
		face.v21.face_ext.FACEMap faceMap = ff.createFACEMap();
		
		TreeIterator<Notifier> allContents = faceResource.getResourceSet().getAllContents();
		while (allContents.hasNext()) {
			Object next = allContents.next();
			
			if (next instanceof face.v21.face.LogicalDataModel ||
				next instanceof face.v21.face.PlatformDataModel ||
				next instanceof face.v21.face.conceptual.Entity ||
				next instanceof face.v21.face.conceptual.View)
			{
				allContents.prune();
				continue;
			}
			if (next instanceof face.v21.face.conceptual.Observable)
			{
				face.v21.face.conceptual.Observable observable = (face.v21.face.conceptual.Observable) next;

				face.v21.face_ext.MapEntry mapEntry = ff.createMapEntry();
				mapEntry.setObservable(observable);
				mapEntry.setMeasurement(getMeasurement(faceResource.getResourceSet(), observable));
				faceMap.getMapEntry().add(mapEntry);
			}
		}
		faceExtResource.getContents().add(faceMap);
		
		return Status.OK_STATUS;
	}
	
	private static face.v21.face.logical.Measurement getMeasurement(ResourceSet resourceSet, face.v21.face.conceptual.Observable observable)
	{
		TreeIterator<Notifier> allContents = resourceSet.getAllContents();
		while (allContents.hasNext()) {
			Object next = allContents.next();
			
			if (next instanceof face.v21.face.ConceptualDataModel ||
				next instanceof face.v21.face.PlatformDataModel ||
				next instanceof face.v21.face.logical.Entity ||
				next instanceof face.v21.face.logical.View ||
				next instanceof face.v21.face.logical.Conversion) 
			{
				allContents.prune();
				continue;
			}
			if (next instanceof face.v21.face.logical.Measurement)
			{
				face.v21.face.logical.Measurement measurement = (face.v21.face.logical.Measurement) next;
            	face.v21.face.conceptual.Observable realizedObs = measurement.getRealizes();
				if(realizedObs != null && realizedObs.equals(observable))
				{
					return measurement;
				}
			}
		}
		return null;
}
	
	private static Log getLogger() {
		
		return new Log() {
			
			public void log(int level, String message) {
				Activator.getDefault().getLog().log(new Status(level, Activator.PLUGIN_ID, message));
			}
			
			public void log(int level, String message, Object param) {
				log(level, message);
			}
			
			public void log(String message) {
				log(IStatus.INFO, message);
			}
			
			public void log(String message, Object param) {
				log(message);
			}
			
		};
	}
}
