/*
DISTRIBUTION STATEMENT D. Distribution authorized to the Department of Defense and U.S. DoD contractors only.

Product produced under DoD SENSIAC contract HC104705D4000 under the sponsorship of the Defense Technical Information Center, 
ATTN: DTIC-AI, 8723 John J. Kingman Rd., Ste 0944, Fort Belvoir, VA 22060-6218.  
SENSIAC is a DoD Information Analysis Center Sponsored by the Defense Technical Information Center.

HANDLING AND DESTRUCTION NOTICE - Comply with distribution statement and destroy by any method that will prevent disclosure 
of the contents or reconstruction of the document.

Copyright (c) Vanderbilt University, 2013

ALL RIGHTS RESERVED, UNLESS OTHERWISE STATED

Developed under contract for the U.S. Department of Defense
by the Institute for Software Integrated Systems (ISIS), Vanderbilt University

Vanderbilt University disclaims all warranties with regard to this software, 
including all implied warranties of merchantability and fitness.  In no event 
shall Vanderbilt University be liable for any special, indirect or 
consequential damages or any damages whatsoever resulting from loss of use, 
data or profits, whether in an action of contract, negligence or other tortious
action, arising out of or in connection with the use or performance of this 
software.
 */
package edu.vanderbilt.isis.face.v21.datamodel.common.ldmgen;

import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.Status;
import org.eclipse.emf.common.util.BasicDiagnostic;
import org.eclipse.emf.common.util.Diagnostic;
import org.eclipse.emf.common.util.URI;
import org.eclipse.emf.ecore.resource.Resource;
import org.eclipse.m2m.internal.qvt.oml.InternalTransformationExecutor;
import org.eclipse.m2m.internal.qvt.oml.trace.Trace;
import org.eclipse.m2m.qvt.oml.BasicModelExtent;
import org.eclipse.m2m.qvt.oml.ExecutionContextImpl;
import org.eclipse.m2m.qvt.oml.ExecutionDiagnostic;
import org.eclipse.m2m.qvt.oml.ModelExtent;
import org.eclipse.m2m.qvt.oml.TransformationExecutor;
import org.eclipse.m2m.qvt.oml.util.Log;

import edu.vanderbilt.isis.face.v21.datamodel.common.Activator;


@SuppressWarnings("restriction")
public class PlatformDataModelGenerator {
	
	public static IStatus generate(Resource faceResource) {
		
		URI transformationURI = URI.createURI("platform:/plugin/edu.vanderbilt.isis.face.v21.datamodel.common/src/edu/vanderbilt/isis/face/v21/datamodel/common/ldmgen/resource/ldm_to_pdm.qvto", true);
		
		ModelExtent inoutFace = new BasicModelExtent(faceResource.getContents());

		ExecutionContextImpl context = new ExecutionContextImpl();
		context.setLog(getLogger());
		context.setConfigProperty("keepModeling", true);
		TransformationExecutor executor = new TransformationExecutor(transformationURI);
		ExecutionDiagnostic result = executor.execute(context, inoutFace);
		
		return BasicDiagnostic.toIStatus(result);
	}

	private static Log getLogger() {
		
		return new Log() {
			
			public void log(int level, String message) {
				Activator.getDefault().getLog().log(new Status(level, Activator.PLUGIN_ID, message));
			}
			
			public void log(int level, String message, Object param) {
				log(level, message);
			}
			
			public void log(String message) {
				log(IStatus.INFO, message);
			}
			
			public void log(String message, Object param) {
				log(message);
			}
			
		};
	}
}
